 
/* Qsweep_spin.h ******************
*  Part of QPeace software
*  See README file in QPeace distribution for copyright information
*
*  SJ Schwartz   (s.j.schwartz@qmw.ac.uk)
*  Queen Mary, University of London
*
*  May 2001 Re-organised Qpeace software and created this module from others
*  May 2002 Adding pitch angle capabilities
*           Split off Qsweep_export and Qsweep_spin_read modules, but all
*              are covered by this single header
*  June 2002 Changed PEACE_SWEEP angles to be FLOW directions
*  Sept 2004 Added sun sensor and other bits to SWEEP struture to cope
*              with flakier data; incl s/c potential, ...
*  Oct  2004 Added Qphisc and related bits
*/

#ifndef _QSWEEP_SPIN
#define _QSWEEP_SPIN

#ifdef __cplusplus
extern "C"{
#endif

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <limits.h>
#include "ret_codes.h"
#include "user_defs.h"
#include "libVIDF.h"
#include "libbase_idfs.h"

#include "Qidfs_utils.h"
#include "Qpeace_err_diag.h"

#include "qie.h"

#define QP_DEFAULT_MISSING 0.0
#define QOUTSIDE_MIN   -1.e38      
      /* equality test on -3.e38 fails so test < QOUTSIDE_MIN instead */
#define NSWEEPS_4_SWEEPS_IN_SPIN 4    
                      /* 4 sweeps gives 3 dt's, then remove max, min*/
#define PEACE_SWEEP_GAP_DEFAULT  1.0   /* secs for gap from 1 sweep to next */
#define PEACE_SPIN_GAP_DEFAULT   4.5   /* ditto for spin */


#define N_MAX_IDFS_FILES 10		      
#define N_MAX_IDFS_START_NUDGES 5
#define PHI_SC_GAP_SECONDS   60.0
#define PHI_SC_GAP_DEFAULT     0.0  /* default set phi_sc = 0 volts if gap */
#define PROBE2PLASMA 1.0     /* 1 volt nominal probe to plasma poten for efw*/
#define SAFE_EMIN    0.1     /* 0.1eV min - safe in case s/c pot gives neg E */
#define SAFE_EMAX    1.0     /* ditto for max - this most unlikely to be used*/
#define NEGATIVE_ENERGY2KMS_ERROR -1.0  /* trapped as < 0; v = sqrt(v^2 + phi)*/
                                        /* with phi < -v^2  !!! */

typedef enum ARITH_OPER_E{
   NONE,
   ADD,
   SUBTRACT,
   MULTIPLY,
   DIVIDE
} ARITH_OPER;

typedef enum SWEEP_CONGRUENT_TEST_E{
   BIN_UNIT_MISMATCH = -2,
   PSD_UNIT_MISMATCH = -1,
   CONGRUENT         = 0,
   NLEV_MISMATCH     = 1,
   NSEN_MISMATCH     = 2,
   EBINS_MISMATCH    = 3,
   THBINS_MISMATCH   = 4,
   UNKNOWN_CONGRUENT_ERROR = 5
} SWEEP_CONGRUENT_TEST;

typedef enum PEACE_GATHER_CHOICE_E{
   GATHER_SWEEPS = 0,
   GATHER_SPINS,
   GATHER_SECS,
   NO_GATHER
} PEACE_GATHER_CHOICE;


typedef struct PEACE_SWEEP_S{
/* This structure stores all(?) the info for a single sweep of PEACE data in
*  science units.
*
* ***************** ENERGY LEVEL ORDER ASCENDING ******************
*  Note that energy levels are assumed to be in ASCENDING order of energy,
*  i.e., inverted with respect to decaying energy sweeps
*
* ***************** ANGLES are FLOW DIRECTIONS *******************
*  Note also that all angles <=> directions are FLOW directions
*  Theta bins must be in ASCENDING order
*/
   IDFS_TIME btime;          /* start time of sweep                          */
   IDFS_TIME etime;          /* end time of sweep                            */
   IDFS_TIME suntime;        /* sun sensor time; to determine spin           */
   int n_sen;                /* number of sensors <=> pol bins               */
   int n_lev;                /* energy levels - or samples in idfs-speak     */
   float phi_lo;             /* start az angle, in degrees                   */
   float phi_hi;             /* end az angle                                 */
   PHI_ZERO_FRAME phi_frame; /* at present only PEACE_TZERO                  */
   float *thlo;              /* pntr to array of n_sen val for pol bin start */
   float *thhi;              /* ditto for pol bin end                        */
   float *elo;               /* energy bin start                             */
   float *ehi;               /* energy bin end                               */
   float phi_sc;             /* s/c potential in Volts                       */
   BIN_UNITS bin_units;      /* ev, km/s, etc. from enum                     */
   float *psd;               /* pntr to 1st element in array z[n_sen][n_lev] */
                             /* so psd[jth][ilev] = psd[n_lev*jth+ilev]      */
   PSD_UNITS psd_units;      /* units for psd array                          */
   float missing_value;      /* value to be treated as missing               */
   float *pa;                /* pointer to array of [n_sen][n_lev] pitch angs*/
   IDFS_LINEAGE *lineage;    /* pointer to lineage structure of proj, miss,..*/
} PEACE_SWEEP;

typedef struct PEACE_GATHER_GAPS_S{
   float sweep_gap;          /* secs from start of sweep to next; gap if > */
   float spin_gap;           /* secs from start of spin to next; gap if > */
} PEACE_GATHER_GAPS;   

typedef struct PEACE_SPIN3D_S{
/*
* This structure stores a full spin of PEACE 3D data in the form of individual
* sweeps. Most of the info is held within the sweep structures themselves,
* so this is quite simple. In particular, the azimuths are in the sweep data
*
* NOTE: as of V2 all sweeps have their own bins and the sweeps are not held
* in an array, but pointed to by an array of pointers
*/
   IDFS_TIME btime;        /* start time of spin                */
   IDFS_TIME etime;        /* end time of spin                  */
   float tspin;            /* spin period in seconds            */
   int n_swp;              /* number of sweeps in the spin      */
   PEACE_SWEEP **sweeppp;  /* pointer to array of n_swp sweep structure ptrs */
} PEACE_SPIN3D;

typedef struct PEACE_1D_SWEEP_CUT_S{
/* Hold 1-D radial cut from a sweep simplified to centre times, energies,....*/
   IDFS_TIME starttime;      /* start time of sweep                          */
   IDFS_TIME endtime;        /* end time of sweep                          */
   int n_lev;                /* energy levels - or samples in idfs-speak     */
   float phi_cen;            /* centre az angle, in degrees                  */
   float thcen;              /* centre of polar bin in degrees               */
   float *ecen;              /* energy bin centres                           */
   BIN_UNITS bin_units;      /* ev, km/s, etc. from enum                     */
   float *psd1D;             /* pntr to 1st element in array z[n_lev]        */
   PSD_UNITS psd_units;      /* units for psd array                          */
   IDFS_LINEAGE *lineage;    /* pointer to lineage structure of proj, miss,..*/
} PEACE_1D_SWEEP_CUT;

typedef struct PEACE_PITCH_ANGLE_OPTS_S{
/* hold a few options for idfs pitch angle calcs */
   T_OR_F do_pa;             /* QPTRUE to do it; QPFALSE otherwise */
   int    n_pabins;          /* How many pitch angle bins          */
   T_OR_F mu_bins;           /* QPTRUE for equal mu=cos(th) bins   */
} PEACE_PITCH_ANGLE_OPTS;
 
typedef struct PEACE_SC_POTENTIAL_S{
/* holds arrays of s/c potential */
  float *phisc;                /* from EFW                              */
  IDFS_TIME *phisc_time;       /* NB EFW PP data has centred tags       */
  int n;                       /* size of arrays                        */
  float dt_gap;                /* secs. dt's > this are treated as gaps */
  float phisc_gap;             /* value of phisc within a gap           */
  IDFS_LINEAGE phisc_lineage;  /* EFW virtual                           */
} PEACE_SC_POTENTIAL;
 

typedef enum PHISC_CHOICE_E{
    NO_SC_POT_CORR = 0,
    DO_SC_POT_CORR,
    CONST_SC_POT
} PHISC_CHOICE;
 
   
/* Sweep data */

char * qpeace_version_str(void);

SDDAS_SHORT malloc_sweep_arrays(PEACE_SWEEP *swpp, T_OR_F do_pa);

void nullify_sweep_arrays(PEACE_SWEEP *swpp);       

PEACE_SWEEP  *create_sweep_full_copy(PEACE_SWEEP *swp);

void free_sweep_arrays(PEACE_SWEEP *swp);

void free_n_sweeps(int n, PEACE_SWEEP **swppp);

void print_sweep_structure(PEACE_SWEEP *swpp);

/* Sweep manipulation */

SDDAS_SHORT CombineSweeps(
        PEACE_SWEEP *swp_new, /* target; could be part full */
        float *weight,       /* 2-D array; same size as new*/
	PEACE_SWEEP *swp_old, /* old sweep */
	INTERP_QUANTITY interpq  /* differential or integ (cnts)*/
	);
	
SDDAS_SHORT fill_sweep_bin_area(float *A, PEACE_SWEEP *swpp);

SDDAS_SHORT divide_by_area(float *A, PEACE_SWEEP *swpp);

SDDAS_SHORT multiply_by_area(float *A, PEACE_SWEEP *swpp);

int test_sweeps_congruent(PEACE_SWEEP *swp1, PEACE_SWEEP *swp2);

PEACE_SWEEP *sweep_arithmetic(PEACE_SWEEP *swp_pri, PEACE_SWEEP *swp_sec,
                              ARITH_OPER oper);

PEACE_SWEEP *rebin_sweep(PEACE_SWEEP *bin_holder, PEACE_SWEEP *swp);

SDDAS_SHORT rebin_n_sweeps(PEACE_SWEEP *bin_holder, int nswps, 
     PEACE_SWEEP **swpsinpp, int *nrebinned, PEACE_SWEEP ***swpsoutppp);
     
void print_weight(float *weight, int n_th, int n_lev);

void sweeps_autoscaler(PEACE_SWEEP **swppp, int nswp, 
                      float *psdmin, float *psdmax);
		      
void sweep_scaler(PEACE_SWEEP *swpp, float mult);		      
    
PEACE_1D_SWEEP_CUT *sweep_1D_cut(float theta, PEACE_SWEEP *swp);

void free_1D_cut_arrays (PEACE_1D_SWEEP_CUT *swpcut);

void print_1D_cut(PEACE_1D_SWEEP_CUT *swpcut);

SDDAS_SHORT rebin_sweep_to_pa(PEACE_SWEEP **swpp, int nswp,
          PEACE_PITCH_ANGLE_OPTS *paopts, PEACE_SWEEP **paswpp);

SDDAS_SHORT Rebin_n_sweeps_to_pa(PEACE_SWEEP **swppp, int nswps, 
    int nswpgrp, PEACE_PITCH_ANGLE_OPTS *paopts, 
    PEACE_SWEEP ***paswppp_p, int *npaswp);

int iend_sweep_gather(PEACE_SWEEP **swppp, int nswps, int istart,
     PEACE_GATHER_CHOICE gather, PEACE_GATHER_GAPS gaps, float gsecs, 
     int gswp_spin);
     
void set_pa_opts_defaults(PEACE_PITCH_ANGLE_OPTS *paopts);

void sweep_spin_gap_defaults(PEACE_GATHER_GAPS *gapp);

SDDAS_SHORT gather_sweeps (PEACE_SWEEP **swppp, int nswp, PEACE_SWEEP *binh,
   PEACE_GATHER_CHOICE gatch, PEACE_GATHER_GAPS gaps, float gath_secs,
   int gath_swp_spins, PEACE_SWEEP ***gath_swpppp, int *ngat);

int inbin(float value, float *lobins, float *hibins, int nbins);

/* Spin data */

SDDAS_SHORT malloc_spin_sweep(PEACE_SPIN3D *spinp, int n_sen, int n_lev,
                       T_OR_F do_pa);

SDDAS_SHORT free_spin_data (PEACE_SPIN3D *spinp);

SDDAS_SHORT free_n_spins(PEACE_SPIN3D **spinpp, int nspins);

PEACE_SPIN3D *copy_spin (PEACE_SPIN3D *spin);

void print_peace_spin_info(PEACE_SPIN3D *spinp);

int find_az_sweep ( PEACE_SPIN3D *spinp, float az );

SDDAS_SHORT Rebin_n_spins_to_pa(PEACE_SPIN3D **spinpp, int nspins, 
     PEACE_PITCH_ANGLE_OPTS *paopts, PEACE_SWEEP ***paswppp_p);


/* Qsweep_spin_read routines */

SDDAS_SHORT fill_sweep(PEACE_SWEEP *swpp , IDFS_LINEAGE *lngp,
              struct idf_data *idfp, SDDAS_ULONG dkey, 
	      T_OR_F have_bins, T_OR_F do_pa);

SDDAS_SHORT fill_single_sen_sweep(PEACE_SWEEP *swpp, int isen,
               SDDAS_ULONG dkey, struct idf_data *idfp, 
	       T_OR_F do_pa);

SDDAS_SHORT GetPeaceSpin(IDFS_LINEAGE *lngp, IDFS_TIME *btimep,
   BIN_UNITS bin_units, PSD_UNITS psd_units, 
   T_OR_F do_pa, PEACE_SPIN3D *spinp);

SDDAS_SHORT GetNPeaceSpins(
   IDFS_LINEAGE *lngp, 
   IDFS_TIME *btime,
   IDFS_TIME *etime,
   BIN_UNITS bin_units, 
   PSD_UNITS psd_units, 
   T_OR_F do_pa,
   int *nspins, 
   PEACE_SPIN3D ***spinppp);

SDDAS_SHORT GetN1PeaceSpins(   /* no longer used Feb 2005 */
   IDFS_LINEAGE *lineage, 
   IDFS_TIME *btime,
   IDFS_TIME *etime,
   BIN_UNITS bin_units, 
   PSD_UNITS psd_units, 
   T_OR_F do_pa,
   int *nspins, 
   PEACE_SPIN3D ***spinppp);


PEACE_SWEEP *PeaceGetAzSlice(
       IDFS_LINEAGE *lineage, IDFS_TIME *btime, float az,
       BIN_UNITS bin_units,
       PSD_UNITS psd_units,
       PEACE_PITCH_ANGLE_OPTS *paopts);

PEACE_SWEEP *PeaceGetAzSliceV2(
       IDFS_LINEAGE *lineage, IDFS_TIME *btime, IDFS_TIME *etime, float az,
       BIN_UNITS bin_units, PSD_UNITS psd_units,
       PEACE_PITCH_ANGLE_OPTS *paopts,
       PHISC_CHOICE phiscch,
       float phisc_value, float phisc_gapsecs,
       PEACE_GATHER_CHOICE gatch, PEACE_GATHER_GAPS gaps, 
       float gath_secs, int gath_swp_spins);
       
SDDAS_SHORT GetN1PeaceSweeps(  /* no longer used Feb 2005 */
        IDFS_LINEAGE *lngp, 
	IDFS_TIME *btimep,
        IDFS_TIME *etimep,
	BIN_UNITS bin_units, 
	PSD_UNITS psd_units,
	T_OR_F do_pa,
	int *nswps, 
	PEACE_SWEEP ***swpppp);

SDDAS_SHORT GetNPeaceSweeps(
        IDFS_LINEAGE *lngp, 
	IDFS_TIME *btimep,
        IDFS_TIME *etimep,
	BIN_UNITS bin_units, 
	PSD_UNITS psd_units,
	T_OR_F do_pa,
	int *nswps, 
	PEACE_SWEEP ***swpppp);

SDDAS_SHORT GetNPotCorrectedPeaceSweeps(
        IDFS_LINEAGE *lngp, 
	IDFS_TIME *btimep,
        IDFS_TIME *etimep,
	BIN_UNITS bin_units, 
	PSD_UNITS psd_units,
	T_OR_F do_pa,
	T_OR_F const_phisc,
	float phisc_value, float phisc_gapsecs,
	int *nswps, 
	PEACE_SWEEP ***swpppp);

int idfs_sweep_counter (SDDAS_ULONG dkey, IDFS_LINEAGE *lngp, 
                        struct idf_data *idfp,
                        IDFS_TIME *bt,  IDFS_TIME *et);
			
int idfs_sweep_nlevels (SDDAS_ULONG dkey, IDFS_LINEAGE *lngp,
      struct idf_data *idfp, int *nsenp);
				  
SDDAS_SHORT init_spin3d(PEACE_SPIN3D *spin_dp,   /* no longer used Feb 2005*/
                        IDFS_LINEAGE  *lngp,
                        struct idf_data *idfp,
                        SDDAS_ULONG dkey, int *n_sens, int *n_lev,
			T_OR_F do_pa);

SDDAS_SHORT idfs_n_spin_sweep_etime(
        IDFS_LINEAGE *lngp, 
	IDFS_TIME *btimep,
        IDFS_TIME *etimep,
	T_OR_F do_pa,
	PEACE_GATHER_CHOICE gathch,
	int n2count);

float sweep_duration_calculator(IDFS_TIME *swpstart, int n); /* cut Feb 2005*/

SDDAS_SHORT fill_spin3d (PEACE_SPIN3D *spinp,   /* no longer used Feb 2005 */
                        IDFS_LINEAGE  *lngp,
                        struct idf_data *idfp,
                        SDDAS_ULONG dkey,
                        BIN_UNITS bin_units, 
			PSD_UNITS psd_units,
                        PHI_ZERO_FRAME phi_frame,
			T_OR_F do_pa );


/* Qsweep_export routines */

SDDAS_SHORT nsweeps2QiCDFContents(PEACE_SWEEP **swppp, int nswps,
                                  QiCDFContents **QiCDFp);

SDDAS_SHORT nswps2QiCDF_g_atts(QiCDFContents *QiCDF, PEACE_SWEEP **swppp);

SDDAS_SHORT nswps2QiCDF_default_attr(QiVarAttribute *attp, int iatt);

SDDAS_SHORT nswps2QiCDF_time_vars(QiCDFVariable *epo, QiCDFVariable *epo_delta,
                                  PEACE_SWEEP **swppp, int nswps);

SDDAS_SHORT nswps2QiCDF_psd_var(QiCDFVariable *psdv, 
                                  PEACE_SWEEP **swppp, int nswps);

SDDAS_SHORT nswps2QiCDF_phi_vars(QiCDFVariable *phiv, QiCDFVariable *phiv_delta, 
                                  PEACE_SWEEP **swppp, int nswps);

SDDAS_SHORT nswps2QiCDF_theta_vars(QiCDFVariable *thv, QiCDFVariable *thv_delta,
                                  PEACE_SWEEP **swppp, int nswps);

SDDAS_SHORT nswps2QiCDF_lev_vars(QiCDFVariable *levv, QiCDFVariable *levv_delta, 
                                  PEACE_SWEEP **swppp, int nswps);

SDDAS_SHORT nswps2QiCDF_phisc_var(QiCDFVariable *phisc, 
                                  PEACE_SWEEP **swppp, int nswps);

long   PQiChangeVattr(char *attr_name,  char *new_entry, QiCDFVariable *var);

int QiCEFmsg (int mode, const char *txt0, const char *txt1, const char *txt2);

/*  Qphisc routines */

void SC_Pot_init(PEACE_SC_POTENTIAL *phisc_s);

SDDAS_SHORT SC_Pot_fetch(
    PEACE_SC_POTENTIAL *phisc_s, 
    IDFS_TIME *btime, 
    IDFS_TIME *etime);

SDDAS_SHORT Phisc_fill_not_available(PEACE_SC_POTENTIAL *phisc,
    IDFS_TIME *btime, IDFS_TIME *etime);
    
SDDAS_SHORT free_phisc_arrays(PEACE_SC_POTENTIAL *phisc);

void phisc_dump(PEACE_SC_POTENTIAL *phisc);

SDDAS_SHORT phisc2sweep(PEACE_SC_POTENTIAL *phisc, PEACE_SWEEP *swpp, 
   int *start);

SDDAS_SHORT phisc2sweep_all(PEACE_SC_POTENTIAL *phisc, 
                   PEACE_SWEEP **swppp, int nswp);

SDDAS_SHORT sweep_sc_potential_adjust(PEACE_SWEEP **swppp,int nswps);

float add_ev2_kms(float kms, float ev);

void sweep_make_level_missing(PEACE_SWEEP *swpp, int ilev);



#ifdef __cplusplus
}
#endif

#endif  /* end ifndef _QSWEEP_SPIN */
