/* Qpeace_plot.c ******************
*  Part of QPeace software
*  See README file in QPeace distribution for copyright information
*
*  SJ Schwartz   (s.j.schwartz@qmw.ac.uk)
*  Queen Mary, University of London
*  May 2001 - re-organised Qpeace routines and created this module from others
*  Oct 2001 - added slice and map routines
*  June 2002 Changed labels and slice to FLOW directions
*
*  Routines for plotting PEACE sweeps and spins
*  Uses generic plotting routines found in PGSpectro
*
*/

#ifdef __cplusplus
extern "C"{
#endif

#include "Qpeace_plot.h"


/***********************************************************************/
PGSpectro_ret Sweep_wheel_plot(PEACE_SWEEP *swpp,
                               SWEEP_WHEEL_OPTIONS *swop){
/***********************************************************************/
/* plot the contents of a single sweep structure as a wheel plot */

PGWheel wheel, *whp;
int lev,jth, n_lev, n_sen;
int kz, i, idummy;
int ncircles = NCIRCLES;
float rcircles[NCIRCLES], rcirc, drcirc;
float psd_min, psd_max, psd_temp;

char title[500],suptitle[500],supsuptitle[500];
char xlabel[100],ylabel[100],wedgelabel[100];
char subtitle[500], subsubtitle[500];
char zerodeg[] = {'0','\260', '\0'};  /* oct 260 = dec 176 = pgplot deg */
char oneeightydeg[] = {'1', '8', '0', '\260', '\0'};

int bhr, bmin, bsec, bmsec,ehr,emin,esec,emsec;
long nano;
PGSpectro_ret pgret_val;
float psd_abs_min,missing;


whp = &wheel;

PGWheel_set_defaults (whp);
whp->domain.xndcmin=swop->xndcmin;
whp->domain.xndcmax=swop->xndcmax;
whp->domain.yndcmin=swop->yndcmin;
whp->domain.yndcmax=swop->yndcmax;
whp->text_scale = swop->text_scale;

/* whp->skip_missing = YES;*//* may need to add to SWEEP_WHEEL_OPTIONS struct */
/* whp->zmissing = 0.0; */   /* zeroes treated as missing */
whp->skip_missing = swop->skip_missing;
missing = whp->zmissing = swpp->missing_value;  /* up-dated may2002*/

n_lev = swpp->n_lev;
n_sen  = swpp->n_sen;

whp->z = malloc (n_lev * n_sen * sizeof (float));
if (whp->z == NULL){
   printf("failed to malloc array for wheel plot\n");
   return PGSPECTRO_MALLOC_FAILURE;
}
whp->thbinlow = malloc (n_sen * sizeof (float));
whp->thbinhigh = malloc (n_sen * sizeof (float));
if (whp->thbinlow == NULL || whp->thbinhigh == NULL){
   printf("Failed to malloc thbin arrays for wheel plot\n");
   if (whp->z != NULL) free(whp->z);  /* fixed == to != Mar 2002 !!!*/
   if (whp->thbinlow != NULL) free (whp->thbinlow);
   if (whp->thbinhigh != NULL) free (whp->thbinhigh);
   return PGSPECTRO_MALLOC_FAILURE;
}


/* re-order psd array and find max and min values */
/* z[n_lev][n_sen] so z[lev][jth] = z[n_sen * lev + jth] */
/* psd[n_sen][n_lev] so psd[jth][lev] = psd[n_lev*jth + lev]   */

psd_abs_min = 100. * FLT_MIN;  /* initialise min/max fixed 8 May 2001 */
i=0;
psd_min = psd_max = psd_abs_min;
idummy = n_sen * n_lev;        /* find non-zero psd to initialise min max */
for( i=0; i<idummy; i++){
   if (fabs(swpp->psd[i]) > psd_abs_min && swpp->psd[i] != missing){
      psd_min = psd_max = swpp->psd[i];
      break;
   }
}

for (lev = 0; lev < n_lev; lev++){
  kz = n_sen * lev;
  for (jth = 0; jth < n_sen; jth++){
    psd_temp = swpp->psd[ n_lev * jth + lev ];
    whp->z[ kz + jth ] = psd_temp;
/*    if (fabs(psd_temp) > FLT_MIN){ */ /* zeroes are "missing values" */
    if (psd_temp != missing){           /* not any more! */
      psd_min = (psd_min < psd_temp ) ? psd_min : psd_temp;
      psd_max = (psd_max > psd_temp ) ? psd_max : psd_temp;
    }
  }
}

if (fabs(psd_max)> 0.1*FLT_MAX){  /* remove inf values */
   psd_max = (psd_max > 0.0) ? 0.1*FLT_MAX : -0.1*FLT_MAX;
}
if (fabs(psd_min)> 0.1*FLT_MAX){  /* remove inf values */
   psd_min = (psd_min > 0.0) ? 0.1*FLT_MAX : -0.1*FLT_MAX;
}
if(psd_max < psd_min){psd_temp=psd_min; psd_min = psd_max; psd_max=psd_temp;}
if (fabs(psd_max - psd_min)<FLT_MIN){  /* make psd range finite */
   psd_max = (fabs(psd_min)<FLT_MIN) ? 1.0 : 1.1*fabs(psd_min);
}

whp->rbinlow  = swpp->elo;      whp->rbinhigh =  swpp->ehi;
scale_then_rotate_angles(swpp->thlo, whp->thbinlow, n_sen, 90.0 , 1.0);
scale_then_rotate_angles(swpp->thhi, whp->thbinhigh, n_sen, 90.0, 1.0);

whp->nr = swpp->n_lev;          whp->nth = swpp->n_sen;

if (swop->zautoscale == YES){
   whp->zscalemin = swop->zmin = psd_min;  /* set zmin and also return it*/
   whp->zscalemax = swop->zmax = psd_max;
} else {
   whp->zscalemin = swop->zmin;
   whp->zscalemax = swop->zmax;
}

to_hms_time(&swpp->btime, &bhr, &bmin, &bsec, &nano);
bmsec = nano/1000000;
to_hms_time(&swpp->etime, &ehr, &emin, &esec, &nano);
emsec = nano/1000000;

 sprintf(xlabel,"%s", BIN_UNITS_STRING(swpp->bin_units));
 sprintf(ylabel,"%s", BIN_UNITS_STRING(swpp->bin_units));
 sprintf(supsuptitle, " %s %s" ,
       swpp->lineage->mission,
       swpp->lineage->experiment);
 sprintf(suptitle," %s",swpp->lineage->instrument);
 sprintf(title," %s",swpp->lineage->virtual_inst);
 sprintf(subtitle, " %4d %03d %02d:%02d:%02d.%03d, %4d %03d %02d:%02d:%02d.%03d ",
       swpp->btime.year, swpp->btime.doy, bhr, bmin, bsec, bmsec,
       swpp->etime.year, swpp->etime.doy, ehr, emin, esec, emsec)  ;
 sprintf(subsubtitle, " %.2f < phi(azim) < %.2f ", swpp->phi_lo, swpp->phi_hi);
 sprintf(wedgelabel,"%s",PSD_UNITS_STRING(swpp->psd_units));

 whp->title       =  title;
 whp->subtitle    = subtitle;
 whp->subsubtitle = subsubtitle;
 whp->suptitle    = suptitle;
 whp->supsuptitle = supsuptitle;
 whp->xlabel      = xlabel;
 whp->ylabel      = ylabel;
 whp->wedge.label = wedgelabel;
 whp->title_just  = swop->title_just;

if (swop->rautoscale == YES){
   i=0;  /* protect autoscale against negative r */
   while (i<n_lev){  
      whp->domain.rmin = swpp->elo[i++];
      if (whp->domain.rmin > 0.0) break;
   }
   if (whp->domain.rmin < 0.0) whp->domain.rmin = SAFE_EMIN;  
   whp->domain.rmax = swpp->ehi[n_lev -1];
   if (whp->domain.rmax < 0.0) whp->domain.rmax = SAFE_EMAX;
   swop->rmin = whp->domain.rmin;
   swop->rmax = whp->domain.rmax;
} else {
   whp->domain.rmin = swop->rmin;
   whp->domain.rmax = swop->rmax;
}

whp->ncircles = ncircles;  /* set ncircle guide circles */

if(DIAG_PRINT_3D){
   printf("sweep wheel opts rmin, rmax = %g, %g\n",swop->rmin,swop->rmax);
   printf("wheelp->domain rmin,rmax = %g, %g\n",whp->domain.rmin,
         whp->domain.rmax);
}

if (swop->rlinlog == PGSPECTRO_LOG){
   drcirc = (SafeLog10(whp->domain.rmax) -
             SafeLog10(whp->domain.rmin))/(ncircles +1);
   rcirc = SafeLog10(whp->domain.rmin) + 0.5*drcirc;
   for (  i = 0; i< ncircles; i++, rcirc +=drcirc ){
      if (DIAG_PRINT_3D){
          printf("rcirc, 10^rcirc, nice(10^rcirc) %g, %g, %g \n",
                rcirc, pow(10.,rcirc), near_nice_number(pow(10.,rcirc)));
      }
      rcircles[i] = near_nice_number(pow(10.,rcirc));
      if (DIAG_PRINT_3D) printf("i, rcircles[i]: %d %g\n",i,rcircles[i]);
   }
} else {
   drcirc = (whp->domain.rmax - whp->domain.rmin)/(ncircles +1);
   rcirc = whp->domain.rmin + 0.5*drcirc;
   for (  i = 0; i< ncircles; i++, rcirc +=drcirc ){
      rcircles[i] = near_nice_number(rcirc);
      if (DIAG_PRINT_3D) printf("i, rcircles[i]: %d %g\n",i,rcircles[i]);
   }
}

whp->rcircles = rcircles;


whp->color_table = swop->color_table;
whp->rlinlog = swop->rlinlog;
whp->zlinlog = swop->zlinlog;

pgret_val = PGWheel_Plot(whp);

if (pgret_val != PGSPECTRO_OK){
   printf("PGWheel_Plot returned error %d\n", pgret_val);
}
cpgmtxt("T",-1.5, 0.5, 0.5, zerodeg);
cpgmtxt("B",-1.5, 0.5, 0.5, oneeightydeg);

if (whp->z != NULL) free(whp->z);
if (whp->thbinlow != NULL) free (whp->thbinlow);
if (whp->thbinhigh != NULL) free (whp->thbinhigh);


return pgret_val;

}


/***************************************************************/
void Set_sweep_wheel_options_default(SWEEP_WHEEL_OPTIONS *swop){
/***************************************************************/

swop->color_table = PGSPECTRO_RAINBOW;
swop->zlinlog = PGSPECTRO_LOG;
swop->rlinlog = PGSPECTRO_LOG;
swop->zautoscale = YES;
swop->rautoscale = YES;
swop->skip_missing = YES;
swop->zmin = 1.0;   swop->zmax = 10.0;
swop->rmin = 1.0;   swop->rmax = 10.0;
swop->xndcmin = 0.1; swop->yndcmin = 0.15;
swop->xndcmax = swop->yndcmax = 0.8;
swop->title_just = 0.5;
swop->text_scale = 1.0;
swop->pg_device = "/Xserv";

return;
}


/**********************************************************************/
PGSpectro_ret PlotTwoSweeps(PEACE_SWEEP *swp_pri, PEACE_SWEEP *swp_sec,
       SWEEP_WHEEL_OPTIONS *swop){
/**********************************************************************/
/* plot two peace sweeps in a wheel plot; primary on left 
*  calling routine sets main sweep options, and sets up page,
*  advances to new panel if required, etc.
*/


int pgid;
float *thlo2,*thhi2,*savelo, *savehi;
int n_sen2;
PGSpectro_ret pg_ret_val;

/* check inputs and bale out if necessary */
if (swp_pri == NULL || swp_sec == NULL){
   printf("one of requested sweeps for plot is null; plot abandoned\n");
   return PEACE3D_ERROR;
}

cpgqid(&pgid);
if(DIAG_PRINT_3D)printf("\npgid = %d\n",pgid);
if (!pgid){   /* just in case no device, we'll open one here */
  if (cpgopen(swop->pg_device) <=0){
     printf("Failed to open pgplot device successfully\n");
     return PEACE3D_ERROR;
  } else {
       cpgshls(0,0,1,0);
       cpgshls(1,0,0,0);
       cpgpage();
  }
} 

/* calling routine will need to call cpgpage() if required to advance
*  page on an already open device. Inserting a call here would force the
*  user to hit return to advance to next page
*/

swop->title_just = 1.0;  /* rest of options set in calling routine */

pg_ret_val = Sweep_wheel_plot(swp_pri, swop);
if (pg_ret_val != PGSPECTRO_OK){
   printf("Sweep_wheel_plot returned PGSPECTRO error %d\n", pg_ret_val);
   return PEACE3D_ERROR;
}
       
swop->title_just = 0.0;
swop->zautoscale = NO; /* re-use scalings so both halves match!!*/
swop->rautoscale = NO; /* note Sweep_wheel_plot sets min/max found from auto*/

n_sen2 = swp_sec->n_sen;  /* flip sec sweep to right hand side */
thlo2 = (float *)malloc(n_sen2 * sizeof(float));
thhi2 = (float *)malloc(n_sen2 * sizeof(float));
savelo = swp_sec->thlo;
savehi = swp_sec->thhi;
scale_then_rotate_angles(savelo, thlo2, n_sen2, 0.0, -1.0);
scale_then_rotate_angles(savehi, thhi2, n_sen2, 0.0, -1.0);
swp_sec->thlo = thlo2;
swp_sec->thhi = thhi2;

pg_ret_val = Sweep_wheel_plot(swp_sec, swop);
if (pg_ret_val != PGSPECTRO_OK){
   printf("Sweep_wheel_plot returned PGSPECTRO error %d\n", pg_ret_val);
}

/* restore sweep angles and cleanup */
swp_sec->thlo = savelo;
swp_sec->thhi = savehi;

if (thlo2 != NULL) free(thlo2);
if (thhi2 != NULL) free(thhi2);

cpgupdt();   /* flush pgplot buffer */

return PGSPECTRO_OK;
}
     
/**************************************************************************/
PGSpectro_ret Qjas_plot( PEACE_SWEEP *swps[], /* pointer to nswp sweep ptrs */
                   int nswp,           /* number of sweeps */
		   PGSpectro *pgspec,  /* brings in various settings */
		   IDFS_TIME idf_start,/* plot time range */
		   IDFS_TIME idf_end){
/**************************************************************************/

/* pgspec needs to bring in:
*
*     zscalemin, zscalemax for colour scale
*     color_table
*     domain.xndcmin, domain.xndcmax  for overall size
*     zlinlog  [xlinlog and ylinlog reset to ...LIN]
*     wedge  structure for wedge placement & width; label reset below
*     
*     NB: PGSpectro_set_defaults will do all this, and only zscalemin/max
*     are probably ridiculous
*/

char date_start_str[20], date_end_str[20]; 
double epoch_midnight;
char wedgelabel[MAX_PSD_UNITS_STR_LEN];
char title_str[500], thlo_str[20], thhi_str[20];  /* sub_title_str removed */
char no_title_str[10], no_subtitle_str[10];

PGSpectro_ret pgsret;
int nx, ny, nlev;
int i, itest;
float yndcbot, dyndc;
float zwedgemin, zwedgemax;
double epoch_test;

if (QJAS_DIAG){
   printf("\n------------------------------------------------\n");
   printf("Qjas_plot inputs: nswp = %d. Sweep 0 is\n", nswp);
   print_sweep_structure(swps[0]);
   printf("Sweep nswp-1 is\n");
   print_sweep_structure(swps[nswp-1]);
}
/* check that all sweeps are congruent; if not bale out immediately */
for (i=0; i<nswp -1; i++){
  if ((itest=test_sweeps_congruent(swps[i], swps[i+1]))!=CONGRUENT ){
      printf("Qjas_plot received incongruent sweeps");
      printf(" at i=%d (test = %d),and failed\n",i, itest);
      return PGSPECTRO_QJAS_INCONGRUENT;
  }
}

/* initialise date strings, midnight, time world coords, timex */
qjas_time_init(&idf_start, &idf_end, date_start_str, date_end_str,
                    pgspec, &epoch_midnight);

if (QJAS_DIAG){
   printf("qjas_time_init: epoch_midnight = %g\n", epoch_midnight);
   printf("swps[0] btime = ");
   print_idfs_time(&(swps[0]->btime));
   epoch_test = idfs_time_to_epoch_secs(&(swps[0]->btime));
   printf(" = EpochSecs %g\n", epoch_test);
}

/* draw wedge full size of plot */
if (pgspec->char_size < 0.0) {  /* else let user set */
    pgspec->char_size = 0.7*PGSPECTRO_DEFAULT_CHAR_SIZE; 
    cpgsch(0.7*PGSPECTRO_DEFAULT_CHAR_SIZE);
}  else {
    cpgsch(pgspec->char_size);
}
cpgsvp(pgspec->domain.xndcmin, pgspec->domain.xndcmax, 
       pgspec->domain.yndcmin, pgspec->domain.yndcmax);
set_pgp_rainbow(pgspec->color_table);
sprintf(wedgelabel, "%s", PSD_UNITS_STRING(swps[0]->psd_units));
pgspec->wedge.label=wedgelabel;
switch(pgspec->zlinlog){   /* since we do wedge manually, need to do : */
   case PGSPECTRO_LOG:
      zwedgemin = SafeLog10(pgspec->zscalemin);
      zwedgemax = SafeLog10(pgspec->zscalemax);
      break;
   case PGSPECTRO_LIN: default:
      zwedgemin = pgspec->zscalemin;
      zwedgemax = pgspec->zscalemax;
      break;
}
    
spectro_wedge(pgspec->wedge, zwedgemin, zwedgemax, pgspec->zlinlog);

cpgmtxt("T",1.0, 0.0, 1.0, BIN_UNITS_STRING(swps[0]->bin_units));
		    
/* allocate arrays; we could re-use theta arrays, 
*  i.e., pgspec->ybinlow=swps[0]->thlo and sim for high, but we
*  make local copies here */
nx = nswp;               pgspec->nx = nswp;
ny = swps[0]->n_sen;     pgspec->ny = ny;
if ((pgsret=malloc_spectro_arrays(pgspec, nx, ny))!=PGSPECTRO_OK){
  printf("ERROR: malloc_spectro_arrays returned %d\n", pgsret);
  return pgsret;
}

/* fill time (x) arrays and theta arrays */
pgspec->xlinlog = PGSPECTRO_LIN;
for (i=0; i<nx; i++){
  pgspec->xbinlow[i] = (float)(idfs_time_to_epoch_secs(&(swps[i]->btime)) -
                                   epoch_midnight); 
  pgspec->xbinhigh[i] = (float)(idfs_time_to_epoch_secs(&(swps[i]->etime)) -
                                   epoch_midnight);
}
pgspec->ylinlog = PGSPECTRO_LIN;
for (i=0; i<ny; i++){
  pgspec->ybinlow[i] = swps[0]->thlo[i];				    
  pgspec->ybinhigh[i] = swps[0]->thhi[i];
}

/* bottom subpanel will have azimuth vs time */
yndcbot = pgspec->domain.yndcmin;
nlev = swps[0]->n_lev;
dyndc   = (pgspec->domain.yndcmax - yndcbot)/(nlev + 1);


/* fill azimuths */
pgspec->domain.yndcmin = yndcbot;
pgspec->domain.yndcmax = yndcbot + dyndc;
Qjas_az_panel(pgspec, swps);  /* plot bottom panel of azimuths */

sprintf(date_start_str, "%s", " ");
pgspec->datestart = date_start_str; pgspec->dateend = date_start_str; /* blank*/

/* do other panels; top panel needs title and (?) subtitle  */
strcpy(no_title_str, " ");
strcpy(no_subtitle_str, " ");
sprintf(title_str, "%s %s %s %s %s",
           swps[0]->lineage->project,
	   swps[0]->lineage->mission,
	   swps[0]->lineage->experiment,
	   swps[0]->lineage->instrument,
	   swps[0]->lineage->virtual_inst);
pgspec->title = no_title_str;
pgspec->subtitle = no_subtitle_str;
pgspec->xnums = NO;  pgspec->ynums = NO; pgspec->draw_wedge = NO;
pgspec->ylabel_vert = YES;
pgspec->domain.yworldmin = swps[0]->thlo[0];
pgspec->domain.yworldmax = swps[0]->thhi[swps[0]->n_sen -1];
pgspec->skip_missing = YES;
pgspec->zmissing=swps[0]->missing_value;

for (i=0; i<nlev; i++){
   pgspec->domain.yndcmin = yndcbot + (i+1)*dyndc;
   pgspec->domain.yndcmax = yndcbot + (i+2)*dyndc;
   if (i == nlev-1){ 
      pgspec->title = title_str;
   }
   pgsret = Qjas_lev_panel(pgspec, swps, i);
   if (pgsret != PGSPECTRO_OK){
      printf("Qjas_lev_panel Failed on level %d returning %d\n",
                     i, pgsret);
      free_spectro_arrays(pgspec);
      return pgsret;
   }
   if (i == nlev-1){
      sprintf(thlo_str, "Flow %3.0f", pgspec->ybinlow[0]);
      sprintf(thhi_str, "%3.0f", pgspec->ybinhigh[pgspec->ny-1]);
      cpgmtxt("R", 1.0, 0.0, 1.0, thlo_str);
      cpgmtxt("R", 1.0, 1.0, 0.0, thhi_str);
      }
}   
free_spectro_arrays(pgspec);
return PGSPECTRO_OK;

}


/***********************************************************************/
void qjas_time_init(IDFS_TIME *idf_startp, IDFS_TIME *idf_endp,
                    char *date_start_str, char *date_end_str,
		    PGSpectro *pgs, double *epoch_midnight){
/***********************************************************************/

/* set date strings
*  calculate epoch of midnight prior to start date
*  set time world coords
*/

int mon_start, day_start, mon_end, day_end;

QanYearDoyToDate(idf_startp->year, idf_startp->doy, &mon_start, &day_start);
sprintf(date_start_str, "%4d-%02d-%02d", idf_startp->year, mon_start, day_start);

QanYearDoyToDate(idf_endp->year, idf_endp->doy, &mon_end, &day_end);
sprintf(date_end_str, "%4d-%02d-%02d", idf_endp->year, mon_end, day_end);

pgs->datestart = date_start_str;
pgs->dateend   = date_end_str;

*epoch_midnight = QEpochSecs (idf_startp->year, mon_start, day_start, 0, 0, 0, 0);
pgs->domain.xworldmin = (float)(idfs_time_to_epoch_secs(idf_startp) - 
                                               *epoch_midnight);
pgs->domain.xworldmax = (float)(idfs_time_to_epoch_secs(idf_endp)   - 
                                               *epoch_midnight);

pgs->timex=YES;
return;
}

/**********************************************************************/
void Qjas_az_panel(PGSpectro *pgs, PEACE_SWEEP **swps){
/**********************************************************************/
/* plot the azimuths in the sweep structures as a pair-wise line plot.
*  All the plot info is contained in the pgspec structure *
*/

int i, nx;
float t[2],az[2];

pgs->domain.yworldmin = 0.0;
pgs->domain.yworldmax = 360.0;
Qjas_set_vpt_win_from_pgs(pgs);  /* set viewport and window */
cpgsci(1);    /* default foreground colour */
cpgmtxt("B", 2.5, 0.0, 0.5, pgs->datestart); /*annotate x-axis with date */
if (strcmp(pgs->datestart, pgs->dateend) ){
  cpgmtxt("B", 2.5, 1.0, 1.0, pgs->dateend);
}
cpgmtxt("B", 2.5, 0.5, 0.5, "UT");
cpgmtxt("L", 4.5, 0.5, 0.5, "Flow Az");
cpgtbox("BCTSNZXHO", 0.0, 0, "BCTSNV", 0.0, 0);
nx = pgs->nx;
for (i=0; i<nx; i++){
   t[0] = pgs->xbinlow[i];      t[1] = pgs->xbinhigh[i];
   az[0] = swps[i]->phi_lo;     az[1] = swps[i]->phi_hi;
   cpgline(2,t,az);
}
return;
}

/**********************************************************************/
void Qjas_set_vpt_win_from_pgs(PGSpectro *pgs){
/**********************************************************************/

cpgsvp(pgs->domain.xndcmin, pgs->domain.xndcmax,
       pgs->domain.yndcmin, pgs->domain.yndcmax);
cpgswin(pgs->domain.xworldmin, pgs->domain.xworldmax,
        pgs->domain.yworldmin, pgs->domain.yworldmax);

return;
}

/***********************************************************************/
PGSpectro_ret Qjas_lev_panel(PGSpectro *pgs, PEACE_SWEEP **swps, int lev){
/***********************************************************************/
/*  plot a single energy level for several sweeps as an angle-time
*   spectrogram. All relevant info is in PGSpectro structure *
*/

char ylabel[100];
float mid_energy;
PGSpectro_ret pgsret;
int i;

Qjas_set_vpt_win_from_pgs(pgs);  /* set appropriate vpt and window */
pgsret=Qjas_fill_angle_t_array(pgs->z, pgs->nx, pgs->ny, swps, lev);
if (pgsret != PGSPECTRO_OK){
  printf("ERROR in Qjas_lev_panel: Qjas_fill_angle_t_array returned %d\n",
                   pgsret);
  return pgsret;
}
/*if (QJAS_DIAG){
  printf("Level %d Panel Timetags in secs since midnight:\n", lev);
  for (i=0; i<pgs->nx; i++){
     printf("%g  -> %g\n", pgs->xbinlow[i], pgs->xbinhigh[i]);
  }
}*/
mid_energy = 0.5* (swps[0]->elo[lev] + swps[0]->ehi[lev]);
sprintf(ylabel, "%7.2g",mid_energy);
pgs->ylabel = ylabel;
pgsret = PGSpectro_Plot(pgs);
return pgsret;
}

/*********************************************************************/
PGSpectro_ret Qjas_fill_angle_t_array(
               float *z,   /* array z[nswp][nang] */
	       int nswp,   /* number. of swps = no. of timetags */
	       int nang,   /* number of angle pts */
	       PEACE_SWEEP **sweeps,  /* pointer to array of nswp sweep ptrs */
               int lev){   /* energy level to be filled */
/********************************************************************/
/* 
*  Fill array z[nx=nswp][ny=nang] from swps psd arrays 
*
*  Note1: z[iswp][jang] = z[nang*iswp + jang]
*
*  Note2: sweep structure has array psd[n_sen][n_lev] so
*         psd[jang][lev] = psd[n_lev*jang + lev]
*/

int iswp, k, jang, nlev;
PEACE_SWEEP *curr_sweep;

if (nang != sweeps[0]->n_sen){
    printf("Qjas_fill_angle_t_array: nang %d != n_sen %d\n",
           nang,sweeps[0]->n_sen);
    return PGSPECTRO_SIZE_ERROR;
}

nlev = sweeps[0]->n_lev;
if (lev > nlev -1 || lev < 0){
    printf("Qjas_fill_angle_t_array: lev %d out of range 0 - %d\n",
           lev,nlev);
    return PGSPECTRO_SIZE_ERROR;
}

for (iswp=0; iswp < nswp; iswp++){
    k = nang*iswp;
    curr_sweep = sweeps[iswp];
    
    for (jang=0; jang<nang; jang++){
       z[k+jang] = curr_sweep->psd[nlev*jang + lev];
       /*if(QJAS_DIAG){
           printf("iswp = %d k = %d  jang = %d z[k+jang] %g\n", 
	         iswp, k, jang, z[k+jang]);
       }*/
    }
    
}

return PGSPECTRO_OK;

}

/***********************************************************************/
PGSpectro_ret malloc_spectro_arrays(PGSpectro *pgs, int nx, int ny){
/***********************************************************************/

int bad_malloc;

bad_malloc=0;

pgs->xbinlow = (float*)malloc(nx*sizeof(float));
  if (pgs->xbinlow == NULL) bad_malloc++;
pgs->xbinhigh= (float*)malloc(nx*sizeof(float));
  if (pgs->xbinhigh == NULL) bad_malloc++;
pgs->ybinlow = (float*)malloc(ny * sizeof(float));
  if (pgs->ybinlow == NULL) bad_malloc++;
pgs->ybinhigh = (float*)malloc(ny * sizeof(float));
   if (pgs->ybinhigh == NULL) bad_malloc++;
pgs->z = (float*)malloc(nx * ny * sizeof(float));
   if (pgs->z == NULL) bad_malloc++;
if (bad_malloc != 0){
   free_spectro_arrays(pgs);
   return PGSPECTRO_MALLOC_FAILURE;
} else {return PGSPECTRO_OK;}

}

/***********************************************************************/
void free_spectro_arrays(PGSpectro *pgs){
/***********************************************************************/
if (pgs->xbinlow != NULL) free(pgs->xbinlow);
if (pgs->xbinhigh != NULL) free (pgs->xbinhigh);
if (pgs->ybinlow != NULL) free (pgs->ybinlow);
if (pgs->ybinhigh != NULL) free (pgs->ybinhigh);
if (pgs->z != NULL) free (pgs->z);
return;
}

/**********************************************************************/

/***********************************************************************/
PGSpectro_ret Sweep_cuts_plot (PEACE_SWEEP **swpp, int nswps, 
                   float *thetas, int ncuts, 
		    SWEEP_CUT_OPTIONS *scutopt
		    ){
/***********************************************************************/
/* Grab various angle cuts from several sweeps and generate an annotated
*  plot.
*
*  Inputs: swpp         arr swpp[nswps] of nswps sweep structure ptrs 
*          nswps        number of sweeps
*          thetas       nth angles to take cuts of sweeps (all sweeps)
*          nth          number of angles
*          scutopt      structure with various linestyle, colour, ... opts 
* 
* No longer used. Use Sweep_n_cuts_plot instead.*/

int iswp, icuts, ndone, k;
BIN_UNITS bu;
PSD_UNITS pu;
PGXYtrace trace;
PEACE_1D_SWEEP_CUT **swpcuts_pos, **swpcuts_neg;
int pgid;
float xmin, xmax, ymin, ymax, temp;
char legend_str[500],xlab[500],ylab[500];
int hr, min, sec, month, day;
long nano;

/* do some preliminary error checking */

if (nswps <= 0 || ncuts <=0){
     printf("Sweep_cuts_plot: zero sweeps or cuts requested!\n");
     return PGSPECTRO_TRACE_ERROR;
}

if(nswps != scutopt->nswps || ncuts != scutopt->ncuts){
     printf("Sweep_cuts_plot: nswps or ncuts differ from those in scutop\n");
     return PGSPECTRO_TRACE_ERROR;
}

bu = swpp[0]->bin_units;
pu = swpp[0]->psd_units;

for (iswp=1; iswp < nswps; iswp++){
   if (swpp[iswp]->bin_units != bu || swpp[iswp]->psd_units != pu){
      printf("Sweep_cuts_plot: bin or psd units for sweep 0 and %d differ\n",
         iswp);
      return PGSPECTRO_TRACE_ERROR;
   }
}

cpgqid(&pgid);
if(DIAG_PRINT_3D)printf("\npgid = %d\n",pgid);
if (!pgid){   /* just in case no device, we'll open one here */
  if (cpgopen(scutopt->pg_device) <=0){
     printf("Failed to open pgplot device successfully\n");
     return PGSPECTRO_TRACE_ERROR;
  } else {
       cpgshls(0,0,1,0);
       cpgshls(1,0,0,0);
       cpgpage();
  }
} 

/* get all the cuts; note the plots will be in FLOW DIRECTION so +theta cut
*  gives particles travelling in positive direction;
*  Up-dated June 2002 when PEACE_SWEEP redefined to be flow dirs */

swpcuts_pos = (PEACE_1D_SWEEP_CUT **) 
          malloc(nswps*ncuts*sizeof(PEACE_1D_SWEEP_CUT*));
swpcuts_neg = (PEACE_1D_SWEEP_CUT **) 
          malloc(nswps*ncuts*sizeof(PEACE_1D_SWEEP_CUT*));

for (iswp = 0; iswp < nswps; iswp++){
  for (icuts = 0; icuts < ncuts; icuts++){
    swpcuts_pos[iswp*ncuts+icuts] = sweep_1D_cut(thetas[icuts],swpp[iswp]);
    swpcuts_neg[iswp*ncuts+icuts] = sweep_1D_cut(180.- thetas[icuts],swpp[iswp]);
  }
}   
   
/* prepare for plotting */

PGXYtrace_set_defaults(&trace); /* initialise lots of things */
trace.xlinlog         = scutopt->xlinlog;
trace.ylinlog         = scutopt->ylinlog;
trace.line_width      = scutopt->linethickness;
trace.box_thickness   = scutopt->linethickness;
trace.text_line_width = scutopt->linethickness;

if (scutopt->xautoscale == YES){
 xmin = swpcuts_neg[0]->ecen[0];
 xmax = swpcuts_neg[0]->ecen[swpcuts_neg[0]->n_lev -1];
 for (ndone = 1; ndone < nswps*ncuts; ndone++){
   if (xmin > swpcuts_neg[ndone]->ecen[0] ) xmin = swpcuts_neg[ndone]->ecen[0];
   if (xmax < swpcuts_neg[ndone]->ecen[swpcuts_neg[ndone]->n_lev -1] ) 
         xmax   = swpcuts_neg[ndone]->ecen[swpcuts_neg[ndone]->n_lev -1];
 }
 scutopt->xmin = xmin;    scutopt->xmax = xmax;  /* return autoscaled vals */
} else {
    xmin = scutopt->xmin;    xmax = scutopt->xmax;
}

if (scutopt->yautoscale == YES){
  k=0;  /* next single line finds 1st non-zero psd value */
  while((ymin = swpcuts_neg[0]->psd1D[k++]) == 0.0 && k<swpcuts_neg[0]->n_lev);
  for (ndone = 0; ndone < nswps*ncuts; ndone++){
    for (k=0; k< swpcuts_neg[ndone]->n_lev; k++){
	temp = swpcuts_neg[ndone]->psd1D[k];
	if (temp != 0.0){
	   ymin = (ymin > temp) ? temp :  ymin ;
	   ymax = (ymax < temp) ? temp :  ymax ;
	}
	temp = swpcuts_pos[ndone]->psd1D[k];
	if (temp != 0.0){
	   ymin = (ymin > temp) ? temp :  ymin ;
	   ymax = (ymax < temp) ? temp :  ymax ;
	}
    }
  }
  scutopt->ymin = ymin;    scutopt->ymax = ymax;  /* return autoscaled vals */
} else {
    ymin = scutopt->ymin;    ymax = scutopt->ymax;
}

/* do positive side 1st */
trace.plot_domain.xworldmin = xmin;    
trace.plot_domain.xworldmax = xmax;    
trace.plot_domain.yworldmin = ymin;    
trace.plot_domain.yworldmax = ymax; 

trace.plot_domain.xndcmin   = 0.35;      /* neg side from 0.1->0.35 */
trace.plot_domain.xndcmax   = 0.6;
trace.plot_domain.yndcmin   = 0.1;
trace.plot_domain.yndcmax   = 0.9;

cpgsvp(0.1, 0.6, 0.1, 0.9);
cpgsch(trace.title_char_size);
cpgslw(trace.text_line_width);
cpgmtxt("T",1.0, 0.5, 0.5, "Cuts of PEACE Sweep Data");
cpgsch(trace.label_char_size);
sprintf(xlab, "%s (%s)",
     BIN_UNITS_STRING(swpcuts_pos[0]->bin_units),"Flow Dir");
cpgmtxt("B", 2.2, 0.5, 0.5, xlab);

trace.legend_domain.xndcmin = 0.65;     /* legend on side */
trace.legend_domain.xndcmax = 0.9;
trace.legend_domain.yndcmin = 0.1;
trace.legend_domain.yndcmax = 0.9;

trace.title = " ";
trace.title_disp = 1.0;                 /* leave titles blank */
trace.title_just = 1.0;
trace.xlabel = xlab;
strcpy(xlab, " ");
trace.YoNYnumlab = NO;
trace.legend_text = legend_str;
trace.YoNlegendbox = NO;  

/* NB extra legend bits not tested; see Sweep_n_cuts... */
ndone = 0;
for (iswp = 0; iswp < nswps; iswp++){
  trace.colour = scutopt->colours[iswp];
  for (icuts = 0; icuts < ncuts; icuts++, ndone++){
    trace.linestyle = scutopt->linestyles[icuts];
    trace.legend_just = 1.0 - (float) (ndone+0.167)/((float) nswps*ncuts);
    if (ndone != 0){ trace.YoNbox = NO; }
    trace.npts = swpcuts_pos[iswp]->n_lev;
    k = iswp*ncuts+icuts;
    trace.x = swpcuts_pos[k]->ecen;
    trace.y = swpcuts_pos[k]->psd1D;
    sprintf(legend_str, "%s" ,swpcuts_pos[k]->lineage->mission);
    PGXYtrace_plot(&trace);
    sprintf(legend_str,swpcuts_pos[k]->lineage->experiment);
    trace.npts=0;
    trace.legend_just = 1.0 - (float) (ndone+0.333)/((float) nswps*ncuts);
    PGXYtrace_plot(&trace);  /* for next line of legend */
    sprintf(legend_str,swpcuts_pos[k]->lineage->instrument);
    trace.legend_just = 1.0 - (float) (ndone+0.5)/((float) nswps*ncuts);
    PGXYtrace_plot(&trace);  /* for next line of legend */
    sprintf(legend_str,swpcuts_pos[k]->lineage->virtual_inst);
    trace.legend_just = 1.0 - (float) (ndone+0.667)/((float) nswps*ncuts);
    PGXYtrace_plot(&trace);  /* for next line of legend */
  }
}   

trace.plot_domain.xndcmin   = 0.1;      /* neg side; reverse world coords */
trace.plot_domain.xndcmax   = 0.35;
trace.plot_domain.xworldmin = xmax;    
trace.plot_domain.xworldmax = xmin;    

trace.title = " ";
trace.YoNYnumlab = YES;
trace.ylabel = ylab;
sprintf(ylab, "%s",PSD_UNITS_STRING(swpcuts_pos[0]->psd_units)); 


ndone = 0;
trace.YoNbox = YES;
for (iswp = 0; iswp < nswps; iswp++){
  trace.colour = scutopt->colours[iswp];
  for (icuts = 0; icuts < ncuts; icuts++, ndone++){
    trace.linestyle = scutopt->linestyles[icuts];
    trace.legend_just = 1.0 - (float) (ndone+0.833)/((float) nswps*ncuts);
    if (ndone != 0){ trace.YoNbox = NO; trace.YoNlegendbox = NO;}
    k = iswp*ncuts+icuts;
    trace.npts = swpcuts_neg[k]->n_lev;
    trace.x = swpcuts_neg[k]->ecen;
    trace.y = swpcuts_neg[k]->psd1D;
    to_hms_time(&(swpcuts_neg[k]->starttime), &hr, &min, &sec, &nano);
    QanYearDoyToDate(swpcuts_neg[k]->starttime.year, 
            swpcuts_neg[k]->starttime.doy, &month, &day);
    sprintf(legend_str, 
         "Cut %4.2g\xB0: %4d-%02d-%02d %02d:%02d:%02d.%03d",
         thetas[icuts],swpcuts_neg[k]->starttime.year, month, day, 
	 hr, min, sec, (int)((float) nano * 1.e-6));              
    PGXYtrace_plot(&trace);
  }
}   


/* free up memory and return */

for (iswp = 0; iswp < nswps; iswp++){
   for (icuts = 0; icuts < ncuts; icuts++){
      free_1D_cut_arrays(swpcuts_pos[iswp*ncuts + icuts]); 
      free(swpcuts_pos[iswp*ncuts + icuts]); 
      free_1D_cut_arrays(swpcuts_neg[iswp*ncuts + icuts]); 
      free(swpcuts_neg[iswp*ncuts + icuts]); 
   }
}
free(swpcuts_neg);
free(swpcuts_pos);
   
return PGSPECTRO_OK;
}

/***********************************************************************/
PGSpectro_ret Sweep_n_cuts_plot (PEACE_SWEEP **swpp, int nswps, 
                   float *thetas, SWEEP_CUT_OPTIONS *scutopt
		    ){
/***********************************************************************/
/* Grab various angle cuts from several sweeps and generate an annotated
*  plot. Note, can repeat sweeps to take more than one cut, but this routine
*  has one theta for each sweep
*
*  Inputs: swpp         arr swpp[nswps] of nswps sweep structure ptrs 
*          nswps        number of sweeps
*          thetas       nswps angles to take cut of corresponding sweeps 
*          scutopt      structure with various linestyle, colour, ... opts 
* */

int iswp, k;
BIN_UNITS bu;
PSD_UNITS pu;
PGXYtrace trace;
PEACE_1D_SWEEP_CUT **swpcuts_pos, **swpcuts_neg;
int pgid;
float xmin, xmax, ymin, ymax, temp;
char legend_str[500],xlab[500],ylab[500];
int hr, min, sec, month, day;
long nano;

/* do some preliminary error checking */

if (nswps <= 0 ){
     printf("Sweep_cuts_plot: zero sweeps requested!\n");
     return PGSPECTRO_TRACE_ERROR;
}

if(nswps != scutopt->nswps ){
     printf("Sweep_cuts_plot: nswps differs from that in scutop\n");
     return PGSPECTRO_TRACE_ERROR;
}

bu = swpp[0]->bin_units;
pu = swpp[0]->psd_units;

for (iswp=1; iswp < nswps; iswp++){
   if (swpp[iswp]->bin_units != bu || swpp[iswp]->psd_units != pu){
      printf("Sweep_cuts_plot: bin or psd units for sweep 0 and %d differ\n",
         iswp);
      return PGSPECTRO_TRACE_ERROR;
   }
}

cpgqid(&pgid);
if(DIAG_PRINT_3D)printf("\npgid = %d\n",pgid);
if (!pgid){   /* just in case no device, we'll open one here */
  if (cpgopen(scutopt->pg_device) <=0){
     printf("Failed to open pgplot device successfully\n");
     return PGSPECTRO_TRACE_ERROR;
  } else {
       cpgshls(0,0,1,0);
       cpgshls(1,0,0,0);
       cpgpage();
  }
} 

/* get all the cuts; note the plots will be in FLOW DIRECTION so +theta cut
*  gives particles travelling in positive direction;
*  Up-dated June 2002 when PEACE_SWEEP redefined to be flow dirs */

swpcuts_pos = (PEACE_1D_SWEEP_CUT **) 
          malloc(nswps*sizeof(PEACE_1D_SWEEP_CUT*));
swpcuts_neg = (PEACE_1D_SWEEP_CUT **) 
          malloc(nswps*sizeof(PEACE_1D_SWEEP_CUT*));

for (iswp = 0; iswp < nswps; iswp++){
    swpcuts_pos[iswp] = sweep_1D_cut(thetas[iswp],swpp[iswp]);
    swpcuts_neg[iswp] = sweep_1D_cut(180.- thetas[iswp],swpp[iswp]);
}   
   
/* prepare for plotting */

PGXYtrace_set_defaults(&trace); /* initialise lots of things */
trace.xlinlog         = scutopt->xlinlog;
trace.ylinlog         = scutopt->ylinlog;
trace.line_width      = scutopt->linethickness;
trace.box_thickness   = scutopt->linethickness;
trace.text_line_width = scutopt->linethickness;
trace.label_char_size = 1.0*(scutopt->yndcmax-scutopt->yndcmin)/0.8;
trace.title_char_size = 1.5*(scutopt->yndcmax-scutopt->yndcmin)/0.8;
trace.legend_text_char_size = 0.7*(scutopt->yndcmax-scutopt->yndcmin)/0.8;

trace.skip_ymissing = YES;   /* ignored for now */
trace.ymissing = swpp[0]->missing_value;

if (scutopt->xautoscale == YES){
 xmin = swpcuts_neg[0]->ecen[0];
 xmax = swpcuts_neg[0]->ecen[swpcuts_neg[0]->n_lev -1];
 for (iswp = 1; iswp < nswps; iswp++){
   if (xmin > swpcuts_neg[iswp]->ecen[0] ) xmin = swpcuts_neg[iswp]->ecen[0];
   if (xmax < swpcuts_neg[iswp]->ecen[swpcuts_neg[iswp]->n_lev -1] ) 
         xmax   = swpcuts_neg[iswp]->ecen[swpcuts_neg[iswp]->n_lev -1];
 }
 scutopt->xmin = xmin;    scutopt->xmax = xmax;  /* return autoscaled vals */
 if (xmin < 0.0) xmin = SAFE_EMIN; if (xmax < 0.0)xmax = SAFE_EMAX;
} else {
    xmin = scutopt->xmin;    xmax = scutopt->xmax;
}

if (scutopt->yautoscale == YES){
  k=0;  /* next single line finds 1st non-zero psd value */
  while((ymin = swpcuts_neg[0]->psd1D[k++]) == 0.0 && k<swpcuts_neg[0]->n_lev);
  for (iswp = 0; iswp < nswps; iswp++){
    for (k=0; k< swpcuts_neg[iswp]->n_lev; k++){
	temp = swpcuts_neg[iswp]->psd1D[k];
	if (temp != 0.0){
	   ymin = (ymin > temp) ? temp :  ymin ;
	   ymax = (ymax < temp) ? temp :  ymax ;
	}
	temp = swpcuts_pos[iswp]->psd1D[k];
	if (temp != 0.0){
	   ymin = (ymin > temp) ? temp :  ymin ;
	   ymax = (ymax < temp) ? temp :  ymax ;
	}
    }
  }
  scutopt->ymin = ymin;    scutopt->ymax = ymax;  /* return autoscaled vals */
} else {
    ymin = scutopt->ymin;    ymax = scutopt->ymax;
}

/* do positive side 1st */
trace.plot_domain.xworldmin = xmin;    
trace.plot_domain.xworldmax = xmax;    
trace.plot_domain.yworldmin = ymin;    
trace.plot_domain.yworldmax = ymax; 

/*trace.plot_domain.xndcmin   = 0.35;  */    /* neg side from 0.1->0.35 */
trace.plot_domain.xndcmin   = 
   scutopt->xndcmin + 0.3125*(scutopt->xndcmax-scutopt->xndcmin);      
/* trace.plot_domain.xndcmax   = 0.6; */
trace.plot_domain.xndcmax   = 
   scutopt->xndcmin + 0.625*(scutopt->xndcmax-scutopt->xndcmin);      
/* trace.plot_domain.yndcmin   = 0.1; 
trace.plot_domain.yndcmax   = 0.9; */
trace.plot_domain.yndcmin   = scutopt->yndcmin; 
trace.plot_domain.yndcmax   = scutopt->yndcmax;

/* cpgsvp(0.1, 0.6, 0.1, 0.9); */
cpgsvp(scutopt->xndcmin, 
   scutopt->xndcmin + 0.625*(scutopt->xndcmax-scutopt->xndcmin),
   scutopt->yndcmin,scutopt->yndcmax);
cpgsch(trace.title_char_size);
cpgslw(trace.text_line_width);
cpgmtxt("T",1.0, 0.5, 0.5, "Cuts of PEACE Sweep Data");
cpgsch(trace.label_char_size);
sprintf(xlab, "%s (%s)",
     BIN_UNITS_STRING(swpcuts_pos[0]->bin_units),"Flow Dir");
cpgmtxt("B", 2.2, 0.5, 0.5, xlab);

/*trace.legend_domain.xndcmin = 0.65;  */   /* legend on side */
/*trace.legend_domain.xndcmax = 0.9;
trace.legend_domain.yndcmin = 0.1;
trace.legend_domain.yndcmax = 0.9;  */
trace.legend_domain.xndcmin =  
  scutopt->xndcmin + 0.6875*(scutopt->xndcmax-scutopt->xndcmin); 
   /* legend on side */
trace.legend_domain.xndcmax = scutopt->xndcmax;
trace.legend_domain.yndcmin = scutopt->yndcmin;
trace.legend_domain.yndcmax = scutopt->yndcmax; 


trace.title = " ";
trace.title_disp = 1.0;                 /* leave titles blank */
trace.title_just = 1.0;
trace.xlabel = xlab;
strcpy(xlab, " ");
trace.YoNYnumlab = NO;
trace.legend_text = legend_str;
trace.YoNlegendbox = NO;  

for (iswp = 0; iswp < nswps; iswp++){
  trace.colour = scutopt->colours[iswp];
  trace.linestyle = scutopt->linestyles[iswp];
  trace.line_width = scutopt->linethicknesses[iswp];
  trace.legend_just = 1.0 - (float) (iswp+0.143)/((float) nswps);
  if (iswp != 0){ trace.YoNbox = NO; }
  trace.npts = swpcuts_pos[iswp]->n_lev;
  trace.x = swpcuts_pos[iswp]->ecen;
  trace.y = swpcuts_pos[iswp]->psd1D;
    sprintf(legend_str, "%s" ,swpcuts_pos[iswp]->lineage->mission);
    PGXYtrace_plot(&trace);
    sprintf(legend_str,swpcuts_pos[iswp]->lineage->experiment);
    trace.npts=0;
    trace.legend_just = 1.0 - (float) (iswp+0.286)/((float) nswps);
    PGXYtrace_plot(&trace);  /* for next line of legend */
    sprintf(legend_str,swpcuts_pos[iswp]->lineage->instrument);
    trace.legend_just = 1.0 - (float) (iswp+0.429)/((float) nswps);
    PGXYtrace_plot(&trace);  /* for next line of legend */
    sprintf(legend_str,swpcuts_pos[iswp]->lineage->virtual_inst);
    trace.legend_just = 1.0 - (float) (iswp+0.571)/((float) nswps);
    PGXYtrace_plot(&trace);  /* for next line of legend */
}   

/* trace.plot_domain.xndcmin   = 0.1;  */ /* neg side; reverse world coords */
/* trace.plot_domain.xndcmax   = 0.35; */
trace.plot_domain.xndcmin   = scutopt->xndcmin;
trace.plot_domain.xndcmax   = 
   scutopt->xndcmin + 0.3125*(scutopt->xndcmax-scutopt->xndcmin);      
trace.plot_domain.xworldmin = xmax;    
trace.plot_domain.xworldmax = xmin;    

trace.title = " ";
trace.YoNYnumlab = YES;
trace.ylabel = ylab;
sprintf(ylab, "%s",PSD_UNITS_STRING(swpcuts_pos[0]->psd_units)); 

trace.YoNbox = YES;
for (iswp = 0; iswp < nswps; iswp++){
  trace.colour = scutopt->colours[iswp];
  trace.linestyle = scutopt->linestyles[iswp];
  trace.line_width = scutopt->linethicknesses[iswp];
  trace.legend_just = 1.0 - (float) (iswp+0.714)/((float) nswps);
  if (iswp != 0){ trace.YoNbox = NO; trace.YoNlegendbox = NO;}
  trace.npts = swpcuts_neg[iswp]->n_lev;
  trace.x = swpcuts_neg[iswp]->ecen;
  trace.y = swpcuts_neg[iswp]->psd1D;
  to_hms_time(&(swpcuts_neg[iswp]->starttime), &hr, &min, &sec, &nano);
  QanYearDoyToDate(swpcuts_neg[iswp]->starttime.year, 
          swpcuts_neg[iswp]->starttime.doy, &month, &day);
  sprintf(legend_str, 
       "Cut %4.2g\xB0: %4.4d-%2.2d-%2.2d %2.2d:%2.2d:%2.2d.%3.3d",
       thetas[iswp],swpcuts_neg[iswp]->starttime.year, month, day, 
       hr, min, sec, (int)((float) nano * 1.e-6));              
  PGXYtrace_plot(&trace);
  to_hms_time(&(swpcuts_neg[iswp]->endtime), &hr, &min, &sec, &nano);
  QanYearDoyToDate(swpcuts_neg[iswp]->endtime.year, 
          swpcuts_neg[iswp]->endtime.doy, &month, &day);
  sprintf(legend_str, 
       "-> %4.4d-%2.2d-%2.2d %2.2d:%2.2d:%2.2d.%3.3d",
       swpcuts_neg[iswp]->endtime.year, month, day, 
       hr, min, sec, (int)((float) nano * 1.e-6));              
  trace.npts=0;
  trace.legend_just = 1.0 - (float) (iswp+0.857)/((float) nswps);
  PGXYtrace_plot(&trace);
}   


/* free up memory and return */

for (iswp = 0; iswp < nswps; iswp++){
      free_1D_cut_arrays(swpcuts_pos[iswp]); 
      free(swpcuts_pos[iswp]); 
      free_1D_cut_arrays(swpcuts_neg[iswp]); 
      free(swpcuts_neg[iswp]); 
}
free(swpcuts_neg);
free(swpcuts_pos);
   
return PGSPECTRO_OK;
}

/*****************************************************************/
PGSpectro_ret Set_Sweep_cut_options_default(SWEEP_CUT_OPTIONS *scopp){
/*****************************************************************/
/* set defaults for most things; 
*  need to fill in nswps,  colours[nswps], ncuts and linestyles[ncuts]
*
*  NB for Sweep_cuts_plot, each sweep is cut ncuts times, but for
*  Sweep_n_cuts_plot, each sweep is cut once and need linestyles[nswps],
*  linethicknesses[nswps], linecols[nswps]*/

scopp->linethickness = 1.0;
scopp->xlinlog = PGSPECTRO_LOG;
scopp->ylinlog = PGSPECTRO_LOG;
scopp->yautoscale = YES;
scopp->xautoscale = YES;
scopp->xmin = scopp->ymin = 1.0;
scopp->xmax = scopp->ymax = 10.0;
scopp->xndcmin = scopp->yndcmin = 0.1;
scopp->xndcmax = scopp->yndcmax = 0.9;
scopp->pg_device = "/xser";

return PGSPECTRO_OK;
}

#ifdef __cplusplus
}
#endif
