/* Qphisc.c ******************
*  Part of QPeace software
*  See README file in QPeace distribution for copyright information
*
*  SJ Schwartz   (s.j.schwartz@qmw.ac.uk)
*  Queen Mary, University of London
*
*  October 2004 First attempt
*
*  Routines for fetching the spacecraft potential and correcting sweep bins
*  Also needs to correct psd matrices
*/

#ifdef __cplusplus
extern "C"{
#endif

#include "Qidfs_utils.h"
#include "Qsweep_spin.h"
#include "Qpeace_pad.h"

/**************************************************************/
void SC_Pot_init(PEACE_SC_POTENTIAL *phisc_s){
/**************************************************************/

phisc_s->phisc = NULL; phisc_s->phisc_time = NULL;
phisc_s->n=0;
phisc_s->dt_gap    = PHI_SC_GAP_SECONDS;
phisc_s->phisc_gap = PHI_SC_GAP_DEFAULT;
strcpy(phisc_s->phisc_lineage.project,"CLUSTERII");
strcpy(phisc_s->phisc_lineage.mission,"Unknown");
strcpy(phisc_s->phisc_lineage.experiment,"EFW");
strcpy(phisc_s->phisc_lineage.instrument,"PRIME");
strcpy(phisc_s->phisc_lineage.virtual_inst,"CEFPRIME");
strcpy(phisc_s->phisc_lineage.exten,"");
phisc_s->phisc_lineage.vsn=0;

}

/**************************************************************/
SDDAS_SHORT SC_Pot_fetch(
    PEACE_SC_POTENTIAL *phisc_s, 
    IDFS_TIME *btime, 
    IDFS_TIME *etime){
/**************************************************************/
/* Go get the spacecraft potential; this is 
*  phisc = probe_to_plasma potential - probe potential (U_sc)
*
*  need to have:
*
*  phisc_lineage, phisc_gap, and dt_gap set prior to entry
*  btime and etime required
*
*  Routine mallocs arrays for phisc and timetags; returns
*
*     ALL_OKAY           on success
*     PHI_SC_EFW_NONFATAL_ERROR  if couldn't get EFW data; 
*                   result is array[2] with  with gap phisc_value at
*                   btime and etime
*     PHI_SC_FATAL_ERROR if didn't malloc arrays
*     PHI_SC_EOF  if ok but eof reached. btime set to last record time
*
*     strips out fillvals
*/


void *idf_creator, *promote_struct;
struct idf_data *idfp;
SDDAS_ULONG dkey;
SDDAS_SHORT ret_val;
int ipromote;
SDDAS_CHAR *tbls_to_apply, num_tables,chk_fill;
SDDAS_SHORT sensor;
SDDAS_LONG *tbl_oper;
SDDAS_LONG fillval;
SDDAS_FLOAT conv_data;
int nfilled,more_data,i;
IDFS_TIME bt,et,dummyt;

IDFS_LINEAGE *lineage;

lineage = &(phisc_s->phisc_lineage);

CfgInit ();
init_idfs();
dbInitialize (); 

get_version_number (&(lineage->vsn));
ret_val = create_idf_data_structure(&idf_creator);
if (ret_val != ALL_OKAY) {
   free_experiment_info();
   return Phisc_fill_not_available(phisc_s,btime,etime);
}
idfp = (struct idf_data *)idf_creator;

ret_val = get_data_key(lineage->project, lineage->mission,
                lineage->experiment, lineage->instrument,
                lineage->virtual_inst, &dkey);

if (ret_val != ALL_OKAY){
   free_experiment_info();
   return Phisc_fill_not_available(phisc_s,btime,etime);
}

for(ipromote=0; ipromote<2; ipromote++){
  promote_struct = pro_InitPromote (
     btime->year, btime->doy, 1000*btime->sec, 
     etime->year, etime->doy, 1000*etime->sec, 
     SHOW_PROMOTE_STATUS_XWIN);
  pro_AddSource (promote_struct, dkey);
  if (pro_NeedToPromote (promote_struct) == sTrue) {
      printf("Trying to promote EFW data: Pass %d\n",ipromote);
      pro_PromoteAll (promote_struct);
      printf("Finished pass %d\n",ipromote);
  }
  pro_FreePromote (promote_struct);
}

ret_val = init_idf_file (*btime,*etime, dkey, idfp, lineage->vsn);
if (ret_val != ALL_OKAY) {
   free_experiment_info();
   return Phisc_fill_not_available(phisc_s,btime,etime);
}

sensor = 15;  /* U_sc probe */
num_tables=0;
tbls_to_apply = NULL;
tbl_oper = NULL;       /* raw units are Volts */

nfilled = 0;
more_data = 1;

while(more_data){   /* dummy loop to count number of values */

  /* read data; forward, and see if filled */
   ret_val = read_drec(dkey,lineage->exten, lineage->vsn,idfp,sensor,1,1);
   if (ret_val == ALL_OKAY || idfp->filled_data){
     set_times_from_idf(&bt,&et,&dummyt,idfp);
     if(compare_idfs_time(&et,etime)!=T2_LATER)more_data=0;
     if (idfp->filled_data)nfilled++;  /* next time, grab data here */
     if(DIAG_PHISC)printf("nfilled = %d\n",nfilled);
   } else {
      if(DIAG_PHISC){
	 printf("read_drec = %d; not ALL_OKAY or idfp not filled\n",ret_val);
      } 
      if ( ret_val == LOS_STATUS || ret_val == NEXT_FILE_STATUS){
	ret_val = next_idfs_file(dkey,*lineage,&bt,etime,idfp);
	if (ret_val != ALL_OKAY){
            more_data=0;
	}
      } else {  /* not LOS, NEXT_FILE, filled_data or ALL_OKAY */
         more_data=0;   /* don't know what else, so force quit */
      }
   }
}

if (nfilled < 2){
   printf("Phi SC 1st pass expects %d which is < 2; nonfatal\n",nfilled);
   free_experiment_info();
   return Phisc_fill_not_available(phisc_s,btime,etime);
}
   
if(DIAG_PHISC)printf("Now get data for real\n");
/* now go really get the data; since there are possibly multiple files;
*  do this by re-setting things */

ret_val = reset_experiment_info(dkey,lineage->exten,lineage->vsn,
       btime->year, btime->doy, btime->sec, btime->nano,
       etime->year, etime->doy, etime->sec, etime->nano);
if (ret_val != ALL_OKAY){
       free_experiment_info();
       printf("Error resetting EFW to get data\n!");
       return PHI_SC_FATAL_ERROR;
}      
ret_val = file_pos (dkey, lineage->exten, lineage->vsn, idfp,        
        btime->year, btime->doy, btime->sec, btime->nano,
        etime->year, etime->doy, etime->sec, etime->nano);
if (ret_val != ALL_OKAY){
       free_experiment_info();
       printf("Error (re-)positioning EFW to get data\n!");
       return PHI_SC_FATAL_ERROR;
}	

if(DIAG_PHISC)printf("Experiment reset and file repositioned\n");

phisc_s->phisc = malloc (nfilled*sizeof(float));
phisc_s->phisc_time = malloc (nfilled*sizeof(IDFS_TIME));

ret_val = read_idf(dkey,lineage->exten,lineage->vsn, &chk_fill, 
                             _FILL_FLG, 0, 0, 1);
if(chk_fill==1){
   ret_val = read_idf(dkey,lineage->exten, 
                        lineage->vsn, &fillval, _FILL, 0, 0, 1);
} else {fillval = -2019483744;}   /* cut/pasted from vidf just in case */

i = 0;   /* will count values */
more_data = 1;

while(more_data && i < nfilled){   /* real loop to get values */

  /* read data; forward, and see if filled */
   ret_val = read_drec(dkey,lineage->exten, lineage->vsn,idfp,sensor,1,1);
   if (ret_val == ALL_OKAY || idfp->filled_data){
     set_times_from_idf(&bt,&et,&dummyt,idfp);
     if(compare_idfs_time(&et,etime)!=T2_LATER)more_data=0;
     if (idfp->filled_data){     
        ret_val = convert_to_units(dkey, lineage->exten, lineage->vsn, idfp, 
            sensor, SENSOR , 0, num_tables, tbls_to_apply,  tbl_oper, 
	    &conv_data, chk_fill, fillval );
        if (conv_data > QOUTSIDE_MIN && ret_val == ALL_OKAY){ /*remove fillval*/
           phisc_s->phisc[i] = PROBE2PLASMA - conv_data;
	   phisc_s->phisc_time[i] = bt;    
           i++;
	}
     }
   } else {
      if(DIAG_PHISC){
	 printf("read_drec = %d; not ALL_OKAY or idfp not filled\n",ret_val);
      } 
      if ( ret_val == LOS_STATUS || ret_val == NEXT_FILE_STATUS){
	ret_val = next_idfs_file(dkey,*lineage,&bt,etime,idfp);
	if (ret_val != ALL_OKAY){
            more_data=0;
	}
      } else {  /* not LOS, NEXT_FILE, filled_data or ALL_OKAY */
         more_data=0;   /* don't know what else, so force quit */
      }
   }
}
   
 
if (i<2){
   printf("Phi SC got less than 2 pts, but expected %d;\n",nfilled);
   printf("Probably all fillvals, so return minimal array and hope!\n");
   free_experiment_info();
   free_phisc_arrays(phisc_s);   /* get re-malloced in next line */
   return Phisc_fill_not_available(phisc_s,btime,etime);
} else {
   phisc_s->n = i;   /* actual number filled */
   ret_val = ALL_OKAY;
}
free_experiment_info();
return ret_val;

}



/**************************************************************/

SDDAS_SHORT Phisc_fill_not_available(PEACE_SC_POTENTIAL *phisc,
    IDFS_TIME *btime, IDFS_TIME *etime){
/**************************************************************/

/* There has been an error trying to get EFW data, so fill in minimalist 
*  way to keep going. */

phisc->phisc      = malloc (2*sizeof(float));
phisc->phisc_time = malloc (2*sizeof(IDFS_TIME));

phisc->phisc[0] = phisc->phisc[1] = phisc->phisc_gap;
phisc->phisc_time[0] = *btime;
phisc->phisc_time[1] = *etime;
phisc->n = 2;

return PHI_SC_EFW_NONFATAL_ERROR;

}

/**************************************************************/
SDDAS_SHORT free_phisc_arrays(PEACE_SC_POTENTIAL *phisc){
/**************************************************************/
/* free up internal arrays in a phisc structure */
if (phisc->phisc != NULL) free(phisc->phisc);
if (phisc->phisc_time != NULL) free(phisc->phisc_time);
phisc->n = 0;
phisc->phisc = NULL; phisc->phisc_time = NULL;
return ALL_OKAY;
}

void phisc_dump(PEACE_SC_POTENTIAL *phisc){
/* dump out contents of phisc structure */
int i,n;

printf("PEACE SC Potential Structure\n-----------------------\n");
print_idfs_lineage(&(phisc->phisc_lineage));
printf("dt_gap = %g secs    phisc_gap = %g Volts\n",
                 phisc->dt_gap, phisc->phisc_gap);
printf("%d records in structure\n",phisc->n);		 
n=phisc->n;

for(i=0; i<n; i++){
    print_idfs_time(&(phisc->phisc_time[i]));
    printf("  %g eV\n",phisc->phisc[i]);	 
}		 

return;
}		 
		 
/**************************************************************/
SDDAS_SHORT phisc2sweep(PEACE_SC_POTENTIAL *phisc, PEACE_SWEEP *swpp, 
   int *start){
/**************************************************************/
/* find the value of the spacecraft potential for a sweep and put it in
*  the sweep structure. This routine does NOT correct the bins or psd for
*  the spacecraft potential.
*
*  Inputs 
*
*     filled phisc structure, includes gap handling params; 
*             nb time come from pp and so are centre times. 
*
*     sweep pointer      below we find the centre time of the sweep
*
*     start       index where to start searching.
*
*  Algorithm: step through phisc array in dumb way to locate appropriate value,
*  starting with the *start entry in the phisc array. Assuming the start value
*  is reused, this should usually be quicker than a blind binary search
*
*  On return start is set to entry in phisc used, or nmax if no
*  not found or gap.
*
*/
int i, nmax;
IDFS_TIME *ptimes, sbtime;
float *phis;       /* local copies of arrays and things*/
float dtgap, phigap,phi1, phi2;
double t1phisecs,t2phisecs,tswpsecs, t1swp, t2swp;

ptimes    = phisc->phisc_time;
t1swp     = idfs_time_to_epoch_secs(&(swpp->btime));
t2swp     = idfs_time_to_epoch_secs(&(swpp->etime));
tswpsecs  = (t1swp + t2swp)/2.0;
epoch_secs_to_idfs_time(tswpsecs,&sbtime);
phis      = phisc->phisc;
nmax      = phisc->n;
dtgap     = phisc->dt_gap;
phigap    = phisc->phisc_gap;

if (*start > nmax-1 || *start < 0) {
   swpp->phi_sc = phigap;
   *start = nmax;
   return PHI_SC_NOT_FOUND;
}

/* increment until find time in phisc later than sweep start time */
for (i=*start; i<nmax && compare_idfs_time(&(ptimes[i]),&sbtime)!= T1_LATER; i++);

if (i==nmax)  {      /* got to end of phisc array */
    if ( tswpsecs -         /* check if within dtgap*/
         idfs_time_to_epoch_secs(&(ptimes[nmax-1])) < dtgap){
             swpp->phi_sc = phis[nmax-1];         /* yes? use last pt */
	     *start = nmax -1;
	     return ALL_OKAY;
    }
    /* maybe we started too late; search from 0->start */
    for (i=0; i<*start && compare_idfs_time(&(ptimes[i]),&sbtime)!= T1_LATER; i++);
    if (i==*start){   /* no, nothing found; incl case start =0*/  
	swpp->phi_sc = phigap; 
	*start = nmax;
	return PHI_SC_NOT_FOUND;
    }   /* else we found one in 0->start range, so continue */
}

*start     = i;
t2phisecs = idfs_time_to_epoch_secs(&(ptimes[i]));
phi2      = phis[i];

if(*start == 0){   /* handle start of array separately */
   if (t2phisecs-tswpsecs > dtgap){  /* data gap at beginning */
	swpp->phi_sc = phigap; 
	*start = nmax;
	return PHI_SC_NOT_FOUND;
    } else {
	swpp->phi_sc = phi2;  /* constant at ends */
	return ALL_OKAY;
    }
}

t1phisecs = idfs_time_to_epoch_secs(&(ptimes[i-1]));
phi1      = phis[i-1];
   
/* ok, now t1 and t2 should bracket tswp */

if (t2phisecs-t1phisecs < dtgap){  /* normal event */
   swpp->phi_sc = phi1+(phi2-phi1)*(tswpsecs-t1phisecs)/(t2phisecs-t1phisecs);
   return ALL_OKAY;
} else if (t2phisecs-tswpsecs < dtgap){ /* use phi2 */
    swpp->phi_sc = phi2;
    return ALL_OKAY;
} else if ( t1phisecs-tswpsecs < dtgap){ /* use phi1 */
    swpp->phi_sc = phi1;
    return ALL_OKAY;
} else {
   swpp->phi_sc = phigap; 
   *start = nmax;
   return PHI_SC_NOT_FOUND;
}  

}

/**************************************************************/
SDDAS_SHORT phisc2sweep_all(PEACE_SC_POTENTIAL *phisc, PEACE_SWEEP **swppp, int
nswp){
/**************************************************************/
/* Fill all nswp sweeps with s/c potential from phisc. Just a loop to call
*  the routine which does one */

int start,i;
SDDAS_SHORT ret;

if (DIAG_PHISC) phisc_dump(phisc);

for (start = 0, i=0; i < nswp; i++){
   ret = phisc2sweep(phisc,swppp[i],&start);
   if (ret == PHI_SC_FATAL_ERROR){
      printf("phisc2sweep returned fatal error for sweep %d at ",i);
      print_idfs_time(&(swppp[i]->btime)); printf("\n");
      return ret;
   }   
   if (DIAG_PHISC){
      printf("Sweep %d at btime = ",i);
      print_idfs_time(&(swppp[i]->btime));
      printf(" got phisc = %g\n",swppp[i]->phi_sc);
   }
}

return ALL_OKAY;

}

/**************************************************************/
SDDAS_SHORT sweep_sc_potential_adjust(PEACE_SWEEP **swppp,int nswps){
/**************************************************************/
/* correct nswps for spacecraft potential. The values of the s/c pot
*  need to be inserted into the sweeps, e.g., by phisc2sweep_all, prior
*  to calling this routine. Algorithm is:
*
*  Check if bin_units are velocity or energy and branch accordingly
*
*  For each energy bin:
*     Find energy mid-point of old bin
*     Correct bin limits by subtracting s/c potential
*     If Elo < 0 set all psd's for this bin to missing value
*     IF psd_units are differential energy flux:
*         correct psd as:  psd(new) = psd(old) * (Emid_new/Emid_old)^2
*
*  Notes
*
*     For psd = counts, psd(new)=psd(old) preserves total counts
*     For psd = f(v),   psd(new) = psd(old) follows from Liouville's Thm
*     For psd = def,    above alg follows from Liouville's Thm as
*                       def propto E^2
*     pitch angle array unchanged, as it is unaffected by energy
*
*     I have checked the use of mid-pt energies by comparing them with the
*     centre energies in the VIDFs. The difference between the two is less
*     than 1% above 10eV, and less than 10% below 10eV.
*/

int iswp, ie, ith, ne, nth;
PEACE_SWEEP *curswp;
float missing, emidold, emidnew, el, eh, phi;
float *psdlocal;


for (iswp = 0; iswp < nswps; iswp++){
   curswp = swppp[iswp];
   nth    = curswp->n_sen;
   ne     = curswp->n_lev;
   missing= curswp->missing_value;
   phi    = curswp->phi_sc;
   
   for (ie = 0; ie < ne; ie++){
      el = curswp->elo[ie];
      eh = curswp->ehi[ie];
      switch (curswp->bin_units) {
      
      case  BIN_UNITS_KM_S :         /* if E<0, add_... returns neg number */
          emidold = 0.5*(eh*eh+el*el);   /* units of v^2 equiv to energy */
          curswp->elo[ie] = el = add_ev2_kms(el,-phi);
	  curswp->ehi[ie] = eh = add_ev2_kms(eh,-phi);
	  emidnew = 0.5*(eh*eh+el*el); 
          break;
      case BIN_UNITS_EV :
          emidold = 0.5*(eh + el);
	  curswp->elo[ie] = el  -= phi;
	  curswp->ehi[ie] = eh  -= phi;
	  emidnew = 0.5*(eh+el);
	  break;
      default: 
          printf("Unknown bin units in sweep_sc_potential_adjust");
	  return PHI_SC_FATAL_ERROR; 
      }	
      if (el < 0.0){
         sweep_make_level_missing(curswp,ie);
      } else if (curswp->psd_units == PSD_UNITS_DIFF_ENERGY_FLUX){
         /*def needs to have psd corrected */
	 psdlocal = curswp->psd;
	 for (ith = 0; ith < nth; ith++){
	    if (psdlocal[ne*ith + ie] != missing){
	       psdlocal[ne*ith + ie] *= (emidnew*emidnew)/(emidold*emidold);
       	    }
	 }
      }   
   }  /* end for over energy levels */   
   if (strlen(curswp->lineage->virtual_inst)<LINEAGE_MAX_STR -7){
	 strcat(curswp->lineage->virtual_inst,"_scpot");
   }
}


return ALL_OKAY;
}


/**************************************************************/
float add_ev2_kms(float kms, float ev){
/**************************************************************/
/* convert km/s to eV, add ev to it, convert back to km/s */

float e;

e = V_KMS_2_E_EV_FAC * kms * kms + ev;

if (e>0)return E_EV_2_V_KMS_FAC * sqrt(e);
else return NEGATIVE_ENERGY2KMS_ERROR;   /* a fixed negative number */

}

/**************************************************************/
void sweep_make_level_missing(PEACE_SWEEP *swpp, int ilev){
/**************************************************************/
/* set all psd <=> energy level lev to missing value */
/* recall psd[jth][ilev] = psd[n_lev*jth+ilev] */

float missing, *psdlocal;
int jth, nth, nlev;

missing = swpp->missing_value;
nth     = swpp->n_sen;
nlev    = swpp->n_lev;
psdlocal=swpp->psd;

for (jth = 0; jth < nth; jth++)psdlocal[nlev*jth+ilev]=missing;

return;
}


#ifdef __cplusplus
}
#endif
