/* Qsweep_spin_read.c ******************
*  Part of QPeace software
*  See README file in QPeace distribution for copyright information
*
*  SJ Schwartz   (s..schwartz@imperial.ac.uk)
*  Imperial College London
*
*  May  2002 separate out from Qsweep_spin.c
*  June 2002 Resort polar bins to be flow and cope with SPINPAD reverse order
*  July 2002 Complete flow <-> look adjustments
*  Mar  2004 Duplicate promote loop to update database and data
*  Feb  2005 V2 re-write to simplify and make robust against doublestar
*       Main areas changed include:
*          * sweeps in spins each have their own bins
*          * spins chopped from sequence of sweeps (no fixed # swps/spin)
*          * go through once first just to count sweeps (no fixed swp duration) 
* 
*
*  Routines for reading PEACE sweeps and spins from idfs
*
*/

#ifdef __cplusplus
extern "C"{
#endif

#include "Qidfs_utils.h"
#include "Qsweep_spin.h"
#include "Qpeace_pad.h"

/*********************************************************************/
SDDAS_SHORT fill_sweep(PEACE_SWEEP *swpp, IDFS_LINEAGE *lngp,
              struct idf_data *idfp, SDDAS_ULONG dkey, 
	      T_OR_F have_bins, T_OR_F do_pa){
/*********************************************************************/

/* fill a sweep of PEACE data for all sensors. [If already have the
*  energy/theta bin boundaries and lineage, don't get them again.
*  (in practice, for spins this routine used to be passed the sweep number, 
*  zero being the 1st bin); deprecated as of V2, but left in call for
*  backward compatibility.] Expect have_bins to always be passed as QPFALSE
*
*  Need to set bin/psd units and lineage prior to calling this routine
*  and position file to start of spin. Also need to set options in paopts
*  if convert to pitch angles sought and have missing value set.
*
*  swpp will come in with n_sen set (got from vidf) and n_lev 
*  (from idf->num_sample), and will have all arrays malloced.
*/

int  isen, isen_spad, nsen, nlev, npsd, i, is_spinpad, is_rpad, first_filled;
SDDAS_CHAR fwd, full_swp;
SDDAS_SHORT ret_val;
static int swp_step0, swp_step1;
IDFS_TIME dummytime;
float tspin, missing;

is_spinpad = strncmp("SPINPAD",lngp->instrument,7);
if (is_spinpad==0) is_spinpad=1; else is_spinpad = 0;
is_rpad = strncmp("RPAD",lngp->instrument,4);
if (is_rpad ==0) is_rpad= 1;  else is_rpad = 0;

nsen = swpp->n_sen;
/* nlev = swpp->n_lev;
npsd = nsen*nlev;  */

/* find filled sensor, but don't forward file as done later in routine */
ret_val = read2filled_sensor(dkey,lngp,idfp,nsen,&first_filled,QPFALSE);
if (ret_val != ALL_OKAY){
   printf("fill_sweep failed to read2filled_sensor returning %d\n",ret_val);
   return ret_val;
}
/* use returned info in idfp to transfer into sweep */
nlev = swpp->n_lev = idfp->num_sample;
npsd = nsen*nlev;

/* depricated; malloc arrays here rather than in GetNSweeps...
if (idfp->num_sample != swpp->n_lev){
   printf("PEACE3D: n_level check fail\n");
   printf("   idfp->num_sample = %d, n_lev expected = %d\n",
              idfp->num_sample, swpp->n_lev);
   return SWEEP_NLEVEL_CHANGE;
}
*/

     ret_val = malloc_sweep_arrays(swpp, do_pa);  /* moved from GetNSweep */
     if (ret_val != ALL_OKAY){
	  printf("malloc_sweep_arrays failed\n");
 	  return  PEACE3D_ERROR;
     }


if (!have_bins){    /* need  e-bin arrays+lineage; use sen 0*/
   swp_step0 = idfp->swp_data[0];  /* static vars to hold 1st couple of */
   swp_step1 = idfp->swp_data[1];  /* sweep steps as mode change check */
   ret_val = e_bin_edges (dkey, idfp, lngp->vsn, swpp->elo, swpp->ehi,
                           swpp->bin_units, swpp->n_lev);
   if (ret_val != ALL_OKAY){
       printf("PEACE3D: e_bin_edge fail\n");return PEACE3D_ERROR;
   }
   if(!is_rpad){   /* rpad not inverted */
	ret_val = invert_array(swpp->elo, swpp->n_lev);
	if(ret_val!=ALL_OKAY){
	printf("PEACE3D: invert e_bin fail\n");return PEACE3D_ERROR;
	}
	ret_val = invert_array(swpp->ehi, swpp->n_lev);
	if(ret_val!=ALL_OKAY){
	printf("PEACE3D: invert e_bin fail\n");return PEACE3D_ERROR;
	}
   }
   ret_val = check_bins (swpp->elo, swpp->ehi, swpp->n_lev);/*finite bins*/
   if (ret_val != ALL_OKAY){
       printf("PEACE3D: check_bins failed\n"); return PEACE3D_ERROR;
   }
   copy_lineage(swpp->lineage,lngp);
}

full_swp = 1;
missing = swpp->missing_value;

for (i=0; i<npsd; i++)swpp->psd[i] = missing;   /* initialise all */

   
for (isen = 0, isen_spad = nsen -1; isen < nsen; isen++, isen_spad--){
   fwd =  (isen < nsen -1 ) ?   0 : 1 ;
   ret_val = read_drec (dkey,lngp->exten, lngp->vsn, idfp, isen,fwd, full_swp);
   if (ret_val < 0 ) {  
      printf ("\nError %d from read_drec reading sensor %d\n",
             ret_val, isen);
      return ret_val;   /* changed 19/6/2001 from PEACE3D_ERROR */
                        /* for GetNPeaceSweeps to trap NEXT_FILE_STATUS */
   }
   if (!have_bins){         /* fill theta arrays sensor by sensor           */
      if (is_spinpad){ 
			    /* SPINPAD returns flow directions, ordered */
                            /* 180->0, so fill sensors in reverse order.    */
			    /* These will then work with                    */
			    /* re-binning, etc. which assumes ascending bins*/
	 swpp->thlo[isen_spad] = idfp->start_theta;
	 swpp->thhi[isen_spad] = idfp->stop_theta;
      } else if (is_rpad) { /* rpad returns FLOW directions, ordered 0->180 */
			    /* so fill sensros in order                     */
	 swpp->thlo[isen] = idfp->start_theta;
	 swpp->thhi[isen] = idfp->stop_theta;
      } else {
           		    /* other Virtuals return LOOK direction, ordered*/
                            /* 180->0, so fill sensors in order but flip    */
			    /* angles; NB start -> 180 - stop, etc.         */
	 swpp->thlo[isen] = 180.0-idfp->stop_theta;
	 swpp->thhi[isen] = 180.0-idfp->start_theta;
      }
   }

   if (idfp->filled_data){   
             /* no data for this sensor; skip - NB init'ed to missing above*/
       if (DIAG_SWEEP_IDF){print_idf_data(idfp);}
       if (idfp->num_sample != swpp->n_lev){
	  printf("PEACE3D: n_level check fail\n");
	  printf("   idfp->num_sample = %d, n_lev expected = %d\n",
                     idfp->num_sample, swpp->n_lev);
	  return SWEEP_NLEVEL_CHANGE;
       }
       /*************************************
       * fill sweep psd
       *****************************************/
       if (swp_step0 != idfp->swp_data[0] ||
	   swp_step1 != idfp->swp_data[1] ||
	   swpp->n_lev != idfp->num_sample){
	   printf("PEACE3D: fill sweep psd lev check fail\n");
	   return PEACE3D_ERROR;
       }
       ret_val = fill_single_sen_sweep(swpp,isen,dkey,idfp, do_pa);
	 /* NB fill_single_sen_sweep uses isen to fill nsen-1-isen for spinpad */
       if (ret_val!= ALL_OKAY){
	   printf("PEACE3D: fill_single_sweep error\n");
	   return PEACE3D_ERROR;
       }
   }
}                       /* end for loop over sensors */

   /***************************
   * fill btime, etime and azimuths
   ****************************/
if (is_spinpad){  /* for Spinpad use spin start/stop; not for RPAD */
   set_times_from_idf(&dummytime,&dummytime,&(swpp->suntime),idfp);
   swpp->etime = swpp->btime= swpp->suntime;
   tspin = idfp->spin_rate/1000.;
   swpp->etime.sec  = swpp->btime.sec + 1.e-9*swpp->btime.nano + tspin ; 
	                                          /*trunc tspin to int*/
   swpp->etime.nano = 1000000000*(swpp->btime.sec +
	         1.e-9*swpp->btime.nano+tspin -swpp->etime.sec);                  
} else {
   set_times_from_idf(&(swpp->btime),&(swpp->etime),&(swpp->suntime),idfp);
}
   /* az range for full swp; change from look to flow and mod 360 */
swpp->phi_lo = mod360(idfp->start_az[0]+180.0);
swpp->phi_hi = mod360(idfp->stop_az[swpp->n_lev -1]+180.0);
swpp->phi_sc = PHI_SC_GAP_DEFAULT;  /* initialise */  

/* if psd units to fetch 1-count, reset it and modify lineage 
*  Resetting is kind to plotting software */
switch(swpp->psd_units){
   case PSD_UNITS_CNTS_ACCUM_1CNT:  
   case PSD_UNITS_DIFF_ENERGY_FLUX_1CNT:
   case PSD_UNITS_S3_PER_KM6_1CNT:
     if(strlen(swpp->lineage->experiment)<LINEAGE_MAX_STR-4){
          strcat(swpp->lineage->experiment,"1cnt");
     }  /* modify lineage for all 3 1CNT psd units */
   default: break;
}
switch(swpp->psd_units){
   case PSD_UNITS_CNTS_ACCUM_1CNT:  
        swpp->psd_units=PSD_UNITS_CNTS_ACCUM;        break;
   case PSD_UNITS_DIFF_ENERGY_FLUX_1CNT:
        swpp->psd_units= PSD_UNITS_DIFF_ENERGY_FLUX; break; 
   case PSD_UNITS_S3_PER_KM6_1CNT:
        swpp->psd_units = PSD_UNITS_S3_PER_KM6;      break;     
   default: break;
}

return ALL_OKAY;
}

/**********************************************************************/
SDDAS_SHORT fill_single_sen_sweep(PEACE_SWEEP *swpp, int isen,
               SDDAS_ULONG dkey, struct idf_data *idfp, 
	       T_OR_F do_pa){
/**********************************************************************/

/* Get sweep data and, if requested, pa data from sensor isen
*  Fill swpp[isen_to_fill] where isen_to_fill is isen for most 
*      virtual instruments, but reverse (nsen -1 -isen) for SPINPAD */

SDDAS_LONG tbl_oper[MAX_TABLES_TO_APPLY];
SDDAS_CHAR num_tbls, tbls_to_apply[MAX_TABLES_TO_APPLY],chk_fill;
SDDAS_LONG fillval;
int is_rpad;

SDDAS_SHORT ret_val;

float conv_data[MAX_LEVELS],temp,safeoutsidemin,localmissing;
int j,n, nlev, isen_to_fill,k;

is_rpad = strncmp("RPAD",swpp->lineage->instrument,4);
if(is_rpad == 0) is_rpad=1; else is_rpad = 0;

if(strncmp("SPINPAD",swpp->lineage->instrument,7)==0) {
   isen_to_fill = swpp->n_sen -1 - isen;
} else {
   isen_to_fill = isen;
}
nlev = swpp->n_lev;

ret_val = set_psd_tables(swpp->lineage,swpp->psd_units, &num_tbls,
                     tbls_to_apply, tbl_oper);
if (ret_val != ALL_OKAY){
   printf("PEACE3D: Couldn't set psd tables in fill_single_sweep\n");
   return PEACE3D_ERROR;
}

/* go get the fillval used (if any) as defined in the vidf 
*  Should use new ReadVIDF routines, but oh well...
*  Would also be more efficient to do this outside, rather than here
*  in a very inner loop, but that would require either using a global var
*  or passing an extra argument.
*
*  The _FILL_FLG and other params aren't well-documented
*
*  SJS added Jan 2005 to help double star data handling
*  Jul 2006 added 1cnt trap to force all data treated as real
*/

switch(swpp->psd_units){
   case PSD_UNITS_CNTS_ACCUM_1CNT:
   case PSD_UNITS_DIFF_ENERGY_FLUX_1CNT:
   case PSD_UNITS_S3_PER_KM6_1CNT:
     chk_fill = 0;  break;
   default:
	ret_val = read_idf(dkey,swpp->lineage->exten,swpp->lineage->vsn, &chk_fill, 
				_FILL_FLG, 0, 0, 1);
	if(chk_fill==1){
	ret_val = read_idf(dkey,swpp->lineage->exten, 
				swpp->lineage->vsn, &fillval, _FILL, 0, 0, 1);
	} else {
	chk_fill = 1;   /* force convert_to_units to check */
	fillval = QP_DEFAULT_MISSING;
	}   /* i.e. treat zero as missing */
	
	/* raw values = fillval get replaced by idfs OUTSIDE_MIN = -3.4e38 */
    break;
}
if(DIAG_FILL_SINGLE_SEN){
	printf("fill_single_sensor: raw values for sensor %d\n", isen);
		for(k=0; k<idfp->num_sample; k++)
		{
			if(k % 6 == 0) printf("\n");
			printf("%10ld ", idfp->sen_data[k]);	
		}
	printf("\n dkey = %d extension %s version %d   ",dkey, 
		swpp->lineage->exten,swpp->lineage->vsn);
	printf("chk_fill= %d, fillval =  %ld\n",chk_fill, fillval);
	printf("Applying %d tables\nTable Oper\n",num_tbls);
	for (k=0; k<num_tbls; k++){
	printf("%d   %ld\n",tbls_to_apply[k],tbl_oper[k]);
	}
}


ret_val = convert_to_units (dkey,swpp->lineage->exten,
              swpp->lineage->vsn, idfp, isen, SENSOR, 0,
              num_tbls, tbls_to_apply, tbl_oper, conv_data, chk_fill, fillval);
if (ret_val != ALL_OKAY){
    printf("PEACE3D: convert_to_units returned error %d\n", ret_val);
    return PEACE3D_ERROR;
}

if(DIAG_FILL_SINGLE_SEN){
	printf("\nConverted Values:\n");
	for(k=0; k<idfp->num_sample; k++)
	{
		if(k % 6 == 0) printf("\n");
		printf("%10.2e ", conv_data[k]);
	}
}

           /***********************************************
           * copy data in inverse order into psd array unless rpad. Note
           * swpp->psd is [n_sen][n_lev] array so for sensor isen, data is
           * at psd[n_lev * isen + 0] thru [n_lev * isen + (n_lev -1)]
           *************************************************/

safeoutsidemin = 0.95*OUTSIDE_MIN;
localmissing = swpp->missing_value;

switch(is_rpad){
  case 1:     /* rpad; fill in order */
	for (j=nlev*isen_to_fill, n=0; n<nlev; n++){
		temp = conv_data[n];
		swpp->psd[j+n] = (temp < safeoutsidemin) ? localmissing : temp;
		if(DIAG_FILL_SINGLE_SEN_DEEP){
			printf("j = %d, n=%d, temp = %g, conv_data[n] = %g, swpp->psd[j]=%g\n",
			j,n,temp,conv_data[n],swpp->psd[j]);
		}
	}
	break;

  case 0: default:
	for (j=nlev*isen_to_fill, n=nlev -1; n>=0; j++,n--){
		temp = conv_data[n];
		swpp->psd[j] = (temp < safeoutsidemin) ? localmissing : temp;
		if(DIAG_FILL_SINGLE_SEN_DEEP){
			printf("j = %d, n=%d, temp = %g, conv_data[n] = %g, swpp->psd[j]=%g\n",
			j,n,temp,conv_data[n],swpp->psd[j]);
		}
	}
	break;
}

/* won't check for fill vals here; perhaps I should ?*/
if (do_pa){/* if pitch angles requested, do now. Just like psd really */
  if (idfp->num_pitch != nlev){
    printf("Pitch angles requested but idfp->num_pitch = %d != n_lev = %d\n",
                     idfp->num_pitch, nlev);
    return QPEACE_PITCH_ANGLE_ERROR;
  } 
  ret_val = convert_to_units (dkey,swpp->lineage->exten,
              swpp->lineage->vsn, idfp, isen, PITCH_ANGLE, 0,
              0, tbls_to_apply, tbl_oper, conv_data, 0, 0);
  if (ret_val != ALL_OKAY){
      printf("PEACE3D: convert_to_units returned error %d\n", ret_val);
      return PEACE3D_ERROR;
  }
  for (j=nlev*isen_to_fill, n=nlev -1; n>=0; j++,n--){
      swpp->pa[j] = conv_data[n];
  }
}
return ALL_OKAY;
}


/******************************************************************/
SDDAS_SHORT GetPeaceSpin(IDFS_LINEAGE *lngp, IDFS_TIME *btimep,
   BIN_UNITS bin_units, PSD_UNITS psd_units, T_OR_F do_pa, 
   PEACE_SPIN3D *spinp){
/******************************************************************/
/*  Get a complete spin of PEACE 3D data. This module is self-contained
*   and deals with all IDFS calls. It needs to be passed the pointer
*   to a declared spin structure, but it mallocs all the necessary
*   internal storage required. free_spin_data will free all such
*   memory, but not the spin structure itself.
*
*   Inputs are self-evident from their declarations
*   Output is filled spin structure
*/
/*  Now re-written. Algorithm is:
*
*   Find start of spin
*   Find start of next spin, counting sweeps in process
*   Reset, then just call GetNPeaceSweeps
*/

void *idf_creator, *promote_struct;
struct idf_data *idfp;
SDDAS_ULONG dkey, pa_dkey;
SDDAS_SHORT ret_val,pa_status,nsen_sd;
IDFS_TIME etime, btime,suntime;
float tspin;
int nsen, nsen_used, iswp, nswp, nswpinspin;
int ipromote;
PEACE_SWEEP **swppp;

btime = *btimep;
etime = btime;
add_secs_to_idfs_time(&etime,PEACE_SPIN_PERIOD);/* to init for promote */

CfgInit ();
dbInitialize ();
init_idfs();

get_version_number (&(lngp->vsn));

ret_val = create_idf_data_structure(&idf_creator);
if (ret_val != ALL_OKAY){
     printf("Error %d in create idf struct for sweep\n", ret_val);
     free_experiment_info(); return ret_val;
}
idfp = (struct idf_data *)idf_creator;

ret_val = get_data_key(lngp->project, lngp->mission,
                lngp->experiment, lngp->instrument,
                lngp->virtual_inst, &dkey);
if (ret_val != ALL_OKAY){
       printf("Error getting data key\n");
       free_experiment_info(); return ret_val; 
}
/* kludge here, because pitch_angle_source_status requires parent file_open,
*  so can't easily then do promote */
if(do_pa){
   if(strncmp(lngp->project,"CLUST",5)==0){
      ret_val = get_data_key(lngp->project, lngp->mission,
                      "FGM", "VEC","CFUNIT", &pa_dkey);
   } else if (strncmp(lngp->project,"DOUBLEST",8)==0){
      ret_val = get_data_key(lngp->project, lngp->mission,
                      "FGM", "VEC","DFUNIT", &pa_dkey);
   } else {
      ret_val = PEACE3D_ERROR;
   } 	   
   if (ret_val != ALL_OKAY){
	  printf("Error getting pa_data key\n");
	  free_experiment_info();
	  return PEACE3D_ERROR;
   }
}

/* do a promote if needed; twice */

for(ipromote=0; ipromote<2; ipromote++){
  promote_struct = pro_InitPromote (
     btime.year, btime.doy, 1000*btime.sec, 
     etime.year, etime.doy, 1000*etime.sec, 
     SHOW_PROMOTE_STATUS_XWIN);
  pro_AddSource (promote_struct, dkey);
  /* kludge here, because pitch_angle_source_status requires parent file_open,
  *  so can't easily do promote afterwards. Only other option I think is
  *  to read pa_inst lineage directly from VIDF; this would be more generic.*/
  if(do_pa){  pro_AddSource (promote_struct, pa_dkey);}

  if (pro_NeedToPromote (promote_struct) == sTrue) {
      printf("Trying to promote data: Pass %d\n",ipromote);
      pro_PromoteAll (promote_struct);
  }
  pro_FreePromote (promote_struct);
}

ret_val = init_idf_file (btime,etime, dkey, idfp, lngp->vsn);
if(ret_val!=ALL_OKAY){
   printf("Error init_idf_file\n");
   free_experiment_info(); return ret_val;
}

if(do_pa){  /* check status for pa data if requested; must do after file_open */
   ret_val = pitch_angle_source_status(
           dkey,lngp->exten,lngp->vsn,&pa_status, &pa_dkey);
   if ( ret_val!= ALL_OKAY || pa_status != PA_READY ){
	printf("Pitch angle source error: return val = %d; pa_status = %d",
        	ret_val, pa_status);
	printf(" Ignoring pitch angle request\n");
	do_pa = QPFALSE;
   }
} 

/* find number of sensors */
ReadVIDF(dkey, btime.year, btime.doy, btime.sec, btime.nano,
              etime.year,  etime.doy,  etime.sec,  etime.nano,
	      (SDDAS_CHAR *) &nsen_sd, _SEN, 0, 0, -1);  /* in case nsen changes */
nsen = nsen_sd;  /* coerce to int */

ret_val = read2filled_sensor(dkey,lngp,idfp,nsen, &nsen_used, QPTRUE);
if (ret_val != ALL_OKAY){
   printf("GetPeaceSpin failed to find filled sensor\n");
   free_experiment_info();
   return PEACE3D_ERROR;
}

set_times_from_idf(&btime,&etime,&suntime,idfp);
tspin = idfp->spin_rate/1000.;

free_experiment_info();  /* tidy up */

/* let GetNPeaceSweeps do all the work. Set start time to suntime;
*  end time to suntime + 2spins (to be safe), then chop after */

btime = etime = suntime;
add_secs_to_idfs_time (&etime,2.0*tspin);

ret_val = GetNPeaceSweeps(lngp,&btime,&etime,bin_units,psd_units,do_pa,
                   &nswp, &swppp);
if (ret_val != ALL_OKAY){
       printf("Error GetPeaceSpin failed to GetNPeaceSweeps\n");
       free_experiment_info();
       return PEACE3D_ERROR;
}
if (nswp < 1){
       printf("GetPeaceSpin got nswp = %d sweeps; out of range\n",nswp);
       free_experiment_info();
       return PEACE3D_ERROR;
}

btime = swppp[0]->suntime;
iswp = 1;
while ( compare_idfs_time(&(swppp[iswp++]->suntime),&btime) == SAME_TIME && 
        iswp < nswp-1);
nswpinspin = iswp-1;

/* there are nswp sweeps in this spin */

spinp->sweeppp = (PEACE_SWEEP **)malloc(nswpinspin*sizeof (PEACE_SWEEP *));
for (iswp=0; iswp<nswpinspin; iswp++)spinp->sweeppp[iswp] = NULL; /* safety */
ret_val = ALL_OKAY;
for (iswp=0; iswp < nswpinspin; iswp++){
   spinp->sweeppp[iswp]= create_sweep_full_copy(swppp[iswp]);
   if(spinp->sweeppp[iswp] == NULL){
      ret_val = PEACE3D_ERROR;
      break;
   }
}

free_n_sweeps(nswp,swppp); free(swppp); swppp=NULL;
if (ret_val == ALL_OKAY){
   spinp->tspin      = tspin;
   spinp->btime      = btime;
   spinp->etime      = spinp->sweeppp[nswpinspin-1]->etime;
   spinp->n_swp = nswpinspin;
   return ALL_OKAY;
} else {
   printf("GetPeaceSpin failed to fill spin\n");
   free_spin_data(spinp);
   return ret_val;
}

}



/********************************************************************/
SDDAS_SHORT GetNPeaceSpins(
   IDFS_LINEAGE *lngp, 
   IDFS_TIME *btime,
   IDFS_TIME *etime,
   BIN_UNITS bin_units, 
   PSD_UNITS psd_units, 
   T_OR_F do_pa,
   int *nspins, 
   PEACE_SPIN3D ***spinppp){
/******************************************************************/

/*  Get all PEACE data within times specified.
*   Put each spin in a PEACE_SPIN3D structure and return an array of
*   pointers which points to them.
*
*   This module is self-contained
*   and deals with all IDFS calls. It returns the pointer
*   to an array of spin structure pointers, and mallocs all the necessary
*   internal storage required, including the pointer array.
*
*   Inputs are self-evident from their declarations
*   Output is malloced and filled pointer array (and the nspins spin
*   structures to which they point!).
*/

/* re-write by chopping up sequence of sweeps */ 

PEACE_SWEEP **swppp;
PEACE_SPIN3D **spinpp;
int nspin,nswp,nswpinspin, iswp, istart,iend,ispin,i;
SDDAS_SHORT ret;
double tspin;
IDFS_TIME_COMPARE tcomp;

ret=GetNPeaceSweeps(lngp,btime,etime,bin_units,psd_units,do_pa,&nswp,&swppp);
if (ret != ALL_OKAY){
   printf("GetNPeaceSpins failed to get sweeps ret = %d,nswp=%d\n",ret,nswp);
   return PEACE3D_ERROR;
}

if (DIAG_GET_SPIN){
   printf("GetNPeaceSpins got %d sweeps using GetNPeaceSweeps\n",nswp);
   printf("first sweep is\n");
   print_sweep_structure(swppp[0]);
   printf("last sweep is\n");
   print_sweep_structure(swppp[nswp-1]);
}   


nspin = 1;    /* lets count how many spins there are */
tspin = 0.0;
for (iswp =0; iswp < nswp -1; iswp++){
   if (compare_idfs_time(&(swppp[iswp]->suntime),
                  &(swppp[iswp+1]->suntime)) != SAME_TIME ) {
	 nspin++;
	 if (nspin > 1){  /* through away 1st; may be incomplete */
              tspin += idfs_time_to_epoch_secs(&(swppp[iswp+1]->suntime)) -
		  idfs_time_to_epoch_secs(&(swppp[iswp]->suntime));
	 }	  
   }		 
   if (DIAG_GET_SPIN){
      printf("Sweep %d Sun: ",iswp);
      print_idfs_time(&(swppp[iswp]->suntime));
      printf("  tspin = %g\n",tspin);
   }
}	       
if (nspin == 1){
  tspin = idfs_time_to_epoch_secs(&(swppp[nswp-1]->etime)) -
		          idfs_time_to_epoch_secs(&(swppp[0]->suntime));
} else {			  
  tspin = tspin/(double)(nspin);   /* calc average time betw suntimes */
}  
spinpp = (PEACE_SPIN3D **)malloc(nspin * sizeof (PEACE_SPIN3D *));
for (ispin=0; ispin < nspin; ispin++)spinpp[ispin]=NULL;

iend = -1;  /* initialise */
for (ispin=0,i=0; i< nspin; i++){
   iswp = istart = ++iend;
   if (istart == nswp) break;  /* hit end of spin */
   tcomp = SAME_TIME;
   while ( tcomp == SAME_TIME && iswp < nswp-1){
         tcomp = compare_idfs_time(&(swppp[iswp]->suntime),
	             &(swppp[iswp+1]->suntime) ); 
		     iswp++;
   }	 
   iend = iswp;
   nswpinspin = iend-istart+1;
   spinpp[ispin] = (PEACE_SPIN3D *)malloc(sizeof(PEACE_SPIN3D));
   spinpp[ispin]->sweeppp = (PEACE_SWEEP **)malloc(nswpinspin*sizeof (PEACE_SWEEP *));
   for (iswp=0; iswp<nswpinspin; iswp++)spinpp[ispin]->sweeppp[iswp] = NULL; /* safety */
   ret= ALL_OKAY;
   for (iswp=0; iswp < nswpinspin; iswp++){
      spinpp[ispin]->sweeppp[iswp]= create_sweep_full_copy(swppp[iswp+istart]);
      if(spinpp[ispin]->sweeppp[iswp] == NULL){
	 ret = PEACE3D_ERROR;
         if (DIAG_GET_SPIN){
	  printf("GetNPeaceSpins found bad spin/null sweep:");
	  printf(" iswp = %d, nswpinspin=%d,istart=%d\n",
	                iswp,nswpinspin,istart);
	  printf("Sweep %d Sun: ",iswp+istart);
	  print_idfs_time(&(swppp[iswp+istart]->suntime));
	 }
	 break;
      }
   }
   if (ret == ALL_OKAY){
      spinpp[ispin]->tspin = tspin;
      spinpp[ispin]->btime = spinpp[ispin]->sweeppp[0]->suntime;
      spinpp[ispin]->etime = spinpp[ispin]->sweeppp[nswpinspin-1]->etime;
      spinpp[ispin]->n_swp = nswpinspin;
      if (DIAG_GET_SPIN){
         printf("GetNPeaceSpins Spin %d:",ispin);
	 print_peace_spin_info(spinpp[ispin]);
	 for (iswp = 0; iswp < spinpp[ispin]->n_swp; iswp++){
	   printf("Spin %d sweep %d:\n",ispin,iswp);
	   print_sweep_structure(spinpp[ispin]->sweeppp[iswp]);
	 }
      }  
      ispin++;
   } else {   /* not a good spin for some reason; free up and continue */
      free_spin_data (spinpp[ispin]); 
      free(spinpp[ispin]); 
      spinpp[ispin]=NULL;  
   }
}

if (ispin != nspin){
   printf("GetNPeaceSpins got %d out of %d spins\n",ispin,nspin);
}   

*nspins = ispin;  /* actual number of good spins. If any others, they will */
                  /* be empty with null pointers, so only memory is pointer*/
		  /* which will get freed when spinpp is freed */
free_n_sweeps(nswp,swppp); free(swppp); swppp=NULL;

if(ispin == 0) {
  printf("GetNPeaceSpins got zero spins\n");
  free_n_spins(spinpp,nspin);
  free(spinpp); spinpp=NULL;
  return PEACE3D_ERROR;
} else {
   *spinppp = spinpp;  /* pass back ptr */
   return ALL_OKAY;
}
}

/***************************************************************************/
PEACE_SWEEP *PeaceGetAzSlice(
       IDFS_LINEAGE *lineage, IDFS_TIME *btime, float az,
       BIN_UNITS bin_units,
       PSD_UNITS psd_units,
       PEACE_PITCH_ANGLE_OPTS *paopts){
/**************************************************************************/
/* peace_az_slice routine to grab an az slice. This is historial; use
* V2 now, which has more inputs, hence this routine left intact for 
* backward compatibility.
*
*      Inputs:
*        idfs lineage for sweeps [incl ext but not vsn]
*        time sweeps
*        azimuth for sweeps
*
*      Returns pointer to complete sweep structure
*      If PAD requested, return that in a sweep structure
*/

PEACE_SPIN3D spin_data, *spin_datap;
SDDAS_SHORT ret_val;
PEACE_SWEEP *swp=NULL, *swppa = NULL, **swppap=NULL;

int iswp;

if (strcmp(lineage->instrument,"PAD") == 0 &&
    strcmp(lineage->virtual_inst,"CPPADAL") != 0){ /* CPPADAL not treated as PAD */
  swp = (PEACE_SWEEP *)malloc(sizeof(PEACE_SWEEP));
  if (swp == NULL){
     printf("Failed to malloc sweep structure\n");
     return NULL;
  }
  nullify_sweep_arrays(swp);
  ret_val = GetPeacePAD(lineage,btime,bin_units,psd_units,paopts->do_pa, swp);
  if (ret_val != ALL_OKAY){
     printf("GetPeacePAD returned error %d\n",ret_val);
     if (swp != NULL) {free_sweep_arrays(swp);free(swp);}
     return NULL;
  }
  if (paopts->do_pa){
      ret_val=rebin_sweep_to_pa(&swp,1,paopts,&swppa);
      if (DIAG_PLOTAZ_REBIN_PA){
	 printf("GetAzSlice PAD got\n");
	 print_sweep_structure(swp);
	 printf("Rebinned in pitch angle to\n");
	 print_sweep_structure(swppa);
      }
      if (ret_val != ALL_OKAY || swppa == NULL){
         printf("Rebinning PAD to pa failed; returning orig. PAD\n");
	 if (swppa!=NULL){ free_sweep_arrays(swppa); free(swppa);}
	 return swp;
      } else {
         free_sweep_arrays(swp); free(swp);
	 return swppa;
      }      
  } else {
      return swp;
  } 
} else {
  ret_val = GetPeaceSpin(lineage,btime,bin_units,psd_units,
                                     paopts->do_pa, &spin_data);
  if (ret_val != ALL_OKAY){
    printf("Failed to get 1st spin\n");
    return NULL;
  }
  if (paopts->do_pa){
     spin_datap = &spin_data; /* rebin called to fill 1 swp */
     ret_val = Rebin_n_spins_to_pa(&spin_datap,1,paopts, &swppap);
     if (ret_val != ALL_OKAY || *swppap == NULL){
         printf("Rebinning spin sweeps to pa failed\n!");
	 if (swppap!=NULL){ free_sweep_arrays(*swppap); free(*swppap);}
	 return NULL;
     } else {
         free_spin_data(&spin_data);
	 return *swppap;
     }
  } else { 
      iswp = find_az_sweep(&spin_data, az);
      if (iswp < 0){
	printf("requested az %g not found in spin\n", az);
	free_spin_data(&spin_data);
	return NULL;
      }
      swp = create_sweep_full_copy(spin_data.sweeppp[iswp]);
      free_spin_data(&spin_data);
      return swp;
  }
}
/* outermost braces for routine */
/* shouldn't get here !*/
printf("ERROR: Control got to end of PEACEGetAzSlice without returning\n");
return NULL;

}

/************************************************************************/
PEACE_SWEEP *PeaceGetAzSliceV2(
       IDFS_LINEAGE *lineage, IDFS_TIME *btime, IDFS_TIME *etime, float az,
       BIN_UNITS bin_units, PSD_UNITS psd_units,
       PEACE_PITCH_ANGLE_OPTS *paopts,
       PHISC_CHOICE phiscch,
       float phisc_value, float phisc_gapsecs,
       PEACE_GATHER_CHOICE gatch, PEACE_GATHER_GAPS gaps, 
       float gath_secs, int gath_swp_spins){
/************************************************************************/

/* High level PEACE Sweep selection routine to replace original version.
*  Offers choice to correct for s/c potential, rebin in pa and gather sweeps
*  or spins. */

PEACE_SWEEP *swp, *binh;
IDFS_TIME savetime;

PEACE_SWEEP **swppp, **swppp_pa, **gath_swppp;
int nswp, ispinstart,ispinend, istart, iaz, ngat, nswp_pa, i, nsen;
PEACE_PITCH_ANGLE_OPTS single_paopts;

PEACE_SPIN3D spin;
float *lobins, *hibins;

SDDAS_SHORT ret;

swppp = swppp_pa = gath_swppp = NULL;

savetime = *etime;

if (gatch == NO_GATHER){ /* 1 spin needed */
      *etime = *btime;
      add_secs_to_idfs_time( etime,4.0*PEACE_SPIN_PERIOD); /* 4=safe? */   
}  /* otherwise use times passed in to routine */

switch (phiscch){
   case NO_SC_POT_CORR:
          ret = GetNPeaceSweeps(lineage,btime,etime,bin_units,psd_units,
               paopts->do_pa, &nswp, &swppp);
          break;
       case DO_SC_POT_CORR :  /* not const s/c pot */
          ret = GetNPotCorrectedPeaceSweeps(
	        lineage,btime,etime,bin_units,psd_units,paopts->do_pa, 
		QPFALSE, phisc_value,
		phisc_gapsecs,
		&nswp, &swppp);
	  break;
       case CONST_SC_POT :
          ret = GetNPotCorrectedPeaceSweeps(
	        lineage,btime,etime,bin_units,psd_units,paopts->do_pa, 
		QPTRUE, phisc_value,
		phisc_gapsecs,
		&nswp, &swppp);
	  break;
       default:
          printf("Unknown SC Potential Choice");
	  ret = PEACE3D_ERROR;
	  break;
}
if (ret != ALL_OKAY){
   printf("PeaceGetAzSliceV2 failed to get sweeps ret= %d.",ret);
   printf(" Try another source/time?\n");
   *etime = savetime;
   return NULL;
}        


/* set up bins for rebinning, but don't do it yet */
/* Note pa-rebinning requires congruent sweeps: same levs and sensors */
/* Note rebin_n_sweeps does NOT copy pitch angles ! */
/* So one approach is:
*
*  Rebin each sweep (one by one) into pitch angles - at initial angle resol.
*  Then Gather/combine onto final bin spec:
*/

if (paopts->do_pa == QPTRUE){   /* 1st rebin each sweep to pa; old bins */
   single_paopts = *paopts;
   single_paopts.n_pabins = swppp[0]->n_sen;
   single_paopts.mu_bins  = QPFALSE;    /* keep as close as poss to orig*/

   ret = Rebin_n_sweeps_to_pa(swppp, nswp, 1, &single_paopts,
             &swppp_pa,&nswp_pa);
   if (ret != ALL_OKAY){
      printf("Plotaz failed to rebin each sweep in pa, ret = %d\n",ret);
   }
   nswp = nswp_pa;
   free_n_sweeps(nswp,swppp); free(swppp);   /* shuffle to orig ptr*/ 
   swppp=swppp_pa;   swppp_pa = NULL;
}

binh = create_sweep_full_copy(swppp[0]); /* use existing levels:*/
if (paopts->do_pa == QPTRUE){
    free(binh->thlo); free(binh->thhi);
    nsen = binh->n_sen = paopts->n_pabins;
    lobins = binh->thlo = (float *)malloc(nsen*sizeof(float));
    hibins = binh->thhi = (float *)malloc(nsen*sizeof(float));
    if (paopts->mu_bins == QPTRUE){
       set_equal_bins( lobins, hibins, nsen, 1.0, -1.0);
       for (i=0; i<nsen; i++){
	  lobins[i] = RAD2DEG*acos( lobins[i]);
	  hibins[i] = RAD2DEG*acos( hibins[i]);
       }
       lobins[0] = 0.0;  /* force to elim rounding error */
       hibins[nsen -1] = 180.0;
    } else {
       set_equal_bins (lobins, hibins, nsen, 0.0, 180.0);
    }
}

ispinstart = iend_sweep_gather(swppp,nswp,0,GATHER_SPINS,gaps, gath_secs, 1);
ispinstart++;  /* start of spin */

if (gatch == NO_GATHER){ /*select az from spin */
    ispinend = iend_sweep_gather(swppp,nswp,ispinstart,GATHER_SPINS, 
                           gaps,gath_secs, 1);
    if (ispinstart < 0 || ispinend < 0){
        printf("Failed to locate spin\n"); 	   
       *etime = savetime;
        free_n_sweeps(nswp,swppp); free(swppp);
        return NULL;
    }    
    spin.btime = swppp[ispinstart]->btime;  /* construct spin struct */
    spin.etime = swppp[ispinend]  ->etime;
    spin.tspin = PEACE_SPIN_PERIOD;  /* not needed */
    spin.n_swp  = ispinend-ispinstart + 1;
    spin.sweeppp = &(swppp[ispinstart]);
    
    iaz = find_az_sweep (&spin, az);
    
    if (iaz < 0){
       swp = NULL;
    }else {
       swp = create_sweep_full_copy(swppp[ispinstart+iaz]);
    }
} else {   /* go gather up as requested */
    if (gatch == GATHER_SPINS) istart = ispinstart;
    else istart = 0;    /* if user req spins, use spin start */
    ret = gather_sweeps(&(swppp[istart]), nswp-istart, binh,
         gatch, gaps, gath_secs, gath_swp_spins, &gath_swppp, &ngat);
    if (ret != ALL_OKAY){
        swp = NULL;
    } else {
        swp = create_sweep_full_copy(gath_swppp[0]);
	free_n_sweeps(ngat,gath_swppp); free(gath_swppp);
    }
/*    free_sweep_arrays(binh); free(binh); */ /* moved outside jul 2008 */
}

free_sweep_arrays(binh); free(binh);

free_n_sweeps(nswp,swppp); free(swppp);


*etime = savetime;

return swp;

}



/******************************************************************/
SDDAS_SHORT GetNPeaceSweeps(
        IDFS_LINEAGE *lngp, 
	IDFS_TIME *btimep,
        IDFS_TIME *etimep,
	BIN_UNITS bin_units, 
	PSD_UNITS psd_units,
	T_OR_F do_pa,
	int *nswps, 
	PEACE_SWEEP ***swpppp){
/******************************************************************/
/*  Get all PEACE sweep data within times specified. 
*   Put them each in a PEACE_SWEEP structure and return an array of
*   pointers which points to them. Each sweep structure includes its
*   own copies of bin boundaries and lineage.
*   
*   This module is self-contained
*   and deals with all IDFS calls. It returns the pointer
*   to an array of sweep structure pointers, and mallocs all the necessary
*   internal storage required, including the pointer array. 
*
*   Inputs are self-evident from their declarations
*   Output is malloced and filled pointer array (and the nsweeps, sweep 
*   structures to which they point!). 
* 
*   If do_pa = QPTRUE, pitch angle array is filled in sweep structure.
*   This routine does NOT call rebin_sweep_to_pa
*
*   Feb 2005 added next_file_status handing inside this.
*   Mar 2005 added PAD treatment:
*       If no pa rebinning requested, use SPINPAD
*       If pa rebinning requested, just fetch sweeps. fill_sweep modified
*           to find filled sensor, and to fill all unfilled ones with
*           missing vals.
*/

void *idf_creator, *promote_struct;
struct idf_data *idfp;
SDDAS_ULONG dkey, pa_dkey;
SDDAS_SHORT ret_val,pa_status,nsen_sd;
IDFS_TIME etime, btime, lasttime;
PEACE_SWEEP *curswp, **swppp;
IDFS_LINEAGE lngpad;
int i, iswp, nswp;   /* nswp is local copy of *nswps */
int nsen, nsen_used, nlev;
int ipromote;

btime = *btimep;
etime = *etimep;

/* Trap out PAD request; if no pa rebinning use SPINPAD; else just fetch
*  as fill_sweep now finds only filled sensors, and inserts missing into
*  others */

if (strcmp(lngp->instrument,"PAD")==0 && 
   strcmp(lngp->virtual_inst,"CPPADAL") != 0 &&  /* CPPADAL not spinpad */
   do_pa != QPTRUE){
    /* use spinpad instead of old GetNPeacePAD routine */
  copy_lineage(&lngpad,lngp);
  strcpy(lngpad.instrument,"SPINPAD");
  if (strcmp(lngp->virtual_inst,"CPPADL")==0) {
     strcpy(lngpad.virtual_inst,"LSPAD");
  } else if (strcmp(lngp->virtual_inst,"CPPADH")==0) {
        strcpy(lngpad.virtual_inst,"HSPAD");
  } else {
     printf("PAD requested, but not CPPADL or CPPADH\n");
     return PEACE3D_ERROR;
  }
  return GetNPeaceSweeps(&lngpad,btimep,etimep,    /* recursive, but ok */
            bin_units,psd_units,do_pa, nswps,swpppp);   
}


if(DIAG_PRINT_3D){
  printf("3D bin units requested: %d <=> %s\n", 
             bin_units,BIN_UNITS_STRING(bin_units));
  printf("3D psd units requested: %d <=> %s\n", 
             psd_units,PSD_UNITS_STRING(psd_units));
	     
}

CfgInit ();
dbInitialize ();  
init_idfs();

get_version_number (&(lngp->vsn));

ret_val = create_idf_data_structure(&idf_creator);
if (ret_val != ALL_OKAY){
     printf("Error %d in create idf struct for sweep\n", ret_val);
     free_experiment_info(); return ret_val;
}
idfp = (struct idf_data *)idf_creator;

ret_val = get_data_key(lngp->project, lngp->mission,
                lngp->experiment, lngp->instrument,
                lngp->virtual_inst, &dkey);
if (ret_val != ALL_OKAY){
       printf("Error getting data key\n");
       free_experiment_info(); return ret_val; 
}
/* kludge here, because pitch_angle_source_status requires parent file_open,
*  so can't easily then do promote */
if(do_pa){
   if(strncmp(lngp->project,"CLUST",5)==0){
      ret_val = get_data_key(lngp->project, lngp->mission,
                      "FGM", "VEC","CFUNIT", &pa_dkey);
   } else if (strncmp(lngp->project,"DOUBLEST",8)==0){
      ret_val = get_data_key(lngp->project, lngp->mission,
                      "FGM", "VEC","DFUNIT", &pa_dkey);
   } else {
      ret_val = PEACE3D_ERROR;
   } 	   
   if (ret_val != ALL_OKAY){
	  printf("Error getting pa_data key\n");
	  free_experiment_info();
	  return PEACE3D_ERROR;
   }
}

/* do a promote if needed; twice */

for(ipromote=0; ipromote<2; ipromote++){
  promote_struct = pro_InitPromote (
     btime.year, btime.doy, 1000*btime.sec, 
     etime.year, etime.doy, 1000*etime.sec, 
     SHOW_PROMOTE_STATUS_XWIN);
  pro_AddSource (promote_struct, dkey);
  /* kludge here, because pitch_angle_source_status requires parent file_open,
  *  so can't easily do promote afterwards. Only other option I think is
  *  to read pa_inst lineage directly from VIDF; this would be more generic.*/
  if(do_pa){  pro_AddSource (promote_struct, pa_dkey);}

  if (pro_NeedToPromote (promote_struct) == sTrue) {
      printf("Trying to promote data: Pass %d\n",ipromote);
      pro_PromoteAll (promote_struct);
  }
  pro_FreePromote (promote_struct);
}

ret_val = init_idf_file (btime,etime, dkey, idfp, lngp->vsn);
if(ret_val!=ALL_OKAY){
   printf("Error init_idf_file\n");
   free_experiment_info(); return ret_val;
}

if(do_pa){  /* check status for pa data if requested; must do after file_open */
   ret_val = pitch_angle_source_status(
           dkey,lngp->exten,lngp->vsn,&pa_status, &pa_dkey);
   if ( ret_val!= ALL_OKAY || pa_status != PA_READY ){
	printf("Pitch angle source error: return val = %d; pa_status = %d",
        	ret_val, pa_status);
	printf(" Ignoring pitch angle request\n");
	do_pa = QPFALSE;
   }
} 

nswp = idfs_sweep_counter(dkey, lngp, idfp, btimep, etimep);
if(do_pa){   /* need to re-open pa_inst as reset_experiment_info closes it*/
    free_experiment_info(); /* can't seem to get right pa, so restart! */

    CfgInit ();
    dbInitialize ();
    init_idfs();

    ret_val = create_idf_data_structure(&idf_creator);
    if (ret_val != ALL_OKAY){
	 printf("Error %d in create idf struct for sweep\n", ret_val);
	 free_experiment_info(); return ret_val;
    }
    idfp = (struct idf_data *)idf_creator;

    ret_val = init_idf_file (btime,etime, dkey, idfp, lngp->vsn);
    if(ret_val!=ALL_OKAY){
       printf("Error init_idf_file\n");
       free_experiment_info(); return ret_val;
    }

    ret_val = pitch_angle_source_status(
            dkey,lngp->exten,lngp->vsn,&pa_status, &pa_dkey);
    if ( ret_val!= ALL_OKAY || pa_status != PA_READY ){
	 printf("Pitch angle source error: return val = %d; pa_status = %d",
        	 ret_val, pa_status);
	 printf(" Ignoring pitch angle request\n");
	 do_pa = QPFALSE;
    } 
}

if(DIAG_PRINT_3D){printf("Expecting at most %d sweeps\n",nswp);}
if (nswp < 1){
   printf("GetNPeaceSweeps nsweeps = %d out of range\n", nswp);
   free_experiment_info(); return PEACE3D_ERROR;
}
swppp= (PEACE_SWEEP**)malloc(nswp*sizeof(PEACE_SWEEP*));
if (swppp == NULL){
   printf("GetNPeaceSweeps failed to malloc swppp with %d sweep ptrs\n",nswp);
   free_experiment_info();  return PEACE3D_ERROR;
}

*swpppp = swppp;  /* pass the address of the ptr array back */

for (i=0; i<nswp; swppp[i++]=NULL);  /* initialise ptrs to NULL */

btime =  *btimep;  /* local copies */
lasttime = *etimep;
iswp = 0;

while ((compare_idfs_time(&btime,&lasttime)!=T1_LATER )&&
     (ret_val == ALL_OKAY && iswp < nswp)){
  ReadVIDF(dkey, btime.year, btime.doy, btime.sec, btime.nano,
              lasttime.year,  lasttime.doy,  lasttime.sec,  lasttime.nano,
	      (SDDAS_CHAR *) &nsen_sd, _SEN, 0, 0, -1);  /* in case nsen changes */
  nsen = nsen_sd;  /* coerce to int */
  while ( ret_val != NEXT_FILE_STATUS &&    /* trap for end of file */
       (compare_idfs_time(&btime,&lasttime)!=T1_LATER && iswp < nswp )){

    /* get number of energy levels*/
     nsen_used = nsen;

     nlev = idfs_sweep_nlevels(dkey, lngp, idfp, &nsen_used);
     if (nlev < 1){
        printf("GetNPeaceSweeps nlev = %d < 0 error\n",nlev);
	ret_val = SWEEP_NLEVELS_NOT_FOUND;
	break;
     }     		    
     curswp = swppp[iswp] = (PEACE_SWEEP*)malloc(sizeof(PEACE_SWEEP));
     if (curswp == NULL){
        printf("GetNPeaceSweeps failed to malloc sweep structure\n");
	ret_val = PEACE3D_ERROR;
	break;
     }
     nullify_sweep_arrays(curswp);  /* set ptrs inside swp to NULL for safety*/
     
     curswp->n_sen = nsen;
/*     curswp->n_lev = nlev;  now set in fill_sweep before malloc sjs july 2007 */
     curswp->bin_units = bin_units;
     curswp->psd_units = psd_units;
     curswp->phi_frame = PEACE_TZERO;

/* move malloc to fill_sweep
     ret_val = malloc_sweep_arrays(curswp, do_pa);
     if (ret_val != ALL_OKAY){
	  printf("malloc_sweep_arrays failed\n");
 	  ret_val = PEACE3D_ERROR;
	  break;
     }
*/
     
     curswp->missing_value = QP_DEFAULT_MISSING;   /* set defaults */
     curswp->phi_sc    = PHI_SC_GAP_DEFAULT;    

     /* fill sweep. penult. arg is have_bins: QPFALSE (0) gets bins & lineage*/
     ret_val=fill_sweep(curswp, lngp, idfp, dkey, QPFALSE, do_pa);
     if(DIAG_GET_N_SWEEPS){
	print_sweep_structure(curswp);
     }
 
     if(DIAG_PRINT_3D){
        printf("GetNPeaceSweeps iswp = %d\n",iswp);
        print_sweep_structure(curswp);
     }
     if(ret_val == RAN_OFF_END_FILE_ERR){
	free_sweep_arrays(curswp);  free(curswp);
	printf("ran of file end; incomplete sweep started at: ");
	print_idfs_time(&btime);printf("\n");
	ret_val = NEXT_FILE_STATUS;
     } else {
	    btime = curswp->btime;
            iswp++;
     } 
  } /* end while trap for next file */     
  if(ret_val == NEXT_FILE_STATUS){
     ret_val = next_idfs_file(dkey,*lngp,&btime,&lasttime,idfp); 		    
  }
}  /* end while loop gathering sweeps */


if (ret_val != ALL_OKAY || iswp <= 0 || iswp > nswp){
    printf("GetNPeaceSweeps Error after gathering while: ");
    printf(" Returned %d with iswp =%d out of %d expected\n", 
          ret_val, iswp,nswp);
    printf("Current spin start time = ");
    print_idfs_time(&btime);printf("\n");
    if (ret_val == NEXT_FILE_STATUS && iswp > 0 && iswp <= nswp){ 
        printf("NEXT_FILE_STATUS reached\n");
    } else {  
        free_n_sweeps(nswp, swppp); *swpppp = NULL;
	free_experiment_info();
        return PEACE_PAD_ERROR;
    }
}
*nswps = iswp;  /* set to actual number of sweeps filled; this is
                   also the number of sweep structures malloced, 
		   although the size of the pointer array could be
		   bigger */

free_experiment_info();
return ret_val;

}

/******************************************************************/
SDDAS_SHORT GetNPotCorrectedPeaceSweeps(
        IDFS_LINEAGE *lngp, 
	IDFS_TIME *btimep,
        IDFS_TIME *etimep,
	BIN_UNITS bin_units, 
	PSD_UNITS psd_units,
	T_OR_F do_pa,
	T_OR_F const_phisc,
	float phisc_value, float phisc_gapsecs,
	int *nswps, 
	PEACE_SWEEP ***swpppp){
/******************************************************************/
/* Get all peace sweeps in the requested interval and correct their bins
*  for s/c potential. Inputs as for GetNPeaceSweeps with addition of:
*
*   phisc_value   s/c pot (in V) to be used if EFW data not available, either
*                 because there is a data gap or for doublestar, etc.
*   phisc_gapsecs how big a gap in secs should be considered a real gap for
*                 phisc. Gaps larger than this are filled with phisc_value,
*                 smaller ones are linearly interpolated.
*
*  This is a wrappter routine for GetNPeaceSweeps and the Phisc routines.
*  It returns the sweeps and frees up all Phisc memory. The value of phisc used
*  is returned in each sweep
*/

SDDAS_SHORT ret;
PEACE_SC_POTENTIAL phisc;

ret = GetNPeaceSweeps( lngp, btimep, etimep, bin_units, psd_units, do_pa,
             nswps, swpppp);
if (ret != ALL_OKAY){
   printf("GetNPotCorrectedPeaceSweeps: GetNPeaceSweeps returned %d\n",ret);
   return ret;	     
}

SC_Pot_init(&phisc);  /* sets up; assumes lineage is Cluster EFW */
strcpy(phisc.phisc_lineage.mission, lngp->mission);  /* get s/c right */

phisc.dt_gap      = phisc_gapsecs;
phisc.phisc_gap  = phisc_value;

if (const_phisc == QPFALSE){
   ret = SC_Pot_fetch(&phisc,btimep,etimep);
   if (ret == PHI_SC_FATAL_ERROR){
      printf("SC_Pot_fetch returned Fatal Error");
      free_phisc_arrays(&phisc);
      free_n_sweeps(*nswps,*swpppp); free(*swpppp); *swpppp=NULL;
      return PEACE3D_ERROR;
   }
} else {
   Phisc_fill_not_available(&phisc,btimep,etimep);
}

ret = phisc2sweep_all(&phisc,*swpppp,*nswps);
if (ret != ALL_OKAY){
   printf("phisc2sweep returned %d\n!",ret);
   free_phisc_arrays(&phisc);
   free_n_sweeps(*nswps,*swpppp); free(*swpppp);*swpppp=NULL;
   return PEACE3D_ERROR;
}

ret = sweep_sc_potential_adjust(*swpppp,*nswps);
 if (ret != ALL_OKAY){
   printf("sweep_sc_potential_adjust returned %d\n!",ret);
   free_phisc_arrays(&phisc);
   free_n_sweeps(*nswps,*swpppp); free(*swpppp);*swpppp=NULL;
   return PEACE3D_ERROR;
}

free_phisc_arrays(&phisc);

return ALL_OKAY;
     
   
}



/************************************************************************/
int idfs_sweep_counter (SDDAS_ULONG dkey, IDFS_LINEAGE *lngp, 
                        struct idf_data *idfp,
                        IDFS_TIME *bt,  IDFS_TIME *et){
/************************************************************************/
/* count the number of sweeps for the virtual between bt and et, crossing
*  idfs file boundaries if necessary. Re-position back to bt on exit
*
*  File must have been init'ed and positioned prior to entry.
*/

int nswp,isen;
SDDAS_SHORT ret,nsen;
IDFS_TIME tnow,dummyt;

nswp = 0;
tnow = *bt;

ret = ALL_OKAY;  /* to get started */

while ( (compare_idfs_time(&tnow,et)!=T1_LATER )&&
     (ret == ALL_OKAY) ){
     ret = ReadVIDF(dkey, tnow.year, tnow.doy, tnow.sec, tnow.nano,
                    et->year,  et->doy,  et->sec,  et->nano,
		   (SDDAS_CHAR *) &nsen, _SEN, 0, 0, -1);
     isen = nsen;  /* coerce to int */	    
     if (DIAG_IDFS){
        printf("idfs sweep counter\n");
	printf("ReadVIDF returned %d\n",ret);
	print_idfs_time(bt); printf(" to "); print_idfs_time(et);
	print_idfs_lineage(lngp); printf("\n");
	printf("Number of sensors found is %d\n",isen);
     }	    
     isen--;  /* use last sensor for read; count even if not filled */
     if (isen < 0) return SWEEP_COUNT_BAD_SENSOR_NUMBER;
     while (ret != NEXT_FILE_STATUS && 	
                 (compare_idfs_time(&tnow,et)!=T1_LATER ) ){

          ret = read_drec (dkey, lngp->exten, lngp->vsn, idfp, isen, 1, 1);
	      /* read, forward, full_swp because sweeping data ?) */
	  if (ret == ALL_OKAY|| ret == NEXT_FILE_STATUS){
	        nswp++;
	     	set_times_from_idf(&tnow,&dummyt,&dummyt,idfp);
	  }	
	  if (DIAG_SWEEP_COUNTER){
		printf("Sweep counter; tnow = ");
		print_idfs_time(&tnow);
		printf(" at nswp = %d\n",nswp);
	  }		 		 
     }		    
     if (ret == NEXT_FILE_STATUS){
         ret = next_idfs_file(dkey,*lngp,&tnow,et,idfp); 
     }		    
}     

ret = reset_experiment_info(dkey,lngp->exten,lngp->vsn,
       bt->year, bt->doy, bt->sec, bt->nano,
       et->year, et->doy, et->sec, et->nano);
if (ret != ALL_OKAY) return SWEEP_COUNT_RESET_ERR; /* something wrong; pretend end! */

ret = file_pos (dkey, lngp->exten, lngp->vsn, idfp,        
        bt->year, bt->doy, bt->sec, bt->nano,
       et->year, et->doy, et->sec, et->nano);
      
if (ret != ALL_OKAY) return SWEEP_COUNT_POS_ERR; /* something wrong; pretend end! */

if (DIAG_IDFS){printf("sweep counter found %d sweeps\n",nswp);}
return nswp;

}

/*************************************************************************/
int idfs_sweep_nlevels (SDDAS_ULONG dkey, IDFS_LINEAGE *lngp,
      struct idf_data *idfp, int *nsenp){
/*************************************************************************/

/* run through all sensors looking for a filled one, and return the number of
*  energy levels; sensor used returned in nsenp */

SDDAS_CHAR fwd, full_swp;
SDDAS_SHORT ret_val;
int isen, nlev;

fwd = 0; full_swp = 1; isen = 0;

while (!idfp->filled_data && isen < *nsenp){
   ret_val = read_drec(dkey,lngp->exten,lngp->vsn, idfp,isen,fwd,full_swp);
   if (ret_val < 0){
            printf ("\nError %d from read_drec.\n", ret_val);
            return PEACE3D_ERROR;
   }
   isen++;
   if(DIAG_PRINT_3D){
      printf("read_drec'ed sen %d. ret_val = %d, filled_data = %d\n",
                      isen-1, ret_val, idfp->filled_data);
   }
}
if (isen == *nsenp  && !idfp->filled_data){
      printf("idfs_sweep_nlevels Failed; all sensors read but no data filled\n");
      return SWEEP_NLEVELS_NOT_FOUND;
}
nlev = idfp->num_sample;
*nsenp = isen - 1;  /* sensor found; counter incremented once too many times */

if(DIAG_PRINT)printf("Initialise used sensor %d; there are %d n_samples\n",
            isen, nlev);

return nlev;

}      


/*********************************************************************/
SDDAS_SHORT idfs_n_spin_sweep_etime(
        IDFS_LINEAGE *lngp, 
	IDFS_TIME *btimep,
        IDFS_TIME *etimep,
	T_OR_F do_pa,
	PEACE_GATHER_CHOICE gathch,
	int n2count){
/*********************************************************************/
/* For given lineage and btimep, find the etime which is at or beyond 
*  n2count sweeps or spins. This routine init's the file, does its
*  job, and then frees everything
*
*  Inputs:
*    lngp, btimep, do_pa
*    gathch        sweeps or spins only (secs is too easy!).
*    n2ount        how many
*
*  Ouput:
*     etimep       end time of last sweep after n2count spins/sweeps
*     ret_val      ALL_OKAY, errs
*
*  The purpose is to be able to supply GetNPeaceSweeps with an etime
*/ 	
	

void *idf_creator, *promote_struct;
struct idf_data *idfp;
SDDAS_ULONG dkey;
SDDAS_SHORT ret_val,nsen;
IDFS_LINEAGE lineage;
IDFS_TIME btime,dummytime;
int ipromote;
int isen,nsen_found;

btime = *btimep;    /* local copy */
*etimep = *btimep;
add_secs_to_idfs_time(etimep,PEACE_SPIN_PERIOD);

copy_lineage(&lineage,lngp);  /* local copy */


/* Trap out PAD request; if no pa rebinning use SPINPAD; else just use
*  input lineage */

if (strcmp(lngp->instrument,"PAD")==0 && 
    strcmp(lngp->virtual_inst,"CPPADAL") !=0 &&  /* don't use spinpad for this */
    do_pa != QPTRUE){
    /* use spinpad instead of old GetNPeacePAD routine */
  strcpy(lineage.instrument,"SPINPAD");
  if (strcmp(lngp->virtual_inst,"CPPADL")==0) {
     strcpy(lineage.virtual_inst,"LSPAD");
  } else if (strcmp(lngp->virtual_inst,"CPPADH")==0) {
        strcpy(lineage.virtual_inst,"HSPAD");
  } else {
     printf("PAD requested, but not CPPADL or CPPADH\n");
     return PEACE3D_ERROR;
  }
}

CfgInit ();
dbInitialize ();
init_idfs();

get_version_number (&(lngp->vsn));

ret_val = create_idf_data_structure(&idf_creator);
if (ret_val != ALL_OKAY){
     printf("Error %d in create idf struct for sweep\n", ret_val);
     free_experiment_info(); return ret_val;
}
idfp = (struct idf_data *)idf_creator;

ret_val = get_data_key(lineage.project, lineage.mission,
                lineage.experiment, lineage.instrument,
                lineage.virtual_inst, &dkey);
if (ret_val != ALL_OKAY){
       printf("Error getting data key\n");
       free_experiment_info(); return ret_val; 
}

/* do a promote if needed; twice */

for(ipromote=0; ipromote<2; ipromote++){
  promote_struct = pro_InitPromote (
     btime.year, btime.doy, 1000*btime.sec, 
     etimep->year, etimep->doy, 1000*etimep->sec, 
     SHOW_PROMOTE_STATUS_XWIN);
  pro_AddSource (promote_struct, dkey);

  if (pro_NeedToPromote (promote_struct) == sTrue) {
      printf("Trying to promote data: Pass %d\n",ipromote);
      pro_PromoteAll (promote_struct);
  }
  pro_FreePromote (promote_struct);
}

ret_val = init_idf_file (btime,*etimep, dkey, idfp, lineage.vsn);
if(ret_val!=ALL_OKAY){
   printf("Error init_idf_file\n");
   free_experiment_info(); return ret_val;
}



switch (gathch){
   case GATHER_SPINS:
	ret_val = advance_idf_file_spin_start(dkey,idfp,&lineage,&btime,etimep,
              nsen_found);
	while (n2count-- >0){  
	   ret_val = advance_idf_file_spin_start(dkey,idfp,&lineage,&btime,
	    etimep,nsen_found);
	if (ret_val != ALL_OKAY){
          if (ret_val == NEXT_FILE_STATUS){
	     ret_val = next_idfs_file(dkey,lineage,&btime,etimep,idfp);
	     if (ret_val != ALL_OKAY)break;  /* something wrong!*/
             ret_val = advance_idf_file_spin_start(dkey,idfp,&lineage,&btime,
	         etimep,nsen_found);
	     if (ret_val != ALL_OKAY)break;  /* something wrong!*/		 
	  }
	}
      }   
      set_times_from_idf(etimep,&dummytime,&dummytime,idfp);
      /* use start as it is already in next spin */
      break;
   case GATHER_SWEEPS:    
	/*  finding the etime for n2count sweeps */
    while (n2count-- > 0){  
	*etimep = btime;
	add_secs_to_idfs_time(etimep,PEACE_SPIN_PERIOD);
	ret_val = ReadVIDF(dkey, btime.year, btime.doy, btime.sec, btime.nano,
                      etimep->year,  etimep->doy,  etimep->sec,  etimep->nano,
		     (SDDAS_CHAR *) &nsen, _SEN, 0, 0, -1);
       isen = nsen;  /* coerce to int */	    
       if (isen < 0){
	 ret_val= SWEEP_COUNT_BAD_SENSOR_NUMBER;
	 break; 
       }
       ret_val = read2filled_sensor(dkey,&lineage,idfp,isen,&nsen_found,QPTRUE);
       if (ret_val == NEXT_FILE_STATUS || ret_val == RAN_OFF_END_FILE_ERR){
	     ret_val = next_idfs_file(dkey,lineage,&btime,etimep,idfp);
	     if (ret_val != ALL_OKAY)break;  /* something wrong! Exit early*/
       } else if (ret_val != ALL_OKAY) {
              break;
       }
       ret_val = read_drec(dkey,lineage.exten,lineage.vsn,idfp,nsen_found,1,1); 
                                                        	/*forward*/
       if (ret_val == NEXT_FILE_STATUS){
	     ret_val = next_idfs_file(dkey,lineage,&btime,etimep,idfp);
	     if (ret_val != ALL_OKAY)break;  /* something wrong! Exit early*/
       } else if (ret_val != ALL_OKAY) {
              break;
       }
     }
    set_times_from_idf(&btime,etimep,&dummytime,idfp);
    break;
  case GATHER_SECS: case NO_GATHER: default: 
   ret_val = SWEEP_GATHER_ERROR; 
   break;
}     
free_experiment_info();
return ret_val;

}

/*******************************************************************/
SDDAS_SHORT fill_spin3d (PEACE_SPIN3D *spinp,
                        IDFS_LINEAGE  *lngp,
                        struct idf_data *idfp,
                        SDDAS_ULONG dkey,
                        BIN_UNITS bin_units,
                        PSD_UNITS psd_units,
                        PHI_ZERO_FRAME phi_frame,
			T_OR_F do_pa ){
/*****************************************************************/
/* idfs file has been positioned to start of spin
*  spinp has btime and etime set to start and stop of spin
*  malloc_spin_sweep takes care of:
*
*    n_swp sweep structures and all internal arrays+lineage have been malloced
*    sweep structures have n_sen and n_lev set
*
*  ALL sweeps within spin assumed to have same energy levels
*  ALL sweeps within spin assumed to have same theta bins for sensors
*  ALL sweeps within spin assumed to have same lineage
*
*  fill spin structure by filling sweeps
*  use 1st sweep in spin to fill energy level and theta bins and lineage
*  [all done in fill_sweep]
*
*  Returns one of:

*    ret_val of fill_sweep     normal return
*    PEACE3D_INCOMPLETE_SPIN   if data gap detected via tswp_end > spin_end
*
*  Issues warning if sweep straddles spin end time in case calculation of
*  spin end using spin period is slightly off. For a real data gap > sweep
*  time, the next sweep will still be needed to complete spin and will
*  throw an INCOMPLETE_SPIN error.
*/

/* not used/needed in V2 */

}


#ifdef __cplusplus
}
#endif
