/* plotaz.cc ******************
*  Part of QPeace software
*  See README file in QPeace distribution for copyright information
*
*  SJ Schwartz   (s.j.schwartz@qmw.ac.uk)
*  Queen Mary, University of London
*  26 November 2000
*  last up-date 16 July 2001
*  30 Oct 2001  Adding Liouville Mapping and 1D cuts
*  March 2002   Implemented zero as missing value.
*  April 2005   Now at V3.11
*  November 2005 slice line thickness options added.
*
*  Utility routines for generic used, including:
*
*  Setting strings, idfs lineages, etc.
*  Setting conversion tables, getting energy bin limits, ...
*  Opeining and positioning idfs file
*  Manipulating time into format needed by and returned from idfs routines
*
*/


#include <stdio.h>

#include "plotaz.h"


 /*
 * Constructor
 *
 * Creates child widgets of the main PlotAzBox widget
 */

PlotAzBox::PlotAzBox( QWidget *parent, const char *name )
    : QVBox( parent, name )
{
    swp1=NULL;  // initialize to NULL so safe against free
    swp2=NULL;  // these now freed at beginning of RUN loop so swps kept
    swpar=NULL; // for Slice to use
    set_pa_opts_defaults(&paopts1);
    set_pa_opts_defaults(&paopts2);
    
    setMargin(10);
    setSpacing(20);

    QString str;

    QHBox *row1 = new QHBox( this );
    row1->setSpacing( 20 );

    // Create a LineageBox which has comboboxes for the various lineage components
    lineage = new LineageBox( row1 );

    // Column for transfering lineage to sweep boxes
    QVBox *lineage_transfer_col = new QVBox(row1);
    // add some space at top
    new QLabel(" ", lineage_transfer_col);

    // Create two pushbuttons to transfer lineage...
    QPushButton *setprimary = new QPushButton( "Primary Lineage -> ",
                                                lineage_transfer_col );
    // ...and connect the clicked SIGNAL with the SLOT
    connect( setprimary, SIGNAL( clicked() ),
             this, SLOT( slotSetPrimarySweepLineage()  ));
    QHBox *copy_time_box  = new QHBox(lineage_transfer_col);
    QLabel *copy_time_label = new QLabel("Copy Time", copy_time_box);
       copy_time_label->setAlignment(AlignRight | AlignVCenter);
    QPushButton *copy_time = new QPushButton ("|\nV",copy_time_box);
    connect (copy_time, SIGNAL (clicked() ),
             this , SLOT (slotCopyTime() ) );

    QPushButton *setsecondary = new QPushButton( "Secondary Lineage -> ",
                                                lineage_transfer_col );
    connect( setsecondary, SIGNAL( clicked() ),
             this, SLOT( slotSetSecondarySweepLineage()  ));
    // add some space at bottom
    new QLabel(" ", lineage_transfer_col);

    QVBox *sweeps = new QVBox (row1);
       sweeps->setSpacing(20);
    prim_sweep  = new SweepBox(1, Horizontal, sweeps);
      prim_sweep->setFrameStyle(QFrame::Box | QFrame::Sunken);
      prim_sweep->setTitle("Primary Slice - Controls Autoscaling ...");
    sec_sweep  = new SweepBox(1, Horizontal, sweeps);
      sec_sweep->setFrameStyle(QFrame::Box | QFrame::Sunken);
      sec_sweep->setTitle("Secondary Slice - plotted on right");

    QHBox *scale_opts = new QHBox (this);
    scale_opts->setSpacing(2);
    QVBox *scale_labels       = new QVBox (scale_opts);
    	                        scale_labels->setSpacing(2);
    QVBox *scale_units        = new QVBox (scale_opts);
    	                        scale_units->setSpacing(2);
    QVBox *scale_linlog       = new QVBox (scale_opts);
    	                        scale_linlog->setSpacing(2);
    QVBox *scale_auto         = new QVBox (scale_opts);
    	                        scale_auto->setSpacing(2);
    QVBox *scale_minmax_label = new QVBox (scale_opts);
    	                        scale_minmax_label->setSpacing(2);
    QVBox *scale_min          = new QVBox (scale_opts);
    	                        scale_min->setSpacing(2);
    QVBox *scale_max          = new QVBox (scale_opts);
     	                        scale_max->setSpacing(2);

// Energy/velocity options
    new QLabel ("Energy/Velocity Options", scale_labels);
    energy_units = new QComboBox(FALSE,scale_units);
       for (int ibu = 0; ibu < N_BIN_UNITS; ibu++){
          energy_units->insertItem (BIN_UNITS_STRING((BIN_UNITS)ibu), -1);
       }
    energy_linlog = new QComboBox(FALSE,scale_linlog);
       energy_linlog->insertItem("Linear",-1);
       energy_linlog->insertItem("Log",-1);
       energy_linlog->setCurrentItem(1);
   energy_scale = new QComboBox(FALSE,scale_auto);
       energy_scale->insertItem("Autoscale",-1);
       energy_scale->insertItem("Userscale->",-1);
    new QLabel ("min, max", scale_minmax_label);
    emin = new QLineEdit ("1", scale_min);
    emax = new QLineEdit ("10", scale_max);

// PSD Options
    new QLabel ("Phase Space Options", scale_labels);
    psd_units = new QComboBox(FALSE,scale_units);
       for (int ipsu = 0; ipsu < N_PSD_UNITS; ipsu++){
          psd_units->insertItem (PSD_UNITS_STRING((PSD_UNITS)ipsu), -1);
       }
    psd_linlog = new QComboBox(FALSE,scale_linlog);
       psd_linlog->insertItem("Linear",-1);    /* same order as PGSpectro.h */
       psd_linlog->insertItem("Log",-1);
       psd_linlog->setCurrentItem(1);
    psd_scale = new QComboBox(FALSE,scale_auto);
       psd_scale->insertItem("Autoscale",-1); /* same order as PGSpectro.h */
       psd_scale->insertItem("Userscale->",-1);
    new QLabel ("min, max", scale_minmax_label);
    psdmin = new QLineEdit ("1", scale_min);
    psdmax = new QLineEdit ("10", scale_max);
// Analysis Options
    new QLabel("Analysis", scale_labels);
    anal_proc = new QComboBox(FALSE,scale_units);
       anal_proc->insertItem("None",-1);
       anal_proc->insertItem("Add Prim+Sec",-1);
       anal_proc->insertItem("Subtract Prim-Sec",-1);
       anal_proc->insertItem("Multiply Prim*Sec",-1);
       anal_proc->insertItem("Divide Prim/Sec", -1);
    anal_linlog = new QComboBox(FALSE,scale_linlog);
       anal_linlog->insertItem("Linear",-1);    /* same order as PGSpectro.h */
       anal_linlog->insertItem("Log",-1);
       anal_linlog->setCurrentItem(1);
    anal_scale = new QComboBox(FALSE,scale_auto);
       anal_scale->insertItem("Autoscale",-1); /* same order as PGSpectro.h */
       anal_scale->insertItem("Userscale->",-1);
    new QLabel ("min, max", scale_minmax_label);
    analmin = new QLineEdit ("1", scale_min);
    analmax = new QLineEdit ("10", scale_max);

// Plot Options
    QHBox *plot_opts = new QHBox (this);
    plot_opts->setSpacing(2);
    new QLabel ("Plot Options", plot_opts);
    plot_col = new QComboBox(FALSE,plot_opts);
       plot_col->insertItem("Rainbow",-1);   /* same order as PGSpectro.h */
       plot_col->insertItem("Gray White Low",-1);
       plot_col->insertItem("Gray White High", -1);
       plot_col->insertItem("Inv Red-Blue",-1);  
       plot_col->insertItem("Red-Blue",-1);
       plot_col->insertItem("Earth", -1);
    plot_dev = new QComboBox(FALSE,plot_opts);
       plot_dev->insertItem("Landscape GIF File ->",-1);
       plot_dev->insertItem("Portrait GIF File ->",-1);
       plot_dev->insertItem("Landscape PS File ->",-1);
       plot_dev->insertItem("Portrait PS File ->",-1);
    new QLabel ("Xwin Number", plot_opts);
    plot_win_edit = new QLineEdit ("3", plot_opts);
    plot_win_edit->setMaxLength(3); plot_win_edit->setFixedWidth(45);
    new QLabel ("Plot File Stem", plot_opts);
    plot_file_edit = new QLineEdit ("plotaz_plot", plot_opts);
    QPushButton *plotfileselect = new QPushButton("Browse...",plot_opts);
    connect(plotfileselect, SIGNAL(clicked()), this, SLOT (slotPlotfile()) );

/*  for the future - copied from qjas
// Export Options
    QHBox *exp_opts_box = new QHBox (this);
    exp_opts_box->setSpacing(5);
    new QLabel ("Export to File Options", exp_opts_box);
    exp_opt = new QComboBox(FALSE,exp_opts_box);
       exp_opt->insertItem("Ascii Cluster Exchange",-1);
       exp_opt->insertItem("Ascii Tabular",-1);
       exp_opt->insertItem("CDF", -1);
    new QLabel ("Export File Stem", exp_opts_box);
    exp_file_edit = new QLineEdit ("qjas_data", exp_opts_box);
*/

// Action Buttons
    QHButtonGroup *actions = new QHButtonGroup(this);
    connect (actions, SIGNAL(clicked(int)), this, SLOT (slotActions(int)));
    
    new QPushButton( "Xwin",actions );
    new QPushButton( "Print to File", actions);
  
    // slicenmap Runs plotaz (to fetch/plot data) AND slice'n'map GUI
    QPushButton *slicenmap = new QPushButton( "Xwin then Slice'n'Map",actions );
    connect( slicenmap,  SIGNAL(clicked()) , this, SLOT (slotSlice()) );
    QPushButton *help = new QPushButton( "Help", actions);
    QPushButton *save = new QPushButton("Save...",actions);
    QPushButton *restore = new QPushButton("Restore...",actions);
    
    connect (help,SIGNAL(clicked()), this, SLOT (slotHelp()));
    connect (save,     SIGNAL(clicked()),  this, SLOT (slotSave()));
    connect (restore,  SIGNAL(clicked()),  this, SLOT (slotRestore()));
    quit = new QPushButton( "Quit",actions);

// create help window and slicnmap
    helpbwin = new Helpbwin();
    slicewin = new Slicewin(this);

}


/*
*     Constructors for LineageBox and SweepBox
*/

LineageBox::LineageBox( QWidget *parent, const char *name )
    : QVBox( parent, name )
{
    proj = new  QComboBox (FALSE, this);
    proj->insertStrList (project_strs);
    miss = new  QComboBox (FALSE, this);
    for (int isc = 0; isc < N_SC; isc++){
          miss->insertItem (SC_NAME((SC)isc), -1);
    }
    exp  = new  QComboBox (FALSE, this);
    exp->insertStrList (exp_strs);
    inst = new  QComboBox (FALSE, this);
    inst->insertStrList (inst_strs);
    virins = new  QComboBox (FALSE, this);
    virins->insertStrList (virins_strs);
// more to follow
}

SweepBox::SweepBox (int strips, Orientation orientation,
                          QWidget *parent, const char *name )
     : QGroupBox (strips, orientation, parent, name)
{
QHBox *row_lin = new QHBox (this );
// row_lin->setMaximumHeight(20);      // this works if needed
row_lin->setSpacing(2);
lineage_lbl = new QLabel ("Lineage",row_lin);
project = new QLabel ("CLUSTERII", row_lin);
project->setFrameStyle(QFrame::Box | QFrame::Raised);
mission = new QLabel ("CLUSTER-2", row_lin);
mission->setFrameStyle(QFrame::Box | QFrame::Raised);
exp = new QLabel ("PEACE", row_lin);
exp->setFrameStyle(QFrame::Box | QFrame::Raised);
inst = new QLabel ("PAD", row_lin);
inst->setFrameStyle(QFrame::Box | QFrame::Raised);
virins = new QLabel ("CPPADL", row_lin);
virins->setFrameStyle(QFrame::Box | QFrame::Raised);
pachoice = new QComboBox (FALSE, row_lin);
       pachoice->insertItem("no pa binning", -1);
       pachoice->insertItem("pa: angles", -1);
       pachoice->insertItem("pa: cosines", -1); 
connect (pachoice, SIGNAL (activated(int)),this, SLOT (slotForceGather(int)));   
npa = new QLineEdit("13", row_lin);
      QToolTip::add(npa, "Number of pitch angle bins for rebinning");
QHBox *row_gather = new QHBox(this);
  row_gather->setSpacing(2);

gatherchoice = new QComboBox (FALSE, row_gather);
       gatherchoice->insertItem("Gather n sweeps",-1);
       gatherchoice->insertItem("Gather n spins",-1);
       gatherchoice->insertItem("Gather n.n secs",-1);
       gatherchoice->insertItem("No gathering",-1);
       gatherchoice->setCurrentItem(3);  // default is no gathering
gathern = new QLineEdit("1",row_gather);   
// S/C Potential Options
sc_pot = new QComboBox(FALSE,row_gather);
   sc_pot->insertItem("No s/c pot. correction",-1);
   sc_pot->insertItem("Do s/c pot. correction",-1);
   sc_pot->insertItem("Const s/c pot.",-1);
   QToolTip::add(sc_pot,
              "Uses EFW for Cluster; otherwise only const used");      
new QLabel("val, gap secs",row_gather);
sc_pot_value = new QLineEdit("0.0",row_gather);
QToolTip::add(sc_pot_value,"Value for gap or no EFW data in V");
sc_pot_gap   = new QLineEdit("60.0",row_gather);
QToolTip::add(sc_pot_gap,"If gap > this in EFW, gap s/c pot value used");

QHBox *row_time_prompt = new QHBox(this);
new QLabel("Time close to but earlier than spin start:",
                                  row_time_prompt);
QPushButton *next = new QPushButton("Advance to next segment (approx)",row_time_prompt);
connect(next,SIGNAL(clicked()), this, SLOT (slotNextSegment()));

timebox = new TimeBox(this);

QHBox *row_slice = new QHBox (this);
row_slice->setSpacing(2);
slice_lab  = new QLabel ("Azimuthal Slice (deg)",row_slice);
azslice = new QLineEdit ("0", row_slice);
onecountchoice = new QComboBox(FALSE,row_slice);
     onecountchoice->insertItem("Actual Data",-1);
     onecountchoice->insertItem("1 Count Level x");
onecount_mult = new QLineEdit("1.0",row_slice);     
/*
QPushButton *copytime = new QPushButton("Copy Start to End Time",row_slice);
connect (copytime , SIGNAL(clicked()), this ,SLOT (slotCopyStart2EndTime()) );
*/

}

TimeBox:: TimeBox(QWidget *parent, const char *name)
    :  QHBox(parent,name)
{

    yearLabel = new QLabel( "Year", this);

    YearspinBox = new QSpinBox( this );
    YearspinBox->setWrapping( TRUE );
    YearspinBox->setMaxValue( 2100 );
    YearspinBox->setMinValue( 1900 );
    YearspinBox->setValue( 2001 );


    doyLabel = new QLabel( "Doy", this );
    
    DoyspinBox = new QSpinBox( this );
    DoyspinBox->setWrapping( TRUE );
    DoyspinBox->setMaxValue( 366 );
    DoyspinBox->setMinValue( 1 );
    DoyspinBox->setValue(90);
    

    textLabel1_2 = new QLabel( "Date", this );

    dateEdit = new QDateEdit( this );
    dateEdit->setEnabled( TRUE );
    dateEdit->setDate( QDate( 2001, 3, 31 ) );
    dateEdit->setOrder( QDateEdit::YMD);
    dateEdit->setAutoAdvance( TRUE );
    
    timeLabel = new QLabel( "Time", this );

    timeEdit = new QTimeEdit( this );
    timeEdit->setAutoAdvance( TRUE );
    this->setSHour(18);

    connect( dateEdit, SIGNAL( valueChanged(const QDate&) ), this, SLOT( DateChange() ) );
    connect( YearspinBox, SIGNAL( valueChanged(int) ), this, SLOT( YearChange() ) );
    connect( DoyspinBox, SIGNAL( valueChanged(int) ), this, SLOT( DoyChange() ) );

    setTabOrder( YearspinBox, DoyspinBox );
    setTabOrder( DoyspinBox, dateEdit );
}    

void TimeBox::DateChange()
{
    int *doy = new int;
    QDate *sdate = new QDate;
    *sdate = dateEdit->date();
    
    QanDateToYearDoy(sdate->year(), sdate->month(), sdate->day(), doy);
    if((sdate->year() >= 1950) && (sdate->year()<=2100))
    { 
    YearspinBox->setValue(sdate->year());
    DoyspinBox->setValue(*doy);}        
}

void TimeBox::YearChange()
{
   int *month = new int;
   int *day = new int;
   int year, doy;
   
   year =  YearspinBox->value();
   doy  =  DoyspinBox->value();  
   
   QanYearDoyToDate(year, doy, month, day);   
   QDate date(year, *month, *day);   
   dateEdit->setDate(date);     
}


void TimeBox::DoyChange()
{
   int *month = new int;
   int *day = new int;
   int year, doy;
   
   year = YearspinBox->value();
   doy = DoyspinBox->value();
   
   QanYearDoyToDate(year, doy, month, day);   
   QDate date(year, *month, *day);   
   dateEdit->setDate(date);     
}

void TimeBox::setSHour(int hour)
{
int min, sec;
QTime time = timeEdit->time();
min = time.minute();
sec = time.second();
time.setHMS(hour,min,sec);
timeEdit->setTime(time);
}

void TimeBox::setSMinute(int minute)
{
int hour, sec;
QTime time = timeEdit->time();
hour = time.hour();
sec  = time.second();
time.setHMS(hour,minute,sec);
timeEdit->setTime(time);
}

void TimeBox::setSSecond(int second)
{
int hour, min;
QTime time = timeEdit->time();
hour = time.hour();
min  = time.minute();
time.setHMS(hour,min,second);
timeEdit->setTime(time);
}


void TimeBox::GetTimeFromTimeBox(IDFS_TIME *idftp)
{
int year, doy, hr, min, isec;
long nano;
QTime time;

year = YearspinBox->value();
doy  = DoyspinBox->value();

time = timeEdit->time();
hr   = time.hour();
min  = time.minute();
isec = time.second();
nano = 0;

to_idfs_time(year, doy, hr, min, isec, nano, idftp);
if (DIAG_PRINT_3D)print_idfs_time(idftp);

}

void SweepBox::SetLineage (LineageBox *lineagebox){

project->setText(lineagebox->proj->currentText());
mission->setText(lineagebox->miss->currentText());
exp->setText(lineagebox->exp->currentText());
inst->setText(lineagebox->inst->currentText());
virins->setText(lineagebox->virins->currentText());
}

void SweepBox::slotForceGather (int pachoice){

if (pachoice == NO_BINNING) return;  /* do nothing if no pa requested */
if (gatherchoice->currentItem() == NO_GATHER){
   gatherchoice->setCurrentItem(GATHER_SPINS);
   gathern->setText("1");
}
return;   

}

/********************************************************************/
void SweepBox::slotNextSegment(void){

PEACE_GATHER_CHOICE gathch;
IDFS_TIME btime, etime;
IDFS_LINEAGE lineage;
T_OR_F do_pa;
int n2cnt;
SDDAS_SHORT ret_val;
int ihr,imin,isec;
long inano;

transfer_sweep_times( &btime);
etime = btime;      // safety net
transfer_lineage( &lineage);
gathch = (PEACE_GATHER_CHOICE)gatherchoice->currentItem();
do_pa = (pachoice->currentItem()==NO_BINNING) ? QPFALSE : QPTRUE;
n2cnt = gathern->text().toInt();

if (gathch == NO_GATHER) {
    gathch = GATHER_SPINS;  // jump a spin
    n2cnt = 1;
}    

switch (gathch){
   case GATHER_SECS: 
     add_secs_to_idfs_time(&etime, gathern->text().toFloat());
     break;   
   case GATHER_SPINS: 
      ret_val = idfs_n_spin_sweep_etime( &lineage, &btime, &etime, do_pa, gathch, n2cnt);
      if (ret_val != ALL_OKAY){
         printf("idfs_n_spin_sweep_etime in slotNextSegment returned %d\n",ret_val);
      }    
      add_secs_to_idfs_time(&etime, -1.0*PEACE_SPIN_PERIOD); /* above overshoots for pad */
      break;      
   case GATHER_SWEEPS:
      ret_val = idfs_n_spin_sweep_etime( &lineage, &btime, &etime, do_pa, gathch, n2cnt);
      if (ret_val != ALL_OKAY){
         printf("idfs_n_spin_sweep_etime in slotNextSegment returned %d\n",ret_val);
      }
      add_secs_to_idfs_time(&etime, 0.001); /* above undershoots */
      break;      



   default:
      printf("slotNextSegment confused about gather choice = %d\n",gathch);
      break;
}   

timebox->YearspinBox->setValue(etime.year);
timebox->DoyspinBox->setValue(etime.doy);
to_hms_time(&etime,&ihr, &imin, &isec, &inano);
timebox->setSHour(ihr);
timebox->setSMinute(imin);
if (inano>500000000)isec +=1;   //round nanosecs
timebox->setSSecond(isec);
}



/* Constructor and destructor for help window */

Helpbwin::Helpbwin(QWidget *parent, const char *name)
   : QVBox(parent,name){

resize(600,600);
setCaption("plotaz help");

QHBox *buttons = new QHBox(this);
QPushButton *backbutton, *forwardbutton, *topbutton;

backbutton = new QPushButton( "<- Back", buttons, 0 );
forwardbutton = new QPushButton ("Forward ->",buttons, 0);
topbutton = new QPushButton ("Top",buttons,0);

QTextBrowser *helpbrowser = new QTextBrowser(this);

connect( backbutton, SIGNAL( clicked() ), helpbrowser, SLOT( backward() ) );
connect( forwardbutton, SIGNAL( clicked() ), helpbrowser, SLOT( forward() ) );
connect( topbutton, SIGNAL( clicked() ), helpbrowser, SLOT( home() ) );

QString homepath = QString(getenv("QPEACE_HOME")) + "/doc/";

helpbrowser->mimeSourceFactory()->setFilePath(homepath);
helpbrowser->setSource("plotaz.html");

}

/*
void SweepBox::slotCopyStart2EndTime(void){
year2->setText(year->text());
doy2->setText(doy->text());
hr2->setText(hr->text());
min2->setText(min->text());
sec2->setText(sec->text());

}
*/

Helpbwin::~Helpbwin(void){

hide();
}

/* Constructor and destructor for slice window */

Slicewin::Slicewin(QWidget *parent, const char *name)
   : QDialog(parent,name){

// initialize to NULL so safe to throw away on RUN

for (int i=0; i<N_POSSIBLE_CUT_SOURCES; i++)swposs[i]  = NULL;
for (int i=0; i<N_MAX_CUTS; i++) swpp[i] = NULL;

QVBox * slicevbox = new QVBox(this);
//slicevbox->setFixedSize(800,500);   // helps

slicevbox->setSpacing(2);
slicevbox->setMargin(10);
 
setCaption("PEACE Sweep Shifting, Liouville Mapping and Slicing");

new QLabel("Primary => [Parallel Velocity Shift] => Liouville Mapping => Predicted Secondary",slicevbox);

QHBox *shiftbox = new QHBox(slicevbox);
    shiftbox->setSpacing(2);
    new QLabel ("Add parallel velocity (km/s) to v//(primary)", shiftbox);
    v1 = new QLineEdit ("0", shiftbox);
      QToolTip::add(v1, 
       "Parallel velocity to SHIFT all bins BEFORE mapping (if SHIFTED source selected)");
    new QLabel ("   Add parallel velocity (km/s) to v//(secondary)", shiftbox);
    v2 = new QLineEdit ("0", shiftbox);
      QToolTip::add(v2, 
       "Parallel velocity to SHIFT all bins BEFORE mapping (if SHIFTED source selected)");
    shiftbox->setFrameStyle(QFrame::Box | QFrame::Sunken);

QHBox *mapbox = new QHBox(slicevbox);
    mapbox->setSpacing(2);
    new QLabel ("Primary |B1| (arb units)", mapbox);
    b1 = new QLineEdit ("1", mapbox);
    new QLabel ("  Secondary |B2| (arb units)", mapbox);
    b2 = new QLineEdit ("1", mapbox);
    new QLabel ("  Potential Phi2-Phi1 (eV)", mapbox);
    pot = new QLineEdit ("0.0", mapbox);
    mapbox->setFrameStyle(QFrame::Box | QFrame::Sunken);
    QToolTip::add(mapbox,"NOTE: Liouville Mapping requires f(v) phase space density, not Counts or Differential Energy Flux");

new QLabel("\nSelect Sources for Wheel Plots (if desired)",slicevbox);

QHBox *wheel_select = new QHBox(slicevbox);
    wheel_select->setSpacing(2);
    new QLabel ("Left Half", wheel_select);
    lwheel = new Sources(wheel_select);
    new QLabel ("Right Half", wheel_select);
    rwheel = new Sources(wheel_select);
    wheel_select->setFrameStyle(QFrame::Box | QFrame::Sunken);


QHBox *cut_select_opt = new QHBox(slicevbox);
   new QLabel("\nSelect Sources, Pitch Angles (FLOW direction), and Options for 1D Cuts (if desired)",
                                 cut_select_opt);
   new QLabel("\n   Box/Lettering Thickness",cut_select_opt);
   box_thick = new QComboBox(FALSE,cut_select_opt);
      box_thick->insertItem("Thick = 1",0);	 
      box_thick->insertItem("Thick = 2",1);	 
      box_thick->insertItem("Thick = 3",2);	 
      box_thick->insertItem("Thick = 4",3);	 
      box_thick->insertItem("Thick = 5",4);	 
      box_thick->insertItem("Thick = 6",5);	 
      box_thick->insertItem("Thick = 7",6);	 
      box_thick->insertItem("Thick = 8",7);	 
      box_thick->insertItem("Thick = 9",8);	 
      box_thick->insertItem("Thick = 10",9);
      box_thick->setCurrentItem(0);	 

QHBox *cut_select = new QHBox(slicevbox);
    cut_select->setSpacing(2);
    cut_select->setFrameStyle(QFrame::Box | QFrame::Sunken);
    for (int icut = 0; icut < N_MAX_CUTS; icut++){
        cuts[icut] = new Cuts_Selector(icut, cut_select);
	cuts[icut]->setSpacing(2);
    }    

// scaling options cut and pasted from PlotAzBox; would be better
// to put this code once in a set of classes, but I'm in a hurry
QHBox *scale_opts = new QHBox (slicevbox);
    scale_opts->setSpacing(2);
    scale_opts->setMargin(5);
    QVBox *scale_labels       = new QVBox (scale_opts);
    	                        scale_labels->setSpacing(2);
    QVBox *scale_units        = new QVBox (scale_opts);
    	                        scale_units->setSpacing(2);
    QVBox *scale_linlog       = new QVBox (scale_opts);
    	                        scale_linlog->setSpacing(2);
    QVBox *scale_auto         = new QVBox (scale_opts);
    	                        scale_auto->setSpacing(2);
    QVBox *scale_minmax_label = new QVBox (scale_opts);
    	                        scale_minmax_label->setSpacing(2);
    QVBox *scale_min          = new QVBox (scale_opts);
    	                        scale_min->setSpacing(2);
    QVBox *scale_max          = new QVBox (scale_opts);
     	                        scale_max->setSpacing(2);

// Energy/velocity options 
    new QLabel ("Energy/Velocity Options", scale_labels);
    energy_units = new QComboBox(FALSE,scale_units); /* can't change */
    energy_units->insertItem ("As Main Win", -1);
    energy_linlog = new QComboBox(FALSE,scale_linlog);
       energy_linlog->insertItem("Linear",-1);
       energy_linlog->insertItem("Log",-1);
       energy_linlog->setCurrentItem(1);
   energy_scale = new QComboBox(FALSE,scale_auto);
       energy_scale->insertItem("Autoscale",-1);
       energy_scale->insertItem("Userscale->",-1);
    new QLabel ("min, max", scale_minmax_label);
    emin = new QLineEdit ("1", scale_min);
    emax = new QLineEdit ("10", scale_max);
// PSD Options
    new QLabel ("Phase Space Options", scale_labels); /* can't change */
    psd_units = new QComboBox(FALSE,scale_units);
    psd_units->insertItem ("As Main Win", -1);
    psd_linlog = new QComboBox(FALSE,scale_linlog);
       psd_linlog->insertItem("Linear",-1);    /* same order as PGSpectro.h */
       psd_linlog->insertItem("Log",-1);
       psd_linlog->setCurrentItem(1);
    psd_scale = new QComboBox(FALSE,scale_auto);
       psd_scale->insertItem("Autoscale",-1); /* same order as PGSpectro.h */
       psd_scale->insertItem("Userscale->",-1);
    new QLabel ("min, max", scale_minmax_label);
    psdmin = new QLineEdit ("1", scale_min);
    psdmax = new QLineEdit ("10", scale_max);

// Plot Options
    QHBox *plot_opts = new QHBox (slicevbox);
    plot_opts->setSpacing(2);
    new QLabel ("Plot Options", plot_opts);
    plot_col = new QComboBox(FALSE,plot_opts);
       plot_col->insertItem("Rainbow",-1);   /* same order as PGSpectro.h */
       plot_col->insertItem("Gray White Low",-1);
       plot_col->insertItem("Gray White High", -1);
    plot_dev = new QComboBox(FALSE,plot_opts);
       plot_dev->insertItem("Landscape GIF File ->",-1);
       plot_dev->insertItem("Portrait GIF File ->",-1);
       plot_dev->insertItem("Landscape PS File ->",-1);
       plot_dev->insertItem("Portrait PS File ->",-1);
    new QLabel ("Xwin Number", plot_opts);
    plot_win_edit = new QLineEdit ("3", plot_opts);
    plot_win_edit->setMaxLength(3); plot_win_edit->setFixedWidth(45);
    new QLabel ("Plot File Stem", plot_opts);
    plot_file_edit = new QLineEdit ("slice_map_plot", plot_opts);
    QPushButton *plotfileselect = new QPushButton("Browse...",plot_opts);
    connect(plotfileselect, SIGNAL(clicked()), this, SLOT (slotPlotfile()) );

/*  for the future - copied from qjas
// Export Options
    QHBox *exp_opts_box = new QHBox (this);
    exp_opts_box->setSpacing(5);
    new QLabel ("Export to File Options", exp_opts_box);
    exp_opt = new QComboBox(FALSE,exp_opts_box);
       exp_opt->insertItem("Ascii Cluster Exchange",-1);
       exp_opt->insertItem("Ascii Tabular",-1);
       exp_opt->insertItem("CDF", -1);
    new QLabel ("Export File Stem", exp_opts_box);
    exp_file_edit = new QLineEdit ("qjas_data", exp_opts_box);
*/

// Action Buttons
    QHButtonGroup *actions = new QHButtonGroup (slicevbox);
    connect(actions, SIGNAL(clicked(int)), this , SLOT(slotRun(int)));
    new QPushButton( "Xwin",actions );
    new QPushButton( "Print to File", actions);
    
    QPushButton *help = new QPushButton( "Help", actions);
    connect (help,     SIGNAL(clicked()) , parent, SLOT (slotHelp()) );
    QPushButton *dismissbut = new QPushButton( "Dismiss",actions);
    connect (dismissbut, SIGNAL(clicked()), this, SLOT (slotDismiss()) );

slicevbox->adjustSize();

}

Slicewin::~Slicewin(void){
// throw away old sweeps - but NOT Primary & Secondary from main plotaz
for (int i = 0; i<N_POSSIBLE_CUT_SOURCES; i++){
   if (i != PRIMARY && i != SECONDARY){
      if (swposs[i] != NULL){
         free_sweep_arrays(swposs[i]); 
         free(swposs[i]);
	 swposs[i]=NULL;
      }
   }
}         

hide();
}


/* Constructor for cut selection combination */
Cuts_Selector::Cuts_Selector(int icut, QWidget *parent, const char *name)
   : QVBox(parent,name){
    srclab = new QLabel(this);
    srclab->setNum(icut);
    source = new Sources(this);
    pangle = new Pangles(this);
    linesty = new Linestys(this);
    colour  = new Colours(this);
    thick   = new Linesthick(this);
}    

/* Constructor for Sources, etc.  pull downs */
Sources::Sources(QWidget *parent, const char *name)
    : QComboBox(FALSE,parent,name){
   this->insertItem("None",0);    // same order as enum !!
   this->insertItem("Primary", -1);
   this->insertItem("Shifted Primary", -1);
   this->insertItem("Mapped Primary", -1);
   this->insertItem("Sh then Map Prim", -1);
   this->insertItem("Secondary", -1);
   this->insertItem("Shift Secondary", -1);
   this->insertItem("Mapped/Secondary",-1);
   this->insertItem("ShMap/ShSec'ry",-1);
   this->insertItem("Mapped-Secondary",-1);
   this->insertItem("ShMap - ShSec'ry",-1);
}

Pangles::Pangles(QWidget *parent, const char *name)
   : QLineEdit("0", parent, name){
}

Linestys::Linestys(QWidget *parent, const char *name)
    : QComboBox(FALSE,parent,name){
/* these are in enum order for cpgsls calls; there is no zero */
   this->insertItem("Not-used",0);  // can't pass zero to cpgsls so must trap!!
   this->insertItem("Solid",1);
   this->insertItem("Dashed", 2);
   this->insertItem("DashDot", 3);
   this->insertItem("Dotted", 4);
   this->insertItem("DashDotDotDot",5);
   this->setCurrentItem(1);
}

Linesthick::Linesthick(QWidget *parent, const char *name)
     : QComboBox(FALSE,parent,name){
/* add one to these to get thickness */
   this->insertItem("Thick = 1",0);
   this->insertItem("Thick = 2",1);
   this->insertItem("Thick = 3",2);
   this->insertItem("Thick = 4",3);
   this->insertItem("Thick = 5",4);
   this->insertItem("Thick = 6",5);
   this->insertItem("Thick = 7",6);
   this->insertItem("Thick = 8",7);
   this->insertItem("Thick = 9",8);
   this->insertItem("Thick = 10",9);
   this->setCurrentItem(0);
}
        
Colours::Colours(QWidget *parent, const char *name)
    : QComboBox(FALSE,parent,name){
/* these are in enum order from 0 for cpgsci calls */
   this->insertItem("White",0);   
   this->insertItem("Black", -1);
   this->insertItem("Red", -1);
   this->insertItem("Green", -1);
   this->insertItem("Blue",-1);
   this->insertItem("Cyan",-1);
   this->insertItem("Magenta",-1);
   this->insertItem("Yellow",-1);
   this->insertItem("Orange",-1);
   this->insertItem("YellowGreen",-1);
   this->setCurrentItem(1);
}

void PlotAzBox::slotSetPrimarySweepLineage(void){
prim_sweep->SetLineage(lineage);
}

void PlotAzBox::slotSetSecondarySweepLineage(void){
sec_sweep->SetLineage(lineage);
}

void PlotAzBox::slotCopyTime(void)
{
sec_sweep->timebox->dateEdit->setDate(prim_sweep->timebox->dateEdit->date());
sec_sweep->timebox->timeEdit->setTime(prim_sweep->timebox->timeEdit->time());
}

void PlotAzBox::slotActions(int pltbutton){

PEACE_GATHER_GAPS gapopts;
PEACE_GATHER_CHOICE gatherch;
float gathersecs;
int gather_swp_spins;
int device;

PHISC_CHOICE phiscch;
float sc_pot_val, sc_pot_gap;
SDDAS_SHORT ret;


if (pltbutton== XDEV || pltbutton == XWIN_N_SLICE) device = XWIN;
else if (pltbutton==PRINT2FILE) device = (PLOT_CHOICE)plot_dev->currentItem();
else return;  /* not a plot requested */

// throw away old sweeps
if (swp1 != NULL){free_sweep_arrays(swp1); free(swp1);swp1=NULL;}
if (swp2 != NULL){free_sweep_arrays(swp2); free(swp2);swp2=NULL;}
if (swpar != NULL){free_sweep_arrays(swpar); free(swpar); swpar = NULL;}

prim_sweep->transfer_lineage( &lng1);
sec_sweep->transfer_lineage( &lng2);

int pach = prim_sweep->pachoice->currentItem();
if (pach == NO_BINNING){
   paopts1.do_pa = QPFALSE;
} else {
   paopts1.do_pa = QPTRUE;
   if (pach == PA_ANGLES) paopts1.mu_bins = QPFALSE;
   else paopts1.mu_bins = QPTRUE;
   paopts1.n_pabins = prim_sweep->npa->text().toInt();
}

pach = sec_sweep->pachoice->currentItem();
if (pach == NO_BINNING){
   paopts2.do_pa = QPFALSE;
} else {
   paopts2.do_pa = QPTRUE;
   if (pach == PA_ANGLES) paopts2.mu_bins = QPFALSE;
   else paopts2.mu_bins = QPTRUE;
   paopts2.n_pabins = sec_sweep->npa->text().toInt();
}    
prim_sweep->transfer_sweep_times( &idft1a);
sec_sweep->transfer_sweep_times( &idft2a);

az1 = prim_sweep->azslice->text().toFloat();
az2 = sec_sweep->azslice->text().toFloat();
onecnt_mult1 = prim_sweep->onecount_mult->text().toFloat();
onecnt_mult2 = sec_sweep->onecount_mult->text().toFloat();

bin_units = (BIN_UNITS)energy_units->currentItem();
psdu = (PSD_UNITS)psd_units->currentItem();

Set_sweep_wheel_options_default(&swoptions);

swoptions.color_table = (PGSpectro_col)plot_col->currentItem();
swoptions.zlinlog     = (PGSpectro_linlog)psd_linlog->currentItem();
swoptions.rlinlog     = (PGSpectro_linlog)energy_linlog->currentItem();
swoptions.zautoscale  = (Yes_or_No)psd_scale->currentItem();
swoptions.rautoscale  = (Yes_or_No)energy_scale->currentItem();
swoptions.zmin        = psdmin->text().toFloat();
swoptions.zmax        = psdmax->text().toFloat();
swoptions.rmin        = emin->text().toFloat();
swoptions.rmax        = emax->text().toFloat();
swoptions.skip_missing = YES;
    // z=0.0 used to be treated as missing by  Sweep_wheel_plot
    
QString pgfile, pgfileexten;
QString pgdev, pgdevstr;

pgfile=plot_file_edit->text(); // get file name; if xwin, reset below

switch(device){
   case LGIF:
      pgdev.setLatin1("/GIF",-1);
      pgfileexten.setLatin1(".gif", -1);
      break;
   case PGIF:
      pgdev.setLatin1("/VGIF",-1);
      pgfileexten.setLatin1(".gif", -1);
      break;
   case LPS:
      pgdev.setLatin1("/CPS",-1);
      pgfileexten.setLatin1(".eps", -1);
      break;
   case PPS:
      pgdev.setLatin1("/VCPS",-1);
      pgfileexten.setLatin1(".eps", -1);
      break;
   case XWIN:  default:
      bool ok;
      pgfile = plot_win_edit->text();
      int ixwin = pgfile.toInt( &ok, 10);  // see if text in pgfile ok xwin
      if (!ok || ixwin < 1 ||ixwin > 255){ // if not open default xwin
          pgfile.setLatin1(PLOTAZ_DEFAULT_XWIN_STR, -1);
      }
      pgdev.setLatin1("/Xserv", -1);
      pgfileexten.setLatin1("");
      break;
}
char pg_dev_cstr[200];
pgdevstr = pgfile.append(pgfileexten);
pgdevstr.append(pgdev);
strcpy(pg_dev_cstr, pgdevstr.latin1());
// swoptions.pg_device =  pg_dev_cstr;

if (cpgopen(pg_dev_cstr) <=0){
 printf("Failed to open pgplot device %s successfully\n", pg_dev_cstr);
 printf("If file check write permissions; if Xwindow, check pgxwin server\n");
 return;       // early exit, before getting sweeps and malloc memory
}

// get the two sweeps - routine below mallocs them
// use free_sweep_arrays and then free(swp) to cleanup
swp1=NULL; swp2=NULL; swpar=NULL; /* initialise */

phiscch          = (PHISC_CHOICE)prim_sweep->sc_pot->currentItem();
sc_pot_val       = prim_sweep->sc_pot_value->text().toFloat();
sc_pot_gap       = prim_sweep->sc_pot_gap->text().toFloat();
gatherch         = (PEACE_GATHER_CHOICE)prim_sweep->gatherchoice->currentItem();
gathersecs       = prim_sweep->gathern->text().toFloat();
gather_swp_spins = prim_sweep->gathern->text().toInt();

sweep_spin_gap_defaults (&gapopts);  /* default swp, spin gap */
if (strcmp("PAD",lng1.instrument) ==0  ) {
     gapopts.sweep_gap = gapopts.spin_gap;
}

idft1b = idft1a;  // safe
if (gatherch == GATHER_SECS){
   add_secs_to_idfs_time(&idft1b,gathersecs);
} else if (gatherch != NO_GATHER){
   ret = idfs_n_spin_sweep_etime(&lng1,&idft1a,&idft1b,paopts1.do_pa,
       gatherch, gather_swp_spins);
   if (ret != ALL_OKAY){
       printf("idfs_n_spin_sweep_etime returned %d\n",ret);
       cpgclos();
       return;
   }
}
if (prim_sweep->onecountchoice->currentItem() == 1){  /* one count */
  switch(psdu){
     case PSD_UNITS_CNTS_ACCUM:        psdu1 = PSD_UNITS_CNTS_ACCUM_1CNT;       break;
     case PSD_UNITS_DIFF_ENERGY_FLUX:  psdu1 = PSD_UNITS_DIFF_ENERGY_FLUX_1CNT; break;
     case PSD_UNITS_S3_PER_KM6:        psdu1 = PSD_UNITS_S3_PER_KM6_1CNT;       break;
     default: psdu1 = psdu; break;
  }
} else { psdu1 = psdu;
}

swp1 = PeaceGetAzSliceV2 (&lng1, &idft1a, &idft1b, az1,
       bin_units,  psdu1,
       &paopts1, phiscch, sc_pot_val, sc_pot_gap,
       gatherch,  gapopts, gathersecs, gather_swp_spins);

if (swp1==NULL){
   printf("Failed to get 1st sweep\n");
   cpgclos();
   return;}
   
switch(psdu1){
   case PSD_UNITS_CNTS_ACCUM_1CNT:
   case PSD_UNITS_DIFF_ENERGY_FLUX_1CNT:
   case PSD_UNITS_S3_PER_KM6_1CNT:
       sweep_scaler(swp1,onecnt_mult1);
       break;
   default: break;
}
   
if (DIAG_PRINT_AZ){printf("swp1:\n"); print_sweep_structure(swp1);}

phiscch          = (PHISC_CHOICE)sec_sweep->sc_pot->currentItem();
sc_pot_val       = sec_sweep->sc_pot_value->text().toFloat();
sc_pot_gap       = sec_sweep->sc_pot_gap->text().toFloat();
gatherch         = (PEACE_GATHER_CHOICE)sec_sweep->gatherchoice->currentItem();
gathersecs       = sec_sweep->gathern->text().toFloat();
gather_swp_spins = sec_sweep->gathern->text().toInt();

sweep_spin_gap_defaults (&gapopts);  /* default swp, spin gap */
if (strcmp("PAD",lng2.instrument) ==0  ) {
     gapopts.sweep_gap = gapopts.spin_gap;
}

idft2b = idft2a;
if (gatherch == GATHER_SECS){
    add_secs_to_idfs_time(&idft2b,gathersecs);
} else if (gatherch != NO_GATHER){
   ret = idfs_n_spin_sweep_etime(&lng2,&idft2a,&idft2b,paopts2.do_pa,
       gatherch, gather_swp_spins);
   if (ret != ALL_OKAY){
       printf("idfs_n_spin_sweep_etime returned %d\n",ret);
       cpgclos();
       return;
   }
}

if (sec_sweep->onecountchoice->currentItem() == 1){  /* one count */
  switch(psdu){
     case PSD_UNITS_CNTS_ACCUM:        psdu2 = PSD_UNITS_CNTS_ACCUM_1CNT;       break;
     case PSD_UNITS_DIFF_ENERGY_FLUX:  psdu2 = PSD_UNITS_DIFF_ENERGY_FLUX_1CNT; break;
     case PSD_UNITS_S3_PER_KM6:        psdu2 = PSD_UNITS_S3_PER_KM6_1CNT;       break;
     default: psdu2 = psdu; break;
  }
} else { psdu2 = psdu;
}

swp2 = PeaceGetAzSliceV2 (&lng2, &idft2a, &idft2b, az2,
       bin_units, psdu2,
       &paopts2, phiscch, sc_pot_val, sc_pot_gap,
       gatherch,  gapopts, gathersecs, gather_swp_spins);

if (swp2==NULL){
   printf("Failed to get 2nd sweep\n");
   if (swp1 != NULL){free_sweep_arrays(swp1); free(swp1);swp1=NULL;}
   cpgclos();
   return;
}

switch(psdu2){
   case PSD_UNITS_CNTS_ACCUM_1CNT:
   case PSD_UNITS_DIFF_ENERGY_FLUX_1CNT:
   case PSD_UNITS_S3_PER_KM6_1CNT:
       sweep_scaler(swp2,onecnt_mult2);
       break;
   default: break;
}

if (anal_proc->currentItem() != NONE)cpgsubp(1,2);
cpgshls(0,0,1,0); /* background white */
cpgshls(1,0,0,0); /* default colour black */
cpgpage();                                    

pg_ret_val = PlotTwoSweeps(swp1,swp2, &swoptions);
QString s;  /* return actual min/max values to line edit boxes */
emin->setText(s.setNum(swoptions.rmin));
emax->setText(s.setNum(swoptions.rmax));
psdmin->setText(s.setNum(swoptions.zmin));
psdmax->setText(s.setNum(swoptions.zmax));

prim_sweep->etime = swp1->etime;  /* store etimes; maybe for use by next buttons? */
sec_sweep->etime  = swp1->etime;


if (anal_proc->currentItem() != NONE){
   swpar = sweep_arithmetic(swp1,swp2,(ARITH_OPER)anal_proc->currentItem());
   if (swpar == NULL){
      printf("Sweep_arithmetic returned NULL pointer\n");
   } else {
       swoptions.zlinlog     = (PGSpectro_linlog)anal_linlog->currentItem();
       swoptions.zautoscale  = (Yes_or_No)anal_scale->currentItem();
       swoptions.zmin        = analmin->text().toFloat();
       swoptions.zmax        = analmax->text().toFloat();
       strcpy(swpar->lineage->project,"Analysis: ");
       strcpy(swpar->lineage->mission,"Primary");
       switch(anal_proc->currentItem()){
          case ADD:
	     strcpy(swpar->lineage->experiment," + "); break;
	  case SUBTRACT:
 	     strcpy(swpar->lineage->experiment," - "); break;
	  case MULTIPLY:
 	     strcpy(swpar->lineage->experiment," x ");
	     swpar->psd_units = UNKNOWN_PSD_UNITS;  /* for wedge label*/
	     break;
	  case DIVIDE:
 	     strcpy(swpar->lineage->experiment," / ");
	     swpar->psd_units = UNKNOWN_PSD_UNITS;  /* for wedge label*/
	     break;
	  default:
 	     strcpy(swpar->lineage->experiment," ??? ");
	     swpar->psd_units = UNKNOWN_PSD_UNITS;  /* for wedge label*/
	     break;
        }
       strcpy(swpar->lineage->instrument,"Secondary");
       strcpy(swpar->lineage->virtual_inst,"");
       cpgpage();
       pg_ret_val = PlotTwoSweeps(swpar,swpar,&swoptions);
       analmin->setText(s.setNum(swoptions.zmin));
       analmax->setText(s.setNum(swoptions.zmax));
       free_sweep_arrays(swpar); free(swpar); swpar=NULL;
   }
        
}

cpgiden();

/* make sure pgplot dev is properly closed */
cpgclos();

}


void SweepBox::transfer_lineage(IDFS_LINEAGE *lng){

strcpy(lng->project,project->text().latin1());
strcpy(lng->mission,mission->text().latin1());
strcpy(lng->experiment,exp->text().latin1());
strcpy(lng->instrument,inst->text().latin1());
strcpy(lng->virtual_inst,virins->text().latin1());
strcpy(lng->exten,"");
if (DIAG_PRINT_3D)print_idfs_lineage(lng);

}

void SweepBox::transfer_sweep_times(  IDFS_TIME *idfta){

this->timebox->GetTimeFromTimeBox(idfta);  //easy!

//int iyear, idoy, ihr, imin, isec;
//long inano;
//float fsecs;

//iyear = year->text().toInt();
//idoy  = doy->text().toInt();
//ihr   = hr->text().toInt();
//imin  = min->text().toInt();
//fsecs = sec->text().toFloat();

//isec = (int)fsecs;
//inano = (long)(1000000000*(fsecs-isec));

//to_idfs_time(iyear, idoy, ihr, imin, isec, inano, idfta);
//if (DIAG_PRINT_3D)print_idfs_time(idfta);

}


void PlotAzBox::slotSlice(void){
   this->showMinimized();
   slicewin->showNormal();
}

void PlotAzBox::slotPlotfile(void){

QString plotfile = QFileDialog::getSaveFileName( 
  this->plot_file_edit->text() , QString::null   , this,0,
   "Specify Plot File Stem");

if(!plotfile.isEmpty())this->plot_file_edit->setText(plotfile);
}

void PlotAzBox::slotHelp(void){
if ( QApplication::desktop()->width() < 600
            || QApplication::desktop()->height() < 600 ){
   helpbwin->showMaximized();
} else {
   helpbwin->show();
}
}

void Slicewin::slotPlotfile(void){

QString plotfile = QFileDialog::getSaveFileName( 
  this->plot_file_edit->text() , QString::null   , this,0,
   "Specify Plot File Stem");

if(!plotfile.isEmpty())this->plot_file_edit->setText(plotfile);
}

void Slicewin::slotDismiss(void){
   (this->parentWidget())->showFullScreen();
   (this->parentWidget())->showNormal();
   this->hide();
}

void Slicewin::slotRun(int pltbutton){

int device;

PlotAzBox* plazbx = dynamic_cast<PlotAzBox*>(this->parentWidget());


// throw away old sweeps - but NOT Primary & Secondary from main plotaz
for (int i = 0; i<N_POSSIBLE_CUT_SOURCES; i++){
   if (i != PRIMARY && i != SECONDARY){
      if (swposs[i] != NULL){
         free_sweep_arrays(swposs[i]); 
         free(swposs[i]);
	 swposs[i]=NULL;
      }
   }
}         


bprim      = b1->text().toFloat();
bsec       = b2->text().toFloat();
potential  = pot->text().toFloat();
vprim      = v1->text().toFloat();
vsec       = v2->text().toFloat();

scop = &sweep_cut_options;
swop = &sweep_wheel_options;
Set_Sweep_cut_options_default(scop);
Set_sweep_wheel_options_default(swop);
swop->color_table = (PGSpectro_col)plot_col->currentItem();
scop->linethickness = (float)box_thick->currentItem() + 1.0;

scop->xlinlog    =  swop->rlinlog    = 
                        (PGSpectro_linlog)energy_linlog->currentItem();
scop->ylinlog    =  swop->zlinlog    = 
                        (PGSpectro_linlog)psd_linlog->currentItem();
scop->xautoscale =  swop->rautoscale = (Yes_or_No)energy_scale->currentItem();
scop->yautoscale =  swop->zautoscale = (Yes_or_No)psd_scale->currentItem();
scop->xmin       =  swop->rmin       = emin->text().toFloat();
scop->xmax       =  swop->rmax       = emax->text().toFloat();
scop->ymin       =  swop->zmin       = psdmin->text().toFloat();
scop->ymax       =  swop->zmax       = psdmax->text().toFloat();

QString pgfile, pgfileexten;
QString pgdev, pgdevstr;

pgfile=plot_file_edit->text(); // get file name; if xwin, reset below

if (pltbutton== XDEV) device = XWIN;
else if (pltbutton==PRINT2FILE) device = (PLOT_CHOICE)plot_dev->currentItem();
else return;  /* not a plot requested */

switch(device){
   case LGIF:
      pgdev.setLatin1("/GIF",-1);
      pgfileexten.setLatin1(".gif", -1);
      break;
   case PGIF:
      pgdev.setLatin1("/VGIF",-1);
      pgfileexten.setLatin1(".gif", -1);
      break;
   case LPS:
      pgdev.setLatin1("/CPS",-1);
      pgfileexten.setLatin1(".eps", -1);
      break;
   case PPS:
      pgdev.setLatin1("/VCPS",-1);
      pgfileexten.setLatin1(".eps", -1);
      break;
   case XWIN:  default:
      bool ok;
      pgfile = plot_win_edit->text();
      int ixwin = pgfile.toInt( &ok, 10);  // see if text in pgfile ok xwin
      if (!ok || ixwin < 1 ||ixwin > 255){ // if not open default xwin
          pgfile.setLatin1(PLOTAZ_DEFAULT_XWIN_STR, -1);
      }
      pgdev.setLatin1("/Xserv", -1);
      pgfileexten.setLatin1("");
      break;
}
char pg_dev_cstr[200];
pgdevstr = pgfile.append(pgfileexten);
pgdevstr.append(pgdev);
strcpy(pg_dev_cstr, pgdevstr.latin1());
// swoptions.pg_device =  pg_dev_cstr;

if (cpgopen(pg_dev_cstr) <=0){
 printf("Failed to open pgplot device %s successfully\n", pg_dev_cstr);
 printf("If file check write permissions; if Xwindow, check pgxwin server\n");
 return;       // early exit, before getting sweeps and malloc memory
}

for (int i=0; i<N_POSSIBLE_CUT_SOURCES; i++)swpcount[i]=0;
lwhsrc = (SWEEP_SOURCE)lwheel->currentItem();
rwhsrc = (SWEEP_SOURCE)rwheel->currentItem();
if (lwhsrc != NO_SOURCE)swpcount[lwhsrc]++;
if (rwhsrc != NO_SOURCE)swpcount[rwhsrc]++;

nswps = 0;
for (int i=0; i< N_MAX_CUTS; i++){
   SWEEP_SOURCE src_holder;
   if ((src_holder = (SWEEP_SOURCE)cuts[i]->source->currentItem()) 
                                                        != NO_SOURCE){
      cutsrc[nswps]  = src_holder;
      thetas[nswps]  = cuts[i]->pangle->text().toFloat();
      cutlsty[nswps] = (QPGLinestyle)cuts[i]->linesty->currentItem();
      cutcol[nswps]  = (QPGLine_colour)cuts[i]->colour->currentItem();
      cutthick[nswps]  = (float)cuts[i]->thick->currentItem() + 1.0;
      swpcount[src_holder]++;
      nswps++;
   }
}

scop->nswps = nswps;
scop->ncuts = 1;  /* ignored by Sweep_n_cuts...*/
scop->linestyles = cutlsty;
scop->colours    = cutcol;
scop->linethicknesses    = cutthick;

swposs[PRIMARY] = plazbx->swp1;  // get primary and secondary sweeps from main win
swposs[SECONDARY]  = plazbx->swp2;


// make needed sweeps; see if either needs shifting first
if (swpcount[SHIFTED_PRIMARY]>0 
           ||  swpcount[SHIFTED_THEN_MAPPED_PRIMARY]  >0 
           ||  swpcount[SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY]  >0 
	   ||  swpcount[SHIFTED_THEN_MAPPED_MINUS_SHIFTED_SECONDARY] >0 ){
   // need shifted primary sweep
   swposs[SHIFTED_PRIMARY] = PADvpara_boost(swposs[PRIMARY], vprim, 
               PLOTAZ_SHIFT_THETA_FAC, PLOTAZ_SHIFT_LEVEL_FAC);
   if (swposs[SHIFTED_PRIMARY] == NULL){
      printf("Failed to shift primary sweep\n"); cpgclos(); return;
   }
}
if (swpcount[SHIFTED_SECONDARY]>0 
           ||  swpcount[SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY]  >0 
	   ||  swpcount[SHIFTED_THEN_MAPPED_MINUS_SHIFTED_SECONDARY] >0 ){
   // need shifted secondary sweep
   swposs[SHIFTED_SECONDARY] = PADvpara_boost(swposs[SECONDARY], vsec, 
            PLOTAZ_SHIFT_THETA_FAC, PLOTAZ_SHIFT_LEVEL_FAC);
   if (swposs[SHIFTED_SECONDARY] == NULL){
      printf("Failed to shift secondary sweep\n"); cpgclos(); return;
   }
}

// now unshifted mapping
char holdstr[500];
if (swpcount[MAPPED_PRIMARY]>0 
           ||  swpcount[MAPPED_OVER_SECONDARY]  >0 
	   ||  swpcount[MAPPED_MINUS_SECONDARY] >0 ){
   // need mapped sweep
   swposs[MAPPED_PRIMARY] = 
              PADLiouvilleMap(swposs[PRIMARY], bprim, bsec, potential,
	      PLOTAZ_LIOUVILLE_UNSHIFT_DOUBLE);
   if (swposs[MAPPED_PRIMARY] == NULL){
      printf("Failed to map sweep\n"); cpgclos(); return;
   }  // see what sweeps derived from mapped are necessary
   if (swpcount[MAPPED_OVER_SECONDARY]> 0){   // need mapped/sec
      swposs[MAPPED_OVER_SECONDARY] = 
         sweep_arithmetic(swposs[MAPPED_PRIMARY], swposs[SECONDARY], DIVIDE);
      if (swposs[MAPPED_OVER_SECONDARY] == NULL){
         printf("Failed to take map/sec sweep\n"); cpgclos(); return;
      }
      sprintf(holdstr, "%s/%s",
           swposs[MAPPED_PRIMARY]->lineage->mission,
           swposs[SECONDARY]->lineage->mission);
      strncpy(swposs[MAPPED_OVER_SECONDARY]->lineage->mission,holdstr,
          LINEAGE_MAX_STR -1);
      sprintf(holdstr,"%s,%s",
        swposs[MAPPED_PRIMARY]->lineage->instrument,
        swposs[SECONDARY]->lineage->instrument);
      strncpy(swposs[MAPPED_OVER_SECONDARY]->lineage->instrument,
           holdstr,LINEAGE_MAX_STR -1 );
       sprintf(holdstr,"%s,%s",
        swposs[MAPPED_PRIMARY]->lineage->virtual_inst,
        swposs[SECONDARY]->lineage->virtual_inst);
       strncpy(swposs[MAPPED_OVER_SECONDARY]->lineage->virtual_inst,
           holdstr,LINEAGE_MAX_STR -1);
      swposs[MAPPED_OVER_SECONDARY]->psd_units = UNKNOWN_PSD_UNITS;
      swposs[MAPPED_OVER_SECONDARY]->etime = swposs[SECONDARY]->btime; 
   }
   if (swpcount[MAPPED_MINUS_SECONDARY] > 0){   // need mapped - sec
      swposs[MAPPED_MINUS_SECONDARY] = 
        sweep_arithmetic(swposs[MAPPED_PRIMARY], swposs[SECONDARY], SUBTRACT);
      if (swposs[MAPPED_MINUS_SECONDARY] == NULL){
         printf("Failed to take map-sec sweep\n"); cpgclos(); return;
      }
      sprintf(holdstr, "%s - %s",
           swposs[MAPPED_PRIMARY]->lineage->mission,
           swposs[SECONDARY]->lineage->mission);
      strncpy(swposs[MAPPED_MINUS_SECONDARY]->lineage->mission,holdstr,
         LINEAGE_MAX_STR -1);
      sprintf(holdstr,"%s,%s",
        swposs[MAPPED_PRIMARY]->lineage->instrument,
        swposs[SECONDARY]->lineage->instrument);
      strncpy(swposs[MAPPED_MINUS_SECONDARY]->lineage->instrument,
           holdstr,LINEAGE_MAX_STR -1);
      sprintf(holdstr,"%s,%s",
        swposs[MAPPED_PRIMARY]->lineage->virtual_inst,
        swposs[SECONDARY]->lineage->virtual_inst);
      strncpy(swposs[MAPPED_MINUS_SECONDARY]->lineage->virtual_inst,
           holdstr,LINEAGE_MAX_STR -1);
      swposs[MAPPED_MINUS_SECONDARY]->etime = swposs[SECONDARY]->btime; 
   }
}

// repeat mapping with shifted pads
if (swpcount[SHIFTED_THEN_MAPPED_PRIMARY]>0 
           ||  swpcount[SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY]  >0 
	   ||  swpcount[SHIFTED_THEN_MAPPED_MINUS_SHIFTED_SECONDARY] >0 ){
   // need shifted mapped sweep
   swposs[SHIFTED_THEN_MAPPED_PRIMARY] = 
              PADLiouvilleMap(swposs[SHIFTED_PRIMARY], bprim, bsec, potential,
	      PLOTAZ_LIOUVILLE_SHIFTED_DOUBLE);
   if (swposs[SHIFTED_THEN_MAPPED_PRIMARY] == NULL){
      printf("Failed to map sweep\n"); cpgclos(); return;
   } // now see if sweeps derived from shifted_mapped
   if (swpcount[SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY]> 0){   // need mapped/sec
      swposs[SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY] = 
         sweep_arithmetic(swposs[SHIFTED_THEN_MAPPED_PRIMARY], 
	        swposs[SHIFTED_SECONDARY], DIVIDE);
      if (swposs[SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY] == NULL){
         printf("Failed to take shifted map/sec sweep\n"); cpgclos(); return;
      }
      sprintf(holdstr, "%s/%s",
           swposs[SHIFTED_THEN_MAPPED_PRIMARY]->lineage->mission,
           swposs[SHIFTED_SECONDARY]->lineage->mission);
      strncpy(swposs[SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY]->lineage->mission,holdstr,
             LINEAGE_MAX_STR -1 );
      sprintf(holdstr,"%s,%s",
        swposs[SHIFTED_THEN_MAPPED_PRIMARY]->lineage->instrument,
        swposs[SHIFTED_SECONDARY]->lineage->instrument);
      strncpy(swposs[SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY]->lineage->instrument,
           holdstr,LINEAGE_MAX_STR -1);
       sprintf(holdstr,"%s,%s",
        swposs[SHIFTED_THEN_MAPPED_PRIMARY]->lineage->virtual_inst,
        swposs[SHIFTED_SECONDARY]->lineage->virtual_inst);
       strncpy(swposs[SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY]->lineage->virtual_inst,
           holdstr,LINEAGE_MAX_STR -1);
     swposs[SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY]->psd_units = UNKNOWN_PSD_UNITS; 
     swposs[SHIFTED_THEN_MAPPED_OVER_SHIFTED_SECONDARY]->etime = swposs[SHIFTED_SECONDARY]->btime; 
   }
   if (swpcount[SHIFTED_THEN_MAPPED_MINUS_SHIFTED_SECONDARY] > 0){// need mapped - sec
      swposs[SHIFTED_THEN_MAPPED_MINUS_SHIFTED_SECONDARY] = 
        sweep_arithmetic(swposs[SHIFTED_THEN_MAPPED_PRIMARY], 
	    swposs[SHIFTED_SECONDARY], SUBTRACT);
      if (swposs[SHIFTED_THEN_MAPPED_MINUS_SHIFTED_SECONDARY] == NULL){
         printf("Failed to take shifted map-sec sweep\n"); cpgclos(); return;
      }
      sprintf(holdstr, "%s - %s",
           swposs[SHIFTED_THEN_MAPPED_PRIMARY]->lineage->mission,
           swposs[SHIFTED_SECONDARY]->lineage->mission);
      strncpy(swposs[SHIFTED_THEN_MAPPED_MINUS_SHIFTED_SECONDARY]->lineage->mission,
             holdstr, LINEAGE_MAX_STR -1);
      sprintf(holdstr,"%s,%s",
        swposs[SHIFTED_THEN_MAPPED_PRIMARY]->lineage->instrument,
        swposs[SHIFTED_SECONDARY]->lineage->instrument);
      strncpy(swposs[SHIFTED_THEN_MAPPED_MINUS_SHIFTED_SECONDARY]->lineage->instrument,
           holdstr,LINEAGE_MAX_STR -1);
      sprintf(holdstr,"%s,%s",
        swposs[SHIFTED_THEN_MAPPED_PRIMARY]->lineage->virtual_inst,
        swposs[SHIFTED_SECONDARY]->lineage->virtual_inst);
      strncpy(swposs[SHIFTED_THEN_MAPPED_MINUS_SHIFTED_SECONDARY]->lineage->virtual_inst,
           holdstr,LINEAGE_MAX_STR -1);
      swposs[SHIFTED_THEN_MAPPED_MINUS_SHIFTED_SECONDARY]->etime = swposs[SHIFTED_SECONDARY]->btime; 
   }
}
// now should have all the sweeps we need for the plots
// put their pointers into the array
for (int i=0; i<nswps; i++)swpp[i] = swposs[cutsrc[i]];

if ( (lwhsrc != NO_SOURCE || rwhsrc != NO_SOURCE) && nswps > 0)cpgsubp(1,2);

cpgshls(0,0,1,0); /* background white */
cpgshls(1,0,0,0); /* default colour black */

// plot wheels 1st; if only 1 source, plot in both halves
if (lwhsrc != NO_SOURCE || rwhsrc != NO_SOURCE){ // at least 1 exists
   if (lwhsrc == NO_SOURCE)      lwhsrc = rwhsrc;
   else if (rwhsrc == NO_SOURCE) rwhsrc = lwhsrc;
   cpgpage();                                    
   pg_ret_val = PlotTwoSweeps(swposs[lwhsrc], swposs[rwhsrc],swop);
   if (pg_ret_val != PGSPECTRO_OK){
      printf("Slice'n'Map: Wheel plot failed\n");
      cpgclos();
      return;
   }
   QString s;  /* return actual min/max values to line edit boxes */
   emin->setText(s.setNum(swop->rmin));
   emax->setText(s.setNum(swop->rmax));
   psdmin->setText(s.setNum(swop->zmin));
   psdmax->setText(s.setNum(swop->zmax));
}

// now cuts
if (nswps > 0){
   cpgpage();                                    
   pg_ret_val = Sweep_n_cuts_plot(swpp,nswps,thetas, scop);
   if (pg_ret_val != PGSPECTRO_OK){
      printf("Slice'n'Map: Slices plot failed\n");
      cpgclos();
      return;
   }
   QString s;  /* return actual min/max values to line edit boxes */
   emin->setText(s.setNum(scop->xmin));
   emax->setText(s.setNum(scop->xmax));
   psdmin->setText(s.setNum(scop->ymin));
   psdmax->setText(s.setNum(scop->ymax));
}

cpgiden();
cpgclos();


}



void PlotAzBox::slotSave(void){
FILE *out;
QString savefile = QFileDialog::getSaveFileName( );
out = fopen(savefile.latin1(),"w");
if (out == NULL){
   printf("Error opening %s for save\n",savefile.latin1());
   return;
}   
fprintf(out, "%s = %d\n", savestrings[SPROJECT],            lineage->proj->currentItem()        ); 
fprintf(out, "%s = %d\n", savestrings[SMISSION],            lineage->miss->currentItem()        ); 
fprintf(out, "%s = %d\n", savestrings[SEXPERIMENT],         lineage->exp->currentItem()         );
fprintf(out, "%s = %d\n", savestrings[SINSTRUMENT],         lineage->inst->currentItem()        );
fprintf(out, "%s = %d\n", savestrings[SVIRTUAL],            lineage->virins->currentItem()      );

fprintf(out, "%s = %s\n", savestrings[SPROJECT1],           prim_sweep->project->text().latin1());
fprintf(out, "%s = %s\n", savestrings[SMISSION1],           prim_sweep->mission->text().latin1());
fprintf(out, "%s = %s\n", savestrings[SEXPERIMENT1],        prim_sweep->exp->text().latin1()    );
fprintf(out, "%s = %s\n", savestrings[SINSTRUMENT1],        prim_sweep->inst->text().latin1()   );
fprintf(out, "%s = %s\n", savestrings[SVIRTUAL1],           prim_sweep->virins->text().latin1() );
fprintf(out, "%s = %d\n", savestrings[SPA_OPTS1],           prim_sweep->pachoice->currentItem() );
fprintf(out, "%s = %d\n", savestrings[SN_PA1],              prim_sweep->npa->text().toInt()     );
fprintf(out, "%s = %d\n", savestrings[SGATHER_OPTS1],       prim_sweep->gatherchoice->currentItem());
fprintf(out, "%s = %g\n", savestrings[SN_GATHER1],          prim_sweep->gathern->text().toFloat());
fprintf(out, "%s = %d\n", savestrings[SSC_POT_OPTS1],       prim_sweep->sc_pot->currentItem()    );
fprintf(out, "%s = %g\n", savestrings[SSC_POT_VAL1],        prim_sweep->sc_pot_value->text().toFloat());
fprintf(out, "%s = %g\n", savestrings[SSC_POT_SECS1],       prim_sweep->sc_pot_gap->text().toFloat() );

fprintf(out, "%s = %d\n", savestrings[SYR1],         prim_sweep->timebox->YearspinBox->value()     );
fprintf(out, "%s = %d\n", savestrings[SDOY1],        prim_sweep->timebox->DoyspinBox->value()      );
fprintf(out, "%s = %d\n", savestrings[SHR1],         prim_sweep->timebox->timeEdit->time().hour()  );
fprintf(out, "%s = %d\n", savestrings[SMIN1],        prim_sweep->timebox->timeEdit->time().minute());
fprintf(out, "%s = %d\n", savestrings[SSEC1],        prim_sweep->timebox->timeEdit->time().second());
fprintf(out, "%s = %g\n", savestrings[SAZ1],         prim_sweep->azslice->text().toFloat());
fprintf(out, "%s = %d\n", savestrings[SONECNTCH1],   prim_sweep->onecountchoice->currentItem()   );
fprintf(out, "%s = %g\n", savestrings[SONECNT_MULT1],prim_sweep->onecount_mult->text().toFloat() );

fprintf(out, "%s = %s\n", savestrings[SPROJECT2],           sec_sweep->project->text().latin1()  );
fprintf(out, "%s = %s\n", savestrings[SMISSION2],           sec_sweep->mission->text().latin1()  );
fprintf(out, "%s = %s\n", savestrings[SEXPERIMENT2],        sec_sweep->exp->text().latin1()      );
fprintf(out, "%s = %s\n", savestrings[SINSTRUMENT2],        sec_sweep->inst->text().latin1()     );
fprintf(out, "%s = %s\n", savestrings[SVIRTUAL2],           sec_sweep->virins->text().latin1()   );
fprintf(out, "%s = %d\n", savestrings[SPA_OPTS2],           sec_sweep->pachoice->currentItem()   );
fprintf(out, "%s = %d\n", savestrings[SN_PA2],              sec_sweep->npa->text().toInt()       );
fprintf(out, "%s = %d\n", savestrings[SGATHER_OPTS2],       sec_sweep->gatherchoice->currentItem());
fprintf(out, "%s = %g\n", savestrings[SN_GATHER2],          sec_sweep->gathern->text().toFloat() );
fprintf(out, "%s = %d\n", savestrings[SSC_POT_OPTS2],       sec_sweep->sc_pot->currentItem()     );
fprintf(out, "%s = %g\n", savestrings[SSC_POT_VAL2],        sec_sweep->sc_pot_value->text().toFloat());
fprintf(out, "%s = %g\n", savestrings[SSC_POT_SECS2],       sec_sweep->sc_pot_gap->text().toFloat());


fprintf(out, "%s = %d\n", savestrings[SYR2],        sec_sweep->timebox->YearspinBox->value()      );
fprintf(out, "%s = %d\n", savestrings[SDOY2],       sec_sweep->timebox->DoyspinBox->value()       );
fprintf(out, "%s = %d\n", savestrings[SHR2],        sec_sweep->timebox->timeEdit->time().hour()   );
fprintf(out, "%s = %d\n", savestrings[SMIN2],       sec_sweep->timebox->timeEdit->time().minute() );
fprintf(out, "%s = %d\n", savestrings[SSEC2],       sec_sweep->timebox->timeEdit->time().second() );
fprintf(out, "%s = %g\n", savestrings[SAZ2],                sec_sweep->azslice->text().toFloat()  );
fprintf(out, "%s = %d\n", savestrings[SONECNTCH2],    sec_sweep->onecountchoice->currentItem()   );
fprintf(out, "%s = %g\n", savestrings[SONECNT_MULT2], sec_sweep->onecount_mult->text().toFloat() );

fprintf(out, "%s = %d\n", savestrings[SENERGY_UNITS],       energy_units->currentItem()          );
fprintf(out, "%s = %d\n", savestrings[SENERGY_LINLOG],      energy_linlog->currentItem()         );
fprintf(out, "%s = %d\n", savestrings[SENERGY_AUTOSCALE],   energy_scale->currentItem()          );
fprintf(out, "%s = %g\n", savestrings[SEMIN],               emin->text().toFloat()               );
fprintf(out, "%s = %g\n", savestrings[SEMAX],               emax->text().toFloat()               );
fprintf(out, "%s = %d\n", savestrings[SPSD_UNITS],          psd_units->currentItem()             );
fprintf(out, "%s = %d\n", savestrings[SPSD_LINLOG],         psd_linlog->currentItem()            );
fprintf(out, "%s = %d\n", savestrings[SPSD_SCALE],          psd_scale->currentItem()             );
fprintf(out, "%s = %g\n", savestrings[SPSD_MIN],            psdmin->text().toFloat()             );
fprintf(out, "%s = %g\n", savestrings[SPSD_MAX],            psdmax->text().toFloat()             );
fprintf(out, "%s = %d\n", savestrings[SANALYSIS_OPT],       anal_proc->currentItem()             );
fprintf(out, "%s = %d\n", savestrings[SANALYSIS_LINLOG],    anal_linlog->currentItem()           );
fprintf(out, "%s = %d\n", savestrings[SANALYSIS_AUTOSCALE], anal_scale->currentItem()            );
fprintf(out, "%s = %g\n", savestrings[SANALYSIS_MIN],       analmin->text().toFloat()            );
fprintf(out, "%s = %g\n", savestrings[SANALYSIS_MAX],       analmax->text().toFloat()            );
fprintf(out, "%s = %d\n", savestrings[SPLOT_COL],           plot_col->currentItem()              );
fprintf(out, "%s = %d\n", savestrings[SPLOT_DEV],           plot_dev->currentItem()              );
fprintf(out, "%s = %d\n", savestrings[SXWIN_NO],            plot_win_edit->text().toInt()        );
fprintf(out, "%s = %s\n", savestrings[SPLOT_STEM],          plot_file_edit->text().latin1()      );

fprintf(out, "%s = %g\n", savestrings[SVPARA1],             slicewin->v1->text().toFloat()       );
fprintf(out, "%s = %g\n", savestrings[SVPARA2],             slicewin->v2->text().toFloat()       );
fprintf(out, "%s = %g\n", savestrings[SB1],                 slicewin->b1->text().toFloat()       );
fprintf(out, "%s = %g\n", savestrings[SB2],                 slicewin->b2->text().toFloat()       );
fprintf(out, "%s = %g\n", savestrings[SDELPHI],             slicewin->pot->text().toFloat()      );
fprintf(out, "%s = %d\n", savestrings[SLEFT_WHEEL],         slicewin->lwheel->currentItem()      );
fprintf(out, "%s = %d\n", savestrings[SRIGHT_WHEEL],        slicewin->rwheel->currentItem()      );

fprintf(out, "%s = %d\n", savestrings[SBOX_THICK],          slicewin->box_thick->currentItem()   );

fprintf(out, "%s = %d\n", savestrings[SCUT_OPT0],           slicewin->cuts[0]->source->currentItem()    );
fprintf(out, "%s = %g\n", savestrings[SCUT_ANG0],           slicewin->cuts[0]->pangle->text().toFloat() );
fprintf(out, "%s = %d\n", savestrings[SCUT_STYLE0],         slicewin->cuts[0]->linesty->currentItem()   );
fprintf(out, "%s = %d\n", savestrings[SCUT_COL0],           slicewin->cuts[0]->colour->currentItem()    );
fprintf(out, "%s = %d\n", savestrings[SCUT_THICK0],         slicewin->cuts[0]->thick->currentItem()    );

fprintf(out, "%s = %d\n", savestrings[SCUT_OPT1],           slicewin->cuts[1]->source->currentItem()    );
fprintf(out, "%s = %g\n", savestrings[SCUT_ANG1],           slicewin->cuts[1]->pangle->text().toFloat() );
fprintf(out, "%s = %d\n", savestrings[SCUT_STYLE1],         slicewin->cuts[1]->linesty->currentItem()   );
fprintf(out, "%s = %d\n", savestrings[SCUT_COL1],           slicewin->cuts[1]->colour->currentItem()    );
fprintf(out, "%s = %d\n", savestrings[SCUT_THICK1],         slicewin->cuts[1]->thick->currentItem()    );

fprintf(out, "%s = %d\n", savestrings[SCUT_OPT2],           slicewin->cuts[2]->source->currentItem()    );
fprintf(out, "%s = %g\n", savestrings[SCUT_ANG2],           slicewin->cuts[2]->pangle->text().toFloat() );
fprintf(out, "%s = %d\n", savestrings[SCUT_STYLE2],         slicewin->cuts[2]->linesty->currentItem()   );
fprintf(out, "%s = %d\n", savestrings[SCUT_COL2],           slicewin->cuts[2]->colour->currentItem()    );
fprintf(out, "%s = %d\n", savestrings[SCUT_THICK2],         slicewin->cuts[2]->thick->currentItem()    );

fprintf(out, "%s = %d\n", savestrings[SCUT_OPT3],           slicewin->cuts[3]->source->currentItem()    );
fprintf(out, "%s = %g\n", savestrings[SCUT_ANG3],           slicewin->cuts[3]->pangle->text().toFloat() );
fprintf(out, "%s = %d\n", savestrings[SCUT_STYLE3],         slicewin->cuts[3]->linesty->currentItem()   );
fprintf(out, "%s = %d\n", savestrings[SCUT_COL3],           slicewin->cuts[3]->colour->currentItem()    );
fprintf(out, "%s = %d\n", savestrings[SCUT_THICK3],         slicewin->cuts[3]->thick->currentItem()    );

fprintf(out, "%s = %d\n", savestrings[SCUT_OPT4],           slicewin->cuts[4]->source->currentItem()    );
fprintf(out, "%s = %g\n", savestrings[SCUT_ANG4],           slicewin->cuts[4]->pangle->text().toFloat() );
fprintf(out, "%s = %d\n", savestrings[SCUT_STYLE4],         slicewin->cuts[4]->linesty->currentItem()   );
fprintf(out, "%s = %d\n", savestrings[SCUT_COL4],           slicewin->cuts[4]->colour->currentItem()    );
fprintf(out, "%s = %d\n", savestrings[SCUT_THICK4],         slicewin->cuts[4]->thick->currentItem()    );

fprintf(out, "%s = %d\n", savestrings[SCUT_OPT5],           slicewin->cuts[5]->source->currentItem()    );
fprintf(out, "%s = %g\n", savestrings[SCUT_ANG5],           slicewin->cuts[5]->pangle->text().toFloat() );
fprintf(out, "%s = %d\n", savestrings[SCUT_STYLE5],         slicewin->cuts[5]->linesty->currentItem()   );
fprintf(out, "%s = %d\n", savestrings[SCUT_COL5],           slicewin->cuts[5]->colour->currentItem()    );
fprintf(out, "%s = %d\n", savestrings[SCUT_THICK5],         slicewin->cuts[5]->thick->currentItem()    );

fprintf(out, "%s = %d\n", savestrings[SSLICE_E_LINLOG],     slicewin->energy_linlog->currentItem()      );
fprintf(out, "%s = %d\n", savestrings[SSLICE_E_AUTO],       slicewin->energy_scale->currentItem()       );
fprintf(out, "%s = %g\n", savestrings[SSLICE_EMIN],         slicewin->emin->text().toFloat()            );
fprintf(out, "%s = %g\n", savestrings[SSLICE_EMAX],         slicewin->emax->text().toFloat()            );
fprintf(out, "%s = %d\n", savestrings[SSLICE_PSD_LINLOG],   slicewin->psd_linlog->currentItem()         );
fprintf(out, "%s = %d\n", savestrings[SSLICE_PSD_AUTO],     slicewin->psd_scale->currentItem()          );
fprintf(out, "%s = %g\n", savestrings[SSLICE_PSDMIN],       slicewin->psdmin->text().toFloat()          );
fprintf(out, "%s = %g\n", savestrings[SSLICE_PSDMAX],       slicewin->psdmax->text().toFloat()          );

fprintf(out, "%s = %d\n", savestrings[SSLICE_PLOT_COL],     slicewin->plot_col->currentItem()           );
fprintf(out, "%s = %d\n", savestrings[SSLICE_PLOT_DEV],     slicewin->plot_dev->currentItem()           );
fprintf(out, "%s = %d\n", savestrings[SSLICE_XWIN_NO],      slicewin->plot_win_edit->text().toInt()     );
fprintf(out, "%s = %s\n", savestrings[SSLICE_PLOT_STEM],    slicewin->plot_file_edit->text().latin1()   );

fclose(out);
}


void PlotAzBox::slotRestore(void){

char inputline[MAXLENGTH_SAVE_LINE];

int i;
FILE *in;
QString infile = QFileDialog::getOpenFileName();
in = fopen(infile.latin1(),"r");
if (in == NULL){
   printf("Error opening %s for restore\n",infile.latin1());
   return;
}

QString qin, qparam, qvalue;

while (fgets (inputline, MAXLENGTH_SAVE_LINE, in) != NULL){
    
    qin    = inputline;
    qparam = (qin.section("=",0,0)).stripWhiteSpace();
    qvalue = (qin.section("=",-1,-1)).stripWhiteSpace();

    // printf("qparam = |%s| and qvalue = |%s|\n",qparam.latin1(),qvalue.latin1());

    for (i=0; i<SDUMMYSTRING; i++){
        if(qparam ==savestrings[i])break;
    }
    if (i< SDUMMYSTRING){
       switch(i){
       
         case SPROJECT:       lineage->proj->setCurrentItem(qvalue.toInt());               break;	 
	 case SMISSION:       lineage->miss->setCurrentItem(qvalue.toInt());               break;	 
	 case SEXPERIMENT:    lineage->exp->setCurrentItem(qvalue.toInt());                break;
	 case SINSTRUMENT:    lineage->inst->setCurrentItem(qvalue.toInt());               break;
	 case SVIRTUAL:       lineage->virins->setCurrentItem(qvalue.toInt());             break;

	 case SPROJECT1:      prim_sweep->project->setText(qvalue);                        break;
	 case SMISSION1:      prim_sweep->mission->setText(qvalue);                        break;
	 case SEXPERIMENT1:   prim_sweep->exp->setText(qvalue);                            break;
	 case SINSTRUMENT1:   prim_sweep->inst->setText(qvalue);                           break;
	 case SVIRTUAL1:      prim_sweep->virins->setText(qvalue);                         break;
	 case SPA_OPTS1:      prim_sweep->pachoice->setCurrentItem(qvalue.toInt());        break;
	 case SN_PA1:         prim_sweep->npa->setText(qvalue);                            break;
	 case SGATHER_OPTS1:  prim_sweep->gatherchoice->setCurrentItem(qvalue.toInt());    break;
	 case SN_GATHER1:     prim_sweep->gathern->setText(qvalue);                        break;
	 case SSC_POT_OPTS1:  prim_sweep->sc_pot->setCurrentItem(qvalue.toInt());          break;
	 case SSC_POT_VAL1:   prim_sweep->sc_pot_value->setText(qvalue);                   break;
	 case SSC_POT_SECS1:  prim_sweep->sc_pot_gap->setText(qvalue);                     break;

	 case SYR1:           prim_sweep->timebox->YearspinBox->setValue(atoi(qvalue));    break;
	 case SDOY1:          prim_sweep->timebox->DoyspinBox->setValue(atoi(qvalue));     break;
	 case SHR1:           prim_sweep->timebox->setSHour(atoi(qvalue));                 break;
	 case SMIN1:          prim_sweep->timebox->setSMinute(atoi(qvalue));               break;
	 case SSEC1:          prim_sweep->timebox->setSSecond(atoi(qvalue));               break;
	 case SAZ1:           prim_sweep->azslice->setText(qvalue);                        break;
	 case SONECNTCH1:     prim_sweep->onecountchoice->setCurrentItem(qvalue.toInt());  break;
	 case SONECNT_MULT1:  prim_sweep->onecount_mult->setText(qvalue);                  break;

	 case SPROJECT2:      sec_sweep->project->setText(qvalue);                         break;
	 case SMISSION2:      sec_sweep->mission->setText(qvalue);                         break;
	 case SEXPERIMENT2:   sec_sweep->exp->setText(qvalue);                             break;
	 case SINSTRUMENT2:   sec_sweep->inst->setText(qvalue);                            break;
	 case SVIRTUAL2:      sec_sweep->virins->setText(qvalue);                          break;
	 case SPA_OPTS2:      sec_sweep->pachoice->setCurrentItem(qvalue.toInt());         break;
	 case SN_PA2:         sec_sweep->npa->setText(qvalue);                             break;
	 case SGATHER_OPTS2:  sec_sweep->gatherchoice->setCurrentItem(qvalue.toInt());     break;
	 case SN_GATHER2:     sec_sweep->gathern->setText(qvalue);                         break;
	 case SSC_POT_OPTS2:  sec_sweep->sc_pot->setCurrentItem(qvalue.toInt());           break;
	 case SSC_POT_VAL2:   sec_sweep->sc_pot_value->setText(qvalue);                    break;
	 case SSC_POT_SECS2:  sec_sweep->sc_pot_gap->setText(qvalue);                      break;

	 case SYR2:           sec_sweep->timebox->YearspinBox->setValue(atoi(qvalue));     break;
	 case SDOY2:          sec_sweep->timebox->DoyspinBox->setValue(atoi(qvalue));      break;
	 case SHR2:           sec_sweep->timebox->setSHour(atoi(qvalue));                  break;
	 case SMIN2:          sec_sweep->timebox->setSMinute(atoi(qvalue));                break;
	 case SSEC2:          sec_sweep->timebox->setSSecond(atoi(qvalue));                break;
	 case SAZ2:           sec_sweep->azslice->setText(qvalue);                         break;
	 case SONECNTCH2:     sec_sweep->onecountchoice->setCurrentItem(qvalue.toInt());   break;
	 case SONECNT_MULT2:  sec_sweep->onecount_mult->setText(qvalue);                   break;

	 case SENERGY_UNITS:     energy_units->setCurrentItem(qvalue.toInt());             break;
	 case SENERGY_LINLOG:    energy_linlog->setCurrentItem(qvalue.toInt());            break;
	 case SENERGY_AUTOSCALE: energy_scale->setCurrentItem(qvalue.toInt());             break;
	 case SEMIN:          emin->setText(qvalue);                                       break;
	 case SEMAX:          emax->setText(qvalue);                                       break;
	 case SPSD_UNITS:     psd_units->setCurrentItem(qvalue.toInt());                   break;
	 case SPSD_LINLOG:    psd_linlog->setCurrentItem(qvalue.toInt());                  break; 
	 case SPSD_SCALE:     psd_scale->setCurrentItem(qvalue.toInt());                   break;
	 case SPSD_MIN:       psdmin->setText(qvalue);                                     break;
	 case SPSD_MAX:       psdmax->setText(qvalue);                                     break;
	 case SANALYSIS_OPT:  anal_proc->setCurrentItem(qvalue.toInt());                   break;
	 case SANALYSIS_LINLOG:    anal_linlog->setCurrentItem(qvalue.toInt());            break;
	 case SANALYSIS_AUTOSCALE: anal_scale->setCurrentItem(qvalue.toInt());             break;
	 case SANALYSIS_MIN:  analmin->setText(qvalue);                                    break;
	 case SANALYSIS_MAX:  analmax->setText(qvalue);                                    break;
	 case SPLOT_COL:      plot_col->setCurrentItem(qvalue.toInt());                    break;
	 case SPLOT_DEV:      plot_dev->setCurrentItem(qvalue.toInt());                    break;
	 case SXWIN_NO:       plot_win_edit->setText(qvalue);                              break;
	 case SPLOT_STEM:     plot_file_edit->setText(qvalue);                             break;  
   /* slicewin */
         case SVPARA1:        slicewin->v1->setText(qvalue);                               break;
	 case SVPARA2:        slicewin->v2->setText(qvalue);                               break;
	 case SB1:            slicewin->b1->setText(qvalue);                               break;
	 case SB2:            slicewin->b2->setText(qvalue);                               break;
	 case SDELPHI:        slicewin->pot->setText(qvalue);                              break;
	 case SLEFT_WHEEL:    slicewin->lwheel->setCurrentItem(qvalue.toInt());            break;
	 case SRIGHT_WHEEL:   slicewin->rwheel->setCurrentItem(qvalue.toInt());            break;

	 case SBOX_THICK:     slicewin->box_thick->setCurrentItem(qvalue.toInt());         break;
	 
	 case SCUT_OPT0:      slicewin->cuts[0]->source->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_ANG0:      slicewin->cuts[0]->pangle->setText(qvalue);                  break;
	 case SCUT_STYLE0:    slicewin->cuts[0]->linesty->setCurrentItem(qvalue.toInt());  break;
	 case SCUT_COL0:      slicewin->cuts[0]->colour->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_THICK0:    slicewin->cuts[0]->thick->setCurrentItem(qvalue.toInt());    break;

	 case SCUT_OPT1:      slicewin->cuts[1]->source->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_ANG1:      slicewin->cuts[1]->pangle->setText(qvalue);                  break;
	 case SCUT_STYLE1:    slicewin->cuts[1]->linesty->setCurrentItem(qvalue.toInt());  break;
	 case SCUT_COL1:      slicewin->cuts[1]->colour->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_THICK1:    slicewin->cuts[1]->thick->setCurrentItem(qvalue.toInt());    break;

	 case SCUT_OPT2:      slicewin->cuts[2]->source->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_ANG2:      slicewin->cuts[2]->pangle->setText(qvalue);                  break;
	 case SCUT_STYLE2:    slicewin->cuts[2]->linesty->setCurrentItem(qvalue.toInt());  break;
	 case SCUT_COL2:      slicewin->cuts[2]->colour->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_THICK2:    slicewin->cuts[2]->thick->setCurrentItem(qvalue.toInt());    break;
	 
	 case SCUT_OPT3:      slicewin->cuts[3]->source->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_ANG3:      slicewin->cuts[3]->pangle->setText(qvalue);                  break;
	 case SCUT_STYLE3:    slicewin->cuts[3]->linesty->setCurrentItem(qvalue.toInt());  break;
	 case SCUT_COL3:      slicewin->cuts[3]->colour->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_THICK3:    slicewin->cuts[3]->thick->setCurrentItem(qvalue.toInt());    break;
	 
	 case SCUT_OPT4:      slicewin->cuts[4]->source->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_ANG4:      slicewin->cuts[4]->pangle->setText(qvalue);                  break;
	 case SCUT_STYLE4:    slicewin->cuts[4]->linesty->setCurrentItem(qvalue.toInt());  break;
	 case SCUT_COL4:      slicewin->cuts[4]->colour->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_THICK4:    slicewin->cuts[4]->thick->setCurrentItem(qvalue.toInt());    break;
	 
	 case SCUT_OPT5:      slicewin->cuts[5]->source->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_ANG5:      slicewin->cuts[5]->pangle->setText(qvalue);                  break;
	 case SCUT_STYLE5:    slicewin->cuts[5]->linesty->setCurrentItem(qvalue.toInt());  break;
	 case SCUT_COL5:      slicewin->cuts[5]->colour->setCurrentItem(qvalue.toInt());   break;
	 case SCUT_THICK5:    slicewin->cuts[5]->thick->setCurrentItem(qvalue.toInt());    break;
	 
	 case SSLICE_E_LINLOG:  slicewin->energy_linlog->setCurrentItem(qvalue.toInt());   break;
	 case SSLICE_E_AUTO:    slicewin->energy_scale->setCurrentItem(qvalue.toInt());    break;
	 case SSLICE_EMIN:      slicewin->emin->setText(qvalue);                           break;
	 case SSLICE_EMAX:      slicewin->emax->setText(qvalue);                           break;
	 case SSLICE_PSD_LINLOG:slicewin->psd_linlog->setCurrentItem(qvalue.toInt());      break;
	 case SSLICE_PSD_AUTO:  slicewin->psd_scale->setCurrentItem(qvalue.toInt());       break;
	 case SSLICE_PSDMIN:    slicewin->psdmin->setText(qvalue);                         break;
	 case SSLICE_PSDMAX:    slicewin->psdmax->setText(qvalue);                         break;
	 case SSLICE_PLOT_COL:  slicewin->plot_col->setCurrentItem(qvalue.toInt());        break;
	 case SSLICE_PLOT_DEV:  slicewin->plot_dev->setCurrentItem(qvalue.toInt());        break;
	 case SSLICE_XWIN_NO:   slicewin->plot_win_edit->setText(qvalue);                  break;
	 case SSLICE_PLOT_STEM: slicewin->plot_file_edit->setText(qvalue);                 break;  	 
	 default: break;
       }
    }

}

}
